@@ -1,8 +1,8 @@
-/* @(#)cdrecord.h	1.201 10/01/31 Copyright 1995-2010 J. Schilling */
+/* @(#)cdrecord.h	1.205 13/05/28 Copyright 1995-2013 J. Schilling */
 /*
  *	Definitions for cdrecord
  *
- *	Copyright (c) 1995-2010 J. Schilling
+ *	Copyright (c) 1995-2013 J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -1214,3 +1214,24 @@ extern	int	parsecue	__PR((char *cuefname, track_t trackp[]));
 #ifdef	EOF	/* stdio.h has been included */
 extern	void	fparsecue	__PR((FILE *f, track_t trackp[]));
 #endif
+
+/*
+ * vendor.c
+ */
+
+/*
+ * priv.c
+ */
+#ifdef	CDDA2WAV
+extern	void	priv_init	__PR((void));
+extern	void	priv_on		__PR((void));
+extern	void	priv_off	__PR((void));
+#endif
+#if	defined(CDRECORD) || defined(READCD)
+extern	void	priv_drop	__PR((void));
+extern	BOOL	priv_from_priv	__PR((void));
+#endif
+extern	BOOL	priv_eff_priv	__PR((int pname));
+#ifdef	HAVE_SOLARIS_PPRIV
+extern	void	do_pfexec	__PR((int ac, char *av[], ...));
+#endif
