@@ -1,15 +1,15 @@
-/* @(#)drv_mmc.c	1.198 10/05/11 Copyright 1997-2010 J. Schilling */
+/* @(#)drv_mmc.c	1.200 12/03/16 Copyright 1997-2012 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)drv_mmc.c	1.198 10/05/11 Copyright 1997-2010 J. Schilling";
+	"@(#)drv_mmc.c	1.200 12/03/16 Copyright 1997-2012 J. Schilling";
 #endif
 /*
  *	CDR device implementation for
  *	SCSI-3/mmc conforming drives
  *	e.g. Yamaha CDR-400, Ricoh MP6200
  *
- *	Copyright (c) 1997-2010 J. Schilling
+ *	Copyright (c) 1997-2012 J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -40,6 +40,7 @@ static	UConst char sccsid[] =
 #include <schily/btorder.h>
 #include <schily/intcvt.h>
 #include <schily/schily.h>
+#include <schily/nlsdefs.h>
 
 #include <scg/scgcmd.h>
 #include <scg/scsidefs.h>
@@ -404,7 +405,7 @@ mmc_opthelp(scgp, dp, excode)
 {
 	BOOL	haveopts = FALSE;
 
-	error("Driver options:\n");
+	error(_("Driver options:\n"));
 	if (dp->cdr_flags & CDR_BURNFREE) {
 		error("burnfree	Prepare writer to use BURN-Free technology\n");
 		error("noburnfree	Disable using BURN-Free technology\n");
@@ -579,8 +580,8 @@ identify_mmc(scgp, dp, ip)
 	/*
 	 * First handle exceptions....
 	 */
-	if (strncmp(ip->vendor_info, "SONY", 4) == 0 &&
-	    strncmp(ip->prod_ident, "CD-R   CDU928E", 14) == 0) {
+	if (strncmp(ip->inq_vendor_info, "SONY", 4) == 0 &&
+	    strncmp(ip->inq_prod_ident, "CD-R   CDU928E", 14) == 0) {
 		return (&cdr_mmc_sony);
 	}
 
@@ -2860,7 +2861,7 @@ mmc_isplextor(scgp)
 	SCSI	*scgp;
 {
 	if (scgp->inq != NULL &&
-			strncmp(scgp->inq->vendor_info, "PLEXTOR", 7) == 0) {
+			strncmp(scgp->inq->inq_vendor_info, "PLEXTOR", 7) == 0) {
 		return (TRUE);
 	}
 	return (FALSE);
@@ -2871,7 +2872,7 @@ mmc_isyamaha(scgp)
 	SCSI	*scgp;
 {
 	if (scgp->inq != NULL &&
-			strncmp(scgp->inq->vendor_info, "YAMAHA", 6) == 0) {
+			strncmp(scgp->inq->inq_vendor_info, "YAMAHA", 6) == 0) {
 		return (TRUE);
 	}
 	return (FALSE);
