@@ -1,38 +1,36 @@
-/* @(#)wm_packet.c	1.25 04/03/01 Copyright 1995, 1997, 2001-2004 J. Schilling */
+/* @(#)wm_packet.c	1.31 10/12/19 Copyright 1995, 1997, 2001-2010 J. Schilling */
+#include <schily/mconfig.h>
 #ifndef lint
-static	char sccsid[] =
-	"@(#)wm_packet.c	1.25 04/03/01 Copyright 1995, 1997, 2001-2004 J. Schilling";
+static	UConst char sccsid[] =
+	"@(#)wm_packet.c	1.31 10/12/19 Copyright 1995, 1997, 2001-2010 J. Schilling";
 #endif
 /*
  *	CDR write method abtraction layer
  *	packet writing intercace routines
  *
- *	Copyright (c) 1995, 1997, 2001-2004 J. Schilling
+ *	Copyright (c) 1995, 1997, 2001-2010 J. Schilling
  */
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * The contents of this file are subject to the terms of the
+ * Common Development and Distribution License, Version 1.0 only
+ * (the "License").  You may not use this file except in compliance
+ * with the License.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * See the file CDDL.Schily.txt in this distribution for details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; see the file COPYING.  If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * When distributing Covered Code, include this CDDL HEADER in each
+ * file and include the License file CDDL.Schily.txt from this distribution.
  */
 
-#include <mconfig.h>
-#include <stdio.h>
-#include <stdxlib.h>
-#include <unixstd.h>
-#include <timedefs.h>
-#include <standard.h>
-#include <utypes.h>
-#include <schily.h>
+#include <schily/mconfig.h>
+#include <schily/stdio.h>
+#include <schily/stdlib.h>
+#include <schily/unistd.h>
+#include <schily/time.h>
+#include <schily/standard.h>
+#include <schily/utypes.h>
+#include <schily/schily.h>
+#include <schily/nlsdefs.h>
 
 #include <scg/scsitransp.h>
 #include "cdrecord.h"
@@ -64,11 +62,11 @@ write_packet_data(scgp, dp, trackp)
 	int	secsize;
 	int	secspt;
 	int	bytespt;
+	int	bytes_to_read;
 	long	amount;
 	int	pad;
 	int	retried;
 	long	nextblock;
-	int	bytes_to_read;
 	BOOL	neednl	= FALSE;
 	BOOL	islast	= FALSE;
 	char	*bp	= buf;
@@ -79,7 +77,7 @@ long bsize;
 long bfree;
 #define	BCAP
 #ifdef	BCAP
-int per;
+int per = 0;
 #ifdef	XBCAP
 int oper = -1;
 #endif
@@ -87,6 +85,8 @@ int oper = -1;
 
 	if (dp->cdr_dstat->ds_flags & DSF_DVD)
 		secsps = 676.27;
+	if (dp->cdr_dstat->ds_flags & DSF_BD)
+		secsps = 2195.07;
 
 	scgp->silent++;
 	if ((*dp->cdr_buffer_cap)(scgp, &bsize, &bfree) < 0)
@@ -111,16 +111,16 @@ int oper = -1;
 	pad = !isaudio && is_pad(trackp);	/* Pad only data tracks */
 
 	if (debug) {
-		printf("secsize:%d secspt:%d bytespt:%d audio:%d pad:%d\n",
+		printf(_("secsize:%d secspt:%d bytespt:%d audio:%d pad:%d\n"),
 			secsize, secspt, bytespt, isaudio, pad);
 	}
 
 	if (lverbose) {
 		if (tracksize > 0)
-			printf("\rTrack %02d:    0 of %4lld MB written.",
+			printf(_("\rTrack %02d:    0 of %4lld MB written."),
 				track, tracksize >> 20);
 		else
-			printf("\rTrack %02d:    0 MB written.", track);
+			printf(_("\rTrack %02d:    0 MB written."), track);
 		flush();
 		neednl = TRUE;
 	}
@@ -137,7 +137,7 @@ int oper = -1;
 					/* XXX next wr addr ??? */
 		count = get_buf(f, trackp, startsec, &bp, bytes_to_read);
 		if (count < 0)
-			comerr("read error on input file\n");
+			comerr(_("read error on input file\n"));
 		if (count == 0)
 			break;
 		bytes_read += count;
@@ -149,7 +149,7 @@ int oper = -1;
 
 		if (count < bytespt) {
 			if (debug) {
-				printf("\nNOTICE: reducing block size for last record.\n");
+				printf(_("\nNOTICE: reducing block size for last record.\n"));
 				neednl = FALSE;
 			}
 
@@ -157,14 +157,14 @@ int oper = -1;
 				amount = secsize - amount;
 				fillbytes(&bp[count], amount, '\0');
 				count += amount;
-				printf("\nWARNING: padding up to secsize.\n");
+				printf(_("\nWARNING: padding up to secsize.\n"));
 				neednl = FALSE;
 			}
 			if (is_packet(trackp) && trackp->pktsize > 0) {
 				if (count < bytespt) {
 					amount = bytespt - count;
 					count += amount;
-					printf("\nWARNING: padding remainder of packet.\n");
+					printf(_("\nWARNING: padding remainder of packet.\n"));
 					neednl = FALSE;
 				}
 			}
@@ -196,16 +196,16 @@ int oper = -1;
 			}
 		}
 
-		amount =  write_secs(scgp, dp, bp, startsec, bytespt, secspt, islast);
+		amount = write_secs(scgp, dp, bp, startsec, bytespt, secspt, islast);
 		if (amount < 0) {
 			if (is_packet(trackp) && trackp->pktsize == 0 && !retried) {
-				printf("%swrite track data: error after %lld bytes, retry with new packet\n",
+				printf(_("%swrite track data: error after %lld bytes, retry with new packet\n"),
 					neednl?"\n":"", bytes);
 				retried = 1;
 				neednl = FALSE;
 				goto retry;
 			}
-			printf("%swrite track data: error after %lld bytes\n",
+			printf(_("%swrite track data: error after %lld bytes\n"),
 							neednl?"\n":"", bytes);
 			return (-1);
 		}
@@ -218,12 +218,12 @@ int oper = -1;
 			float	fspeed;
 
 			gettimeofday(&tcur, (struct timezone *)0);
-			printf("\rTrack %02d: %4lld", track, bytes >> 20);
+			printf(_("\rTrack %02d: %4lld"), track, bytes >> 20);
 			if (tracksize > 0)
-				printf(" of %4lld MB", tracksize >> 20);
+				printf(_(" of %4lld MB"), tracksize >> 20);
 			else
 				printf(" MB");
-			printf(" written");
+			printf(_(" written"));
 			fper = fifo_percent(TRUE);
 			if (fper >= 0)
 				printf(" (fifo %3d%%)", fper);
@@ -287,8 +287,8 @@ int oper = -1;
 		if ((padbytes >> 20) > 0) {
 			neednl = TRUE;
 		} else if (lverbose) {
-			printf("Track %02d: writing %3lld KB of pad data.\n",
-						track, (Llong)(padbytes >> 10));
+			printf(_("Track %02d: writing %3lld KB of pad data.\n"),
+					track, (Llong)(padbytes >> 10));
 			neednl = FALSE;
 		}
 		pad_track(scgp, dp, trackp, startsec, padbytes,
@@ -296,7 +296,7 @@ int oper = -1;
 		bytes += savbytes;
 		startsec += savbytes / secsize;
 	}
-	printf("%sTrack %02d: Total bytes read/written: %lld/%lld (%lld sectors).\n",
+	printf(_("%sTrack %02d: Total bytes read/written: %lld/%lld (%lld sectors).\n"),
 		neednl?"\n":"", track, bytes_read, bytes, bytes/secsize);
 	return (0);
 }
