@@ -1,14 +1,14 @@
 /*#define	PLUS_DEBUG*/
-/* @(#)find.c	1.90 10/04/27 Copyright 2004-2010 J. Schilling */
+/* @(#)find.c	1.100 15/08/26 Copyright 2004-2015 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)find.c	1.90 10/04/27 Copyright 2004-2010 J. Schilling";
+	"@(#)find.c	1.100 15/08/26 Copyright 2004-2015 J. Schilling";
 #endif
 /*
  *	Another find implementation...
  *
- *	Copyright (c) 2004-2010 J. Schilling
+ *	Copyright (c) 2004-2015 J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -17,6 +17,8 @@ static	UConst char sccsid[] =
  * with the License.
  *
  * See the file CDDL.Schily.txt in this distribution for details.
+ * A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  *
  * When distributing Covered Code, include this CDDL HEADER in each
  * file and include the License file CDDL.Schily.txt from this distribution.
@@ -169,6 +171,8 @@ LOCAL	findn_t	*parseprim	__PR((finda_t *fap));
 EXPORT	void	find_firstprim	__PR((int *pac, char *const **pav));
 EXPORT	BOOL	find_primary	__PR((findn_t *t, int op));
 EXPORT	BOOL	find_pname	__PR((findn_t *t, char *word));
+EXPORT	BOOL	find_hasprint	__PR((findn_t *t));
+EXPORT	BOOL	find_hasexec	__PR((findn_t *t));
 #ifdef	FIND_MAIN
 LOCAL	int	walkfunc	__PR((char *nm, struct stat *fs, int type, struct WALK *state));
 #endif
@@ -177,7 +181,9 @@ LOCAL	inline BOOL find_expr	__PR((char *f, char *ff, struct stat *fs, struct WAL
 #else
 EXPORT	BOOL	find_expr	__PR((char *f, char *ff, struct stat *fs, struct WALK *state, findn_t *t));
 #endif
+#ifdef	HAVE_FORK
 LOCAL	BOOL	doexec		__PR((char *f, findn_t *t, int ac, char **av, struct WALK *state));
+#endif
 LOCAL	int	countenv	__PR((void));
 LOCAL	int	argsize		__PR((int xtype));
 LOCAL	int	extype		__PR((char *name));
@@ -185,7 +191,9 @@ LOCAL	int	extype		__PR((char *name));
 LOCAL	int	xargsize	__PR((int xtype, int maxarg));
 #endif
 LOCAL	BOOL	pluscreate	__PR((FILE *f, int ac, char **av, finda_t *fap));
+#ifdef	HAVE_FORK
 LOCAL	BOOL	plusexec	__PR((char *f, findn_t *t, struct WALK *state));
+#endif
 EXPORT	int	find_plusflush	__PR((void *p, struct WALK *state));
 EXPORT	void	find_usage	__PR((FILE *f));
 #ifdef	FIND_MAIN
@@ -840,9 +848,14 @@ parseprim(fap)
 		n->this = (char *)nextarg(fap, n);
 		switch (*(n->this)) {
 
+		/*
+		 * 'b'lock, 'c'har, 'd'ir, 'D'oor,
+		 * 'e'ventcount, 'f'ile, 'l'ink, 'p'ipe,
+		 * 'P'ort event, 's'ocket
+		 */
 		case 'b': case 'c': case 'd': case 'D':
 		case 'e': case 'f': case 'l': case 'p':
-		case 's':
+		case 'P': case 's':
 			if ((n->this)[1] == '\0') {
 				nexttoken(fap);
 				fap->jmp = ojmp; /* Restore old jump target */
@@ -1246,6 +1259,37 @@ find_pname(t, word)
 	return (find_primary(t, find_token(word)));
 }
 
+EXPORT BOOL
+find_hasprint(t)
+	findn_t	*t;
+{
+	if (t == NULL)
+		return (FALSE);
+
+	if (find_primary(t, PRINT) || find_primary(t, PRINTNNL) ||
+	    find_primary(t, PRINT0))
+		return (TRUE);
+	if (find_primary(t, LS))
+		return (TRUE);
+	return (FALSE);
+}
+
+EXPORT BOOL
+find_hasexec(t)
+	findn_t	*t;
+{
+	if (t == NULL)
+		return (FALSE);
+
+	if (find_primary(t, EXEC) || find_primary(t, EXECPLUS))
+		return (TRUE);
+	if (find_primary(t, EXECDIR) || find_primary(t, EXECDIRPLUS))
+		return (TRUE);
+	if (find_primary(t, OK_EXEC) || find_primary(t, OK_EXECDIR))
+		return (TRUE);
+	return (FALSE);
+}
+
 #ifdef	FIND_MAIN
 LOCAL int
 walkfunc(nm, fs, type, state)
@@ -1440,6 +1484,7 @@ find_expr(f, ff, fs, state, t)
 				closedir(d);
 				return (FALSE);
 			}
+			closedir(d);
 			return (TRUE);
 		}
 		return (FALSE);
@@ -1694,13 +1739,29 @@ find_expr(f, ff, fs, state, t)
 
 	case EXEC:
 	case EXECDIR:
+#ifdef	HAVE_FORK
 		return (doexec(
 			state->level && (t->op == OK_EXECDIR || t->op == EXECDIR)?
 			ff:f,
 			t, t->val.i, (char **)t->this, state));
+#else
+		ferrmsgno(state->std[2], EX_BAD,
+		gettext(
+		"'-%s' is unsupported on this platform, returning FALSE.\n"),
+				find_tname(t->op));
+		return (FALSE);
+#endif
 
 	case EXECPLUS:
+#ifdef	HAVE_FORK
 		return (plusexec(f, t, state));
+#else
+		ferrmsgno(state->std[2], EX_BAD,
+		gettext(
+		"'-%s' is unsupported on this platform, returning FALSE.\n"),
+				find_tname(t->op));
+		return (FALSE);
+#endif
 
 	case FPRINT:
 		fp = t->val.fp;
@@ -1772,6 +1833,7 @@ find_expr(f, ff, fs, state, t)
 	return (FALSE);		/* Unknown operator ??? */
 }
 
+#ifdef	HAVE_FORK
 LOCAL BOOL
 doexec(f, t, ac, av, state)
 	char	*f;
@@ -1781,7 +1843,7 @@ doexec(f, t, ac, av, state)
 	struct WALK *state;
 {
 #ifdef	HAVE_VFORK
-	char	**aav = NULL;
+	char	** volatile aav = NULL;
 #endif
 	pid_t	pid;
 	int	retval;
@@ -1802,7 +1864,17 @@ doexec(f, t, ac, av, state)
 		ferrmsg(state->std[2], gettext("Cannot fork child.\n"));
 #endif
 #ifdef	HAVE_VFORK
-		if (aav)
+		/*
+		 * Ugly code as a workaround for broken Linux include files
+		 * that do not specify vfork() as a problem. This may
+		 * cause aav to be != NULL even when malloc() above was never
+		 * called. Freeing a random address on some platforms causes a
+		 * coredump.
+		 * As similar problems may exist on other platforms, where the
+		 * correct fix to mark aav volatile does not work, we keep the
+		 * workaround to check f and ac as well.
+		 */
+		if (aav && f && ac >= 32)
 			free(aav);
 #endif
 		return (FALSE);
@@ -1812,7 +1884,17 @@ doexec(f, t, ac, av, state)
 			/* LINTED */
 			;
 #ifdef	HAVE_VFORK
-		if (aav)
+		/*
+		 * Ugly code as a workaround for broken Linux include files
+		 * that do not specify vfork() as a problem. This may
+		 * cause aav to be != NULL even when malloc() above was never
+		 * called. Freeing a random address on some platforms causes a
+		 * coredump.
+		 * As similar problems may exist on other platforms, where the
+		 * correct fix to mark aav volatile does not work, we keep the
+		 * workaround to check f and ac as well.
+		 */
+		if (aav && f && ac >= 32)
 			free(aav);
 #endif
 		return (retval == 0);
@@ -1897,6 +1979,7 @@ doexec(f, t, ac, av, state)
 		return (-1);
 	}
 }
+#endif	/* HAVE FORK */
 
 #ifndef	LINE_MAX
 #define	LINE_MAX	1024
@@ -2009,7 +2092,7 @@ extype(name)
 	char	*xname;
 	char	elfbuf[8];
 
-	xname = findinpath(name, X_OK, TRUE);
+	xname = findinpath(name, X_OK, TRUE, NULL);
 	if (name == NULL)
 		xname = name;
 
@@ -2168,6 +2251,7 @@ pluscreate(f, ac, av, fap)
 	return (TRUE);
 }
 
+#ifdef	HAVE_FORK
 LOCAL BOOL
 plusexec(f, t, state)
 	char	*f;
@@ -2221,6 +2305,7 @@ plusexec(f, t, state)
 #endif
 	return (ret);
 }
+#endif	/* HAVE_FORK */
 
 EXPORT int
 find_plusflush(p, state)
@@ -2239,8 +2324,10 @@ find_plusflush(p, state)
 #endif
 		if (plusp->laststr != plusp->endp) {
 			plusp->nextargp[0] = NULL;
+#ifdef	HAVE_FORK
 			if (!doexec(NULL, NULL, plusp->ac, plusp->av, state))
 				ret = FALSE;
+#endif
 		}
 		plusp = plusp->next;
 	}
@@ -2341,8 +2428,6 @@ getflg(optstr, argp)
 	char	*optstr;
 	long	*argp;
 {
-/*	error("optstr: '%s'\n", optstr);*/
-
 	if (optstr[1] != '\0')
 		return (-1);
 
@@ -2369,9 +2454,9 @@ main(ac, av)
 	char	**av;
 {
 	int	cac  = ac;
-	char *	*cav = av;
-	char *	*firstpath;
-	char *	*firstprim;
+	char	**cav = av;
+	char	**firstpath;
+	char	**firstprim;
 	BOOL	help = FALSE;
 	BOOL	prversion = FALSE;
 	finda_t	fa;
