@@ -1,8 +1,8 @@
 #define	USE_REMOTE
-/* @(#)scsi-remote.c	1.13 03/05/16 Copyright 1990,2000-2003 J. Schilling */
+/* @(#)scsi-remote.c	1.34 13/04/21 Copyright 1990,2000-2013 J. Schilling */
 #ifndef lint
 static	char __sccsid[] =
-	"@(#)scsi-remote.c	1.13 03/05/16 Copyright 1990,2000-2003 J. Schilling";
+	"@(#)scsi-remote.c	1.34 13/04/21 Copyright 1990,2000-2013 J. Schilling";
 #endif
 /*
  *	Remote SCSI user level command transport routines
@@ -13,25 +13,29 @@ static	char __sccsid[] =
  *	Choose your name instead of "schily" and make clear that the version
  *	string is related to a modified source.
  *
- *	Copyright (c) 1990,2000-2003 J. Schilling
+ *	Copyright (c) 1990,2000-2013 J. Schilling
  */
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * The contents of this file are subject to the terms of the
+ * Common Development and Distribution License, Version 1.0 only
+ * (the "License").  You may not use this file except in compliance
+ * with the License.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * See the file CDDL.Schily.txt in this distribution for details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; see the file COPYING.  If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * The following exceptions apply:
+ * CDDL 3.6 needs to be replaced by: "You may create a Larger Work by
+ * combining Covered Software with other code if all other code is governed by
+ * the terms of a license that is OSI approved (see www.opensource.org) and
+ * you may distribute the Larger Work as a single product. In such a case,
+ * You must make sure the requirements of this License are fulfilled for
+ * the Covered Software."
+ *
+ * When distributing Covered Code, include this CDDL HEADER in each
+ * file and include the License file CDDL.Schily.txt from this distribution.
  */
 
-#include <mconfig.h>
+#include <schily/mconfig.h>
 
 #if !defined(HAVE_FORK) || !defined(HAVE_SOCKETPAIR) || !defined(HAVE_DUP2)
 #undef	USE_RCMD_RSH
@@ -48,25 +52,20 @@ static	char __sccsid[] =
 #endif
 
 #ifdef	USE_REMOTE
-#include <stdio.h>
-#include <sys/types.h>
-#include <fctldefs.h>
-#ifdef	HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
-#include <errno.h>
-#include <signal.h>
-#ifdef	HAVE_NETDB_H
-#include <netdb.h>
-#endif
-#ifdef	HAVE_PWD_H
-#include <pwd.h>
-#endif
-#include <standard.h>
-#include <stdxlib.h>
-#include <unixstd.h>
-#include <strdefs.h>
-#include <schily.h>
+#include <schily/stdio.h>
+#include <schily/types.h>
+#include <schily/fcntl.h>
+#include <schily/socket.h>
+#include <schily/errno.h>
+#include <schily/signal.h>
+#include <schily/netdb.h>
+#include <schily/pwd.h>
+#include <schily/standard.h>
+#include <schily/stdlib.h>
+#include <schily/unistd.h>
+#include <schily/string.h>
+#include <schily/schily.h>
+#include <schily/priv.h>
 
 #include <scg/scgcmd.h>
 #include <scg/scsitransp.h>
@@ -79,22 +78,29 @@ static	char __sccsid[] =
  * On Cygwin, there are no privilleged ports.
  * On UNIX, rcmd() uses privilleged port that only work for root.
  */
-#ifdef	IS_CYGWIN
+#if	defined(IS_CYGWIN) || defined(__MINGW32__)
 #define	privport_ok()	(1)
 #else
+#if	defined(HAVE_SOLARIS_PPRIV) || defined(HAVE_LINUX_CAPS)
+#define	privport_ok()	ppriv_ok()
+#else
 #define	privport_ok()	(geteuid() == 0)
 #endif
+#endif
 
 #define	CMD_SIZE	80
 
-#define	MAX_SCG		16	/* Max # of SCSI controllers */
+/*
+ * On Linux we have a max bus number of 1000 + 13
+ */
+#define	MAX_SCG		1024	/* Max # of SCSI controllers */
 #define	MAX_TGT		16
 #define	MAX_LUN		8
 
 /*extern	BOOL	debug;*/
 LOCAL	BOOL	debug = 1;
 
-LOCAL	char	_scg_trans_version[] = "remote-1.13";	/* The version for remote SCSI	*/
+LOCAL	char	_scg_trans_version[] = "remote-1.34";	/* The version for remote SCSI	*/
 LOCAL	char	_scg_auth_schily[]	= "schily";	/* The author for this module	*/
 
 LOCAL	int	scgo_rsend		__PR((SCSI *scgp));
@@ -105,6 +111,7 @@ LOCAL	int	scgo_rclose		__PR((SCSI *scgp));
 LOCAL	long	scgo_rmaxdma		__PR((SCSI *scgp, long amt));
 LOCAL	void *	scgo_rgetbuf		__PR((SCSI *scgp, long amt));
 LOCAL	void	scgo_rfreebuf		__PR((SCSI *scgp));
+LOCAL	int	scgo_rnumbus		__PR((SCSI *scgp));
 LOCAL	BOOL	scgo_rhavebus		__PR((SCSI *scgp, int busno));
 LOCAL	int	scgo_rfileno		__PR((SCSI *scgp, int busno, int tgt, int tlun));
 LOCAL	int	scgo_rinitiator_id	__PR((SCSI *scgp));
@@ -117,13 +124,14 @@ LOCAL	int	scgo_rreset		__PR((SCSI *scgp, int what));
  * XXX via scglocal(scgp)->remfd.
  */
 LOCAL	void	rscsiabrt		__PR((int sig));
-LOCAL	int	rscsigetconn		__PR((SCSI *scgp, char* host));
+LOCAL	int	rscsigetconn		__PR((SCSI *scgp, char *host));
 LOCAL	char	*rscsiversion		__PR((SCSI *scgp, int fd, int what));
-LOCAL	int	rscsiopen		__PR((SCSI *scgp, int fd, char* fname));
+LOCAL	int	rscsiopen		__PR((SCSI *scgp, int fd, char *fname));
 LOCAL	int	rscsiclose		__PR((SCSI *scgp, int fd));
 LOCAL	int	rscsimaxdma		__PR((SCSI *scgp, int fd, long amt));
 LOCAL	int	rscsigetbuf		__PR((SCSI *scgp, int fd, long amt));
 LOCAL	int	rscsifreebuf		__PR((SCSI *scgp, int fd));
+LOCAL	int	rscsinumbus		__PR((SCSI *scgp, int fd));
 LOCAL	int	rscsihavebus		__PR((SCSI *scgp, int fd, int bus));
 LOCAL	int	rscsifileno		__PR((SCSI *scgp, int fd, int busno, int tgt, int tlun));
 LOCAL	int	rscsiinitiator_id	__PR((SCSI *scgp, int fd));
@@ -134,11 +142,11 @@ LOCAL	int	rscsifillrbuf		__PR((SCSI *scgp));
 LOCAL	int	rscsirchar		__PR((SCSI *scgp, char *cp));
 LOCAL	int	rscsireadbuf		__PR((SCSI *scgp, int fd, char *buf, int count));
 LOCAL	void	rscsivoidarg		__PR((SCSI *scgp, int fd, int count));
-LOCAL	int	rscsicmd		__PR((SCSI *scgp, int fd, char* name, char* cbuf));
-LOCAL	void	rscsisendcmd		__PR((SCSI *scgp, int fd, char* name, char* cbuf));
-LOCAL	int	rscsigetline		__PR((SCSI *scgp, int fd, char* line, int count));
+LOCAL	int	rscsicmd		__PR((SCSI *scgp, int fd, char *name, char *cbuf));
+LOCAL	void	rscsisendcmd		__PR((SCSI *scgp, int fd, char *name, char *cbuf));
+LOCAL	int	rscsigetline		__PR((SCSI *scgp, int fd, char *line, int count));
 LOCAL	int	rscsireadnum		__PR((SCSI *scgp, int fd));
-LOCAL	int	rscsigetstatus		__PR((SCSI *scgp, int fd, char* name));
+LOCAL	int	rscsigetstatus		__PR((SCSI *scgp, int fd, char *name));
 LOCAL	int	rscsiaborted		__PR((SCSI *scgp, int fd));
 #ifdef	USE_RCMD_RSH
 LOCAL	int	_rcmdrsh		__PR((char **ahost, int inport,
@@ -146,6 +154,9 @@ LOCAL	int	_rcmdrsh		__PR((char **ahost, int inport,
 						const char *remuser,
 						const char *cmd,
 						const char *rsh));
+#if	defined(HAVE_SOLARIS_PPRIV) || defined(HAVE_LINUX_CAPS)
+LOCAL	BOOL	ppriv_ok		__PR((void));
+#endif
 #endif
 
 /*--------------------------------------------------------------------------*/
@@ -166,7 +177,7 @@ struct scg_local {
 };
 
 
-#define scglocal(p)	((struct scg_local *)((p)->local))
+#define	scglocal(p)	((struct scg_local *)((p)->local))
 
 scg_ops_t remote_ops = {
 	scgo_rsend,		/* "S" end	*/
@@ -177,6 +188,7 @@ scg_ops_t remote_ops = {
 	scgo_rmaxdma,		/* "D" MA	*/
 	scgo_rgetbuf,		/* "M" alloc	*/
 	scgo_rfreebuf,		/* "F" free	*/
+	scgo_rnumbus,		/* "N" um Bus	*/
 	scgo_rhavebus,		/* "B" us	*/
 	scgo_rfileno,		/* "T" arget	*/
 	scgo_rinitiator_id,	/* "I" nitiator	*/
@@ -205,41 +217,40 @@ scgo_rversion(scgp, what)
 {
 	int	f;
 
-	if (scgp->local == NULL)
+	if (scgp == (SCSI *)0 || scgp->local == NULL)
 		return ((char *)0);
 
 	f = scglocal(scgp)->remfd;
-	if (scgp != (SCSI *)0) {
-		switch (what) {
 
-		case SCG_VERSION:
-			return (_scg_trans_version);
-		/*
-		 * If you changed this source, you are not allowed to
-		 * return "schily" for the SCG_AUTHOR request.
-		 */
-		case SCG_AUTHOR:
-			return (_scg_auth_schily);
-		case SCG_SCCS_ID:
-			return (__sccsid);
+	switch (what) {
 
-		case SCG_RVERSION:
-			if (scglocal(scgp)->v_version == NULL)
-				scglocal(scgp)->v_version = rscsiversion(scgp, f, SCG_VERSION);
-			return (scglocal(scgp)->v_version);
-		/*
-		 * If you changed this source, you are not allowed to
-		 * return "schily" for the SCG_AUTHOR request.
-		 */
-		case SCG_RAUTHOR:
-			if (scglocal(scgp)->v_author == NULL)
-				scglocal(scgp)->v_author = rscsiversion(scgp, f, SCG_AUTHOR);
-			return (scglocal(scgp)->v_author);
-		case SCG_RSCCS_ID:
-			if (scglocal(scgp)->v_sccs_id == NULL)
-				scglocal(scgp)->v_sccs_id = rscsiversion(scgp, f, SCG_SCCS_ID);
-			return (scglocal(scgp)->v_sccs_id);
-		}
+	case SCG_VERSION:
+		return (_scg_trans_version);
+	/*
+	 * If you changed this source, you are not allowed to
+	 * return "schily" for the SCG_AUTHOR request.
+	 */
+	case SCG_AUTHOR:
+		return (_scg_auth_schily);
+	case SCG_SCCS_ID:
+		return (__sccsid);
+
+	case SCG_RVERSION:
+		if (scglocal(scgp)->v_version == NULL)
+			scglocal(scgp)->v_version = rscsiversion(scgp, f, SCG_VERSION);
+		return (scglocal(scgp)->v_version);
+	/*
+	 * If you changed this source, you are not allowed to
+	 * return "schily" for the SCG_AUTHOR request.
+	 */
+	case SCG_RAUTHOR:
+		if (scglocal(scgp)->v_author == NULL)
+			scglocal(scgp)->v_author = rscsiversion(scgp, f, SCG_AUTHOR);
+		return (scglocal(scgp)->v_author);
+	case SCG_RSCCS_ID:
+		if (scglocal(scgp)->v_sccs_id == NULL)
+		scglocal(scgp)->v_sccs_id = rscsiversion(scgp, f, SCG_SCCS_ID);
+		return (scglocal(scgp)->v_sccs_id);
 	}
 	return ((char *)0);
 }
@@ -259,12 +270,12 @@ scgo_ropen(scgp, device)
 	SCSI	*scgp;
 	char	*device;
 {
-		 int	busno	= scg_scsibus(scgp);
-		 int	tgt	= scg_target(scgp);
-		 int	tlun	= scg_lun(scgp);
+		int	busno	= scg_scsibus(scgp);
+		int	tgt	= scg_target(scgp);
+		int	tlun	= scg_lun(scgp);
 	register int	f;
 	register int	nopen = 0;
-	char		devname[128];
+	char		sdevname[128];
 	char		*p;
 
 	if (scgp->overbose)
@@ -273,14 +284,14 @@ scgo_ropen(scgp, device)
 	if (busno >= MAX_SCG || tgt >= MAX_TGT || tlun >= MAX_LUN) {
 		errno = EINVAL;
 		if (scgp->errstr)
-		    js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
-		       "Illegal value for busno, target or lun '%d,%d,%d'",
-		       busno, tgt, tlun);
+			js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
+			"Illegal value for busno, target or lun '%d,%d,%d'",
+			busno, tgt, tlun);
 
 		return (-1);
 	}
 	if (scgp->local == NULL) {
-		scgp->local = malloc(sizeof(struct scg_local));
+		scgp->local = malloc(sizeof (struct scg_local));
 		if (scgp->local == NULL)
 			return (0);
 		scglocal(scgp)->remfd = -1;
@@ -297,24 +308,24 @@ scgo_ropen(scgp, device)
 	if (device == NULL || (strncmp(device, "REMOTE", 6) != 0) ||
 				(device = strchr(device, ':')) == NULL) {
 		if (scgp->errstr)
-		    js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
-		       "Illegal remote device syntax");
+			js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
+				"Illegal remote device syntax");
 		return (-1);
 	}
 	device++;
 	/*
 	 * Save non user@host:device
 	 */
-	js_snprintf(devname, sizeof(devname), "%s", device);
+	js_snprintf(sdevname, sizeof (sdevname), "%s", device);
 
-	if ((p = strchr(devname, ':')) != NULL)
+	if ((p = strchr(sdevname, ':')) != NULL)
 		*p++ = '\0';
 
-	f = rscsigetconn(scgp, devname);
+	f = rscsigetconn(scgp, sdevname);
 	if (f < 0) {
 		if (scgp->errstr)
-		    js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
-		       "Cannot get connection to remote host");
+			js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
+				"Cannot get connection to remote host");
 		return (-1);
 	}
 	scglocal(scgp)->remfd = f;
@@ -331,7 +342,7 @@ scgo_rclose(scgp)
 	SCSI	*scgp;
 {
 	register int	f;
-		 int	ret;
+		int	ret;
 
 	if (scgp->local == NULL)
 		return (-1);
@@ -415,6 +426,13 @@ scgo_rfreebuf(scgp)
 	rscsifreebuf(scgp, f);
 }
 
+LOCAL int
+scgo_rnumbus(scgp)
+	SCSI	*scgp;
+{
+	return (rscsinumbus(scgp, scglocal(scgp)->remfd));
+}
+
 LOCAL BOOL
 scgo_rhavebus(scgp, busno)
 	SCSI	*scgp;
@@ -457,7 +475,7 @@ scgo_rinitiator_id(scgp)
 	return (rscsiinitiator_id(scgp, scglocal(scgp)->remfd));
 }
 
-LOCAL int 
+LOCAL int
 scgo_risatapi(scgp)
 	SCSI	*scgp;
 {
@@ -537,9 +555,9 @@ rscsigetconn(scgp, host)
 	if ((p = strchr(host, '@')) != NULL) {
 		size_t d = p - host;
 
-		if (d > sizeof(rscsiuser))
-			d = sizeof(rscsiuser);
-		js_snprintf(rscsiuser, sizeof(rscsiuser), "%.*s", (int)d, host);
+		if (d > sizeof (rscsiuser))
+			d = sizeof (rscsiuser);
+		js_snprintf(rscsiuser, sizeof (rscsiuser), "%.*s", (int)d, host);
 		name = rscsiuser;
 		host = &p[1];
 	} else {
@@ -581,8 +599,10 @@ rscsiversion(scgp, fd, what)
 	char	*p;
 	int	ret;
 
-	js_snprintf(cbuf, sizeof(cbuf), "V%d\n", what);
+	js_snprintf(cbuf, sizeof (cbuf), "V%d\n", what);
 	ret = rscsicmd(scgp, fd, "version", cbuf);
+	if (ret <= 0)
+		return (NULL);
 	p = malloc(ret);
 	if (p == NULL)
 		return (p);
@@ -603,13 +623,13 @@ rscsiopen(scgp, fd, fname)
 	int	tgt;
 	int	lun;
 
-	js_snprintf(cbuf, sizeof(cbuf), "O%s\n", fname?fname:"");
+	js_snprintf(cbuf, sizeof (cbuf), "O%s\n", fname?fname:"");
 	ret = rscsicmd(scgp, fd, "open", cbuf);
 	if (ret < 0)
 		return (ret);
 
 	bus = rscsireadnum(scgp, fd);
-	chan = rscsireadnum(scgp, fd);	
+	chan = rscsireadnum(scgp, fd);
 	tgt = rscsireadnum(scgp, fd);
 	lun = rscsireadnum(scgp, fd);
 
@@ -633,7 +653,7 @@ rscsimaxdma(scgp, fd, amt)
 {
 	char	cbuf[CMD_SIZE];
 
-	js_snprintf(cbuf, sizeof(cbuf), "D%ld\n", amt);
+	js_snprintf(cbuf, sizeof (cbuf), "D%ld\n", amt);
 	return (rscsicmd(scgp, fd, "maxdma", cbuf));
 }
 
@@ -647,7 +667,7 @@ rscsigetbuf(scgp, fd, amt)
 	int	size;
 	int	ret;
 
-	js_snprintf(cbuf, sizeof(cbuf), "M%ld\n", amt);
+	js_snprintf(cbuf, sizeof (cbuf), "M%ld\n", amt);
 	ret = rscsicmd(scgp, fd, "getbuf", cbuf);
 	if (ret < 0)
 		return (ret);
@@ -689,6 +709,14 @@ rscsifreebuf(scgp, fd)
 	return (rscsicmd(scgp, fd, "freebuf", "F\n"));
 }
 
+LOCAL int
+rscsinumbus(scgp, fd)
+	SCSI	*scgp;
+	int	fd;
+{
+	return (rscsicmd(scgp, fd, "numbus", "N\n"));
+}
+
 LOCAL int
 rscsihavebus(scgp, fd, busno)
 	SCSI	*scgp;
@@ -697,7 +725,7 @@ rscsihavebus(scgp, fd, busno)
 {
 	char	cbuf[2*CMD_SIZE];
 
-	js_snprintf(cbuf, sizeof(cbuf), "B%d\n%d\n",
+	js_snprintf(cbuf, sizeof (cbuf), "B%d\n%d\n",
 		busno,
 		0);
 	return (rscsicmd(scgp, fd, "havebus", cbuf));
@@ -713,7 +741,7 @@ rscsifileno(scgp, fd, busno, tgt, tlun)
 {
 	char	cbuf[3*CMD_SIZE];
 
-	js_snprintf(cbuf, sizeof(cbuf), "T%d\n%d\n%d\n%d\n",
+	js_snprintf(cbuf, sizeof (cbuf), "T%d\n%d\n%d\n%d\n",
 		busno,
 		0,
 		tgt,
@@ -745,7 +773,7 @@ rscsireset(scgp, fd, what)
 {
 	char	cbuf[CMD_SIZE];
 
-	js_snprintf(cbuf, sizeof(cbuf), "R%d\n", what);
+	js_snprintf(cbuf, sizeof (cbuf), "R%d\n", what);
 	return (rscsicmd(scgp, fd, "reset", cbuf));
 }
 
@@ -760,7 +788,7 @@ rscsiscmd(scgp, fd, sp)
 	int	amt = 0;
 	int	voidsize = 0;
 
-	ret = js_snprintf(cbuf, sizeof(cbuf), "S%d\n%d\n%d\n%d\n%d\n",
+	ret = js_snprintf(cbuf, sizeof (cbuf), "S%d\n%d\n%d\n%d\n%d\n",
 		sp->size, sp->flags,
 		sp->cdb_len, sp->sense_len,
 		sp->timeout);
@@ -769,7 +797,7 @@ rscsiscmd(scgp, fd, sp)
 
 	if ((sp->flags & SCG_RECV_DATA) == 0 && sp->size > 0) {
 		amt = sp->size;
-		if ((ret + amt) <= sizeof(cbuf)) {
+		if ((ret + amt) <= sizeof (cbuf)) {
 			movebytes(sp->addr, &cbuf[ret], amt);
 			ret += amt;
 			amt = 0;
@@ -778,7 +806,7 @@ rscsiscmd(scgp, fd, sp)
 	errno = 0;
 	if (_nixwrite(fd, cbuf, ret) != ret)
 		rscsiaborted(scgp, fd);
-	
+
 	if (amt > 0) {
 		if (_nixwrite(fd, sp->addr, amt) != amt)
 			rscsiaborted(scgp, fd);
@@ -803,7 +831,7 @@ rscsiscmd(scgp, fd, sp)
 		rscsivoidarg(scgp, fd, voidsize);
 	}
 
-	if ((sp->flags & SCG_RECV_DATA) != 0 && ret > 0) 
+	if ((sp->flags & SCG_RECV_DATA) != 0 && ret > 0)
 		rscsireadbuf(scgp, fd, sp->addr, ret);
 
 	return (0);
@@ -817,7 +845,7 @@ rscsifillrbuf(scgp)
 
 	return (scglocal(scgp)->readbcnt =
 			_niread(scglocal(scgp)->remfd,
-			     scglocal(scgp)->readbuf, READBUF_SIZE));
+			    scglocal(scgp)->readbuf, READBUF_SIZE));
 }
 
 LOCAL int
@@ -871,10 +899,10 @@ rscsivoidarg(scgp, fd, n)
 {
 	register int	i;
 	register int	amt;
-		 char	buf[512];
+		char	buf[512];
 
 	for (i = 0; i < n; i += amt) {
-		amt = sizeof(buf);
+		amt = sizeof (buf);
 		if ((n - i) < amt)
 			amt = n - i;
 		rscsireadbuf(scgp, fd, buf, amt);
@@ -934,7 +962,7 @@ rscsireadnum(scgp, fd)
 {
 	char	cbuf[CMD_SIZE];
 
-	rscsigetline(scgp, fd, cbuf, sizeof(cbuf));
+	rscsigetline(scgp, fd, cbuf, sizeof (cbuf));
 	return (atoi(cbuf));
 }
 
@@ -950,16 +978,16 @@ rscsigetstatus(scgp, fd, name)
 	int	count;
 	int	voidsize = 0;
 
-	rscsigetline(scgp, fd, cbuf, sizeof(cbuf));
+	rscsigetline(scgp, fd, cbuf, sizeof (cbuf));
 	code = cbuf[0];
 	number = atoi(&cbuf[1]);
 
 	if (code == 'E' || code == 'F') {
-		rscsigetline(scgp, fd, cbuf, sizeof(cbuf));
+		rscsigetline(scgp, fd, cbuf, sizeof (cbuf));
 		if (code == 'F')	/* should close file ??? */
 			rscsiaborted(scgp, fd);
 
-		rscsigetline(scgp, fd, cbuf, sizeof(cbuf));
+		rscsigetline(scgp, fd, cbuf, sizeof (cbuf));
 		count = atoi(cbuf);
 		if (count > 0) {
 			if (scgp->errstr == NULL) {
@@ -969,7 +997,8 @@ rscsigetstatus(scgp, fd, name)
 				voidsize = count - SCSI_ERRSTR_SIZE;
 				count = SCSI_ERRSTR_SIZE;
 			}
-			rscsireadbuf(scgp, fd, scgp->errstr, count);
+			if (count > 0)
+				rscsireadbuf(scgp, fd, scgp->errstr, count);
 			rscsivoidarg(scgp, fd, voidsize);
 		}
 		if (scgp->debug > 0)
@@ -1012,7 +1041,7 @@ rscsiaborted(scgp, fd)
  *
  * and make sure that we use sigset() instead of signal() if possible.
  */
-#include <waitdefs.h>
+#include <schily/wait.h>
 LOCAL int
 _rcmdrsh(ahost, inport, locuser, remuser, cmd, rsh)
 	char		**ahost;
@@ -1058,7 +1087,7 @@ _rcmdrsh(ahost, inport, locuser, remuser, cmd, rsh)
 		const char	*av0;
 		int		xpid;
 
-		(void)close(pp[0]);
+		(void) close(pp[0]);
 		if (dup2(pp[1], 0) == -1 ||	/* Pipe becomes 'stdin'  */
 		    dup2(0, 1) == -1) {		/* Pipe becomes 'stdout' */
 
@@ -1066,7 +1095,7 @@ _rcmdrsh(ahost, inport, locuser, remuser, cmd, rsh)
 			_exit(EX_BAD);
 			/* NOTREACHED */
 		}
-		(void)close(pp[1]);		/* We don't need this anymore*/
+		(void) close(pp[1]);		/* We don't need this anymore*/
 
 		/*
 		 * Become 'locuser' to tell the rsh program the local user id.
@@ -1078,6 +1107,21 @@ _rcmdrsh(ahost, inport, locuser, remuser, cmd, rsh)
 			_exit(EX_BAD);
 			/* NOTREACHED */
 		}
+		if (getuid() != geteuid() &&
+#ifdef	HAVE_SETREUID
+		    setreuid(-1, pw->pw_uid) == -1) {
+#else
+#ifdef	HAVE_SETEUID
+		    seteuid(pw->pw_uid) == -1) {
+#else
+		    setuid(pw->pw_uid) == -1) {
+#endif
+#endif
+			errmsg("seteuid(%lld) failed.\n",
+							(Llong)pw->pw_uid);
+			_exit(EX_BAD);
+			/* NOTREACHED */
+		}
 
 		/*
 		 * Fork again to completely detach from parent
@@ -1129,13 +1173,13 @@ _rcmdrsh(ahost, inport, locuser, remuser, cmd, rsh)
 		av0 = rsh;
 		if ((p = strrchr(rsh, '/')) != NULL)
 			av0 = ++p;
-		execlp(rsh, av0, *ahost, "-l", remuser, cmd, NULL);
+		execlp(rsh, av0, *ahost, "-l", remuser, cmd, (char *)NULL);
 
 		errmsg("execlp '%s' failed.\n", rsh);
 		_exit(EX_BAD);
 		/* NOTREACHED */
 	} else {
-		(void)close(pp[1]);
+		(void) close(pp[1]);
 		/*
 		 * Wait for the intermediate child.
 		 * The real 'rsh' program is completely detached from us.
@@ -1145,6 +1189,48 @@ _rcmdrsh(ahost, inport, locuser, remuser, cmd, rsh)
 	}
 	return (-1);	/* keep gcc happy */
 }
+
+#ifdef	HAVE_SOLARIS_PPRIV
+
+LOCAL BOOL
+ppriv_ok()
+{
+	priv_set_t	*privset;
+	BOOL		net_privaddr = FALSE;
+
+
+	if ((privset = priv_allocset()) == NULL) {
+		return (FALSE);
+	}
+	if (getppriv(PRIV_EFFECTIVE, privset) == -1) {
+		priv_freeset(privset);
+		return (FALSE);
+	}
+	if (priv_ismember(privset, PRIV_NET_PRIVADDR)) {
+		net_privaddr = TRUE;
+	}
+	priv_freeset(privset);
+
+	return (net_privaddr);
+}
+#else	/* HAVE_SOLARIS_PPRIV */
+
+#ifdef	HAVE_LINUX_CAPS
+LOCAL BOOL
+ppriv_ok()
+{
+	cap_t			cset;
+	cap_flag_value_t	val = CAP_CLEAR;
+
+	cset = cap_get_proc();
+
+	cap_get_flag(cset, CAP_NET_BIND_SERVICE, CAP_EFFECTIVE, &val);
+	return (val == CAP_SET);
+}
+#endif	/* HAVE_LINUX_CAPS */
+
+#endif	/* HAVE_SOLARIS_PPRIV */
+
 #endif	/* USE_RCMD_RSH */
 
 #endif	/* USE_REMOTE */
