@@ -1,7 +1,7 @@
-/* @(#)scsi-bsd.c	1.41 02/10/19 Copyright 1997 J. Schilling */
+/* @(#)scsi-bsd.c	1.52 11/01/01 Copyright 1997-2011 J. Schilling */
 #ifndef lint
 static	char __sccsid[] =
-	"@(#)scsi-bsd.c	1.41 02/10/19 Copyright 1997 J. Schilling";
+	"@(#)scsi-bsd.c	1.52 11/01/01 Copyright 1997-2011 J. Schilling";
 #endif
 /*
  *	Interface for the NetBSD/FreeBSD/OpenBSD generic SCSI implementation.
@@ -18,28 +18,37 @@ static	char __sccsid[] =
  *	Choose your name instead of "schily" and make clear that the version
  *	string is related to a modified source.
  *
- *	Copyright (c) 1997 J. Schilling
+ *	Copyright (c) 1997-2011 J. Schilling
  */
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * The contents of this file are subject to the terms of the
+ * Common Development and Distribution License, Version 1.0 only
+ * (the "License").  You may not use this file except in compliance
+ * with the License.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * See the file CDDL.Schily.txt in this distribution for details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * The following exceptions apply:
+ * CDDL 3.6 needs to be replaced by: "You may create a Larger Work by
+ * combining Covered Software with other code if all other code is governed by
+ * the terms of a license that is OSI approved (see www.opensource.org) and
+ * you may distribute the Larger Work as a single product. In such a case,
+ * You must make sure the requirements of this License are fulfilled for
+ * the Covered Software."
+ *
+ * When distributing Covered Code, include this CDDL HEADER in each
+ * file and include the License file CDDL.Schily.txt from this distribution.
  */
 
 #ifndef HAVE_CAMLIB_H
 
 #undef	sense
 #include <sys/scsiio.h>
+#if defined(__NetBSD__)
+#ifdef	USE_GETRAWPARTITION
+#include <util.h>
+#endif
+#endif
 
 /*
  *	Warning: you may change this source, but if you do that
@@ -48,16 +57,16 @@ static	char __sccsid[] =
  *	Choose your name instead of "schily" and make clear that the version
  *	string is related to a modified source.
  */
-LOCAL	char	_scg_trans_version[] = "scsi-bsd.c-1.41";	/* The version for this transport*/
+LOCAL	char	_scg_trans_version[] = "scsi-bsd.c-1.52";	/* The version for this transport*/
 
-#define	MAX_SCG		16	/* Max # of SCSI controllers */
+#define	MAX_SCG		32	/* Max # of SCSI controllers */
 #define	MAX_TGT		16
 #define	MAX_LUN		8
 
 struct scg_local {
 	short	scgfiles[MAX_SCG][MAX_TGT][MAX_LUN];
 };
-#define scglocal(p)	((struct scg_local *)((p)->local)) 
+#define	scglocal(p)	((struct scg_local *)((p)->local))
 
 /*#define	MAX_DMA_BSD	(32*1024)*/
 #define	MAX_DMA_BSD	(60*1024)	/* More seems to make problems */
@@ -86,6 +95,9 @@ struct scg_local {
 #define	SADDR_LUN(a)	(a).lun
 #endif	/* __NetBSD__ && TYPE_ATAPI */
 
+#if defined(__NetBSD__) || defined(__OpenBSD__)
+LOCAL	int	getslice	__PR((int n));
+#endif
 LOCAL	BOOL	scg_setup	__PR((SCSI *scgp, int f, int busno, int tgt, int tlun));
 
 /*
@@ -131,15 +143,22 @@ scgo_open(scgp, device)
 	SCSI	*scgp;
 	char	*device;
 {
-		 int	busno	= scg_scsibus(scgp);
-		 int	tgt	= scg_target(scgp);
-		 int	tlun	= scg_lun(scgp);
+		int	busno	= scg_scsibus(scgp);
+		int	tgt	= scg_target(scgp);
+		int	tlun	= scg_lun(scgp);
 	register int	f;
 	register int	b;
 	register int	t;
 	register int	l;
 	register int	nopen = 0;
-	char		devname[64];
+	char		dev_name[64];
+#if defined(__NetBSD__) || defined(__OpenBSD__)
+#ifdef	USE_GETRAWPARTITION
+		int	myslicename = getrawpartition();
+#else
+		int	myslicename = 0;
+#endif
+#endif
 
 	if (busno >= MAX_SCG || tgt >= MAX_TGT || tlun >= MAX_LUN) {
 		errno = EINVAL;
@@ -151,13 +170,13 @@ scgo_open(scgp, device)
 	}
 
 	if (scgp->local == NULL) {
-		scgp->local = malloc(sizeof(struct scg_local));
+		scgp->local = malloc(sizeof (struct scg_local));
 		if (scgp->local == NULL)
 			return (0);
 
-		for (b=0; b < MAX_SCG; b++) {
-			for (t=0; t < MAX_TGT; t++) {
-				for (l=0; l < MAX_LUN ; l++)
+		for (b = 0; b < MAX_SCG; b++) {
+			for (t = 0; t < MAX_TGT; t++) {
+				for (l = 0; l < MAX_LUN; l++)
 					scglocal(scgp)->scgfiles[b][t][l] = (short)-1;
 			}
 		}
@@ -166,24 +185,106 @@ scgo_open(scgp, device)
 	if ((device != NULL && *device != '\0') || (busno == -2 && tgt == -2))
 		goto openbydev;
 
+
+#if defined(__NetBSD__) || defined(__OpenBSD__)
+	/*
+	 * I know of no method in NetBSD/OpenBSD to probe the scsibus and get
+	 * the mapping busnumber,target,lun --> devicename.
+	 *
+	 * For now, implement a true brute force hack to find cdroms to allow
+	 * cdrecord to work, but libscg is a generic SCSI transport lib.
+	 *
+	 * Note that this method only finds cd0-cd9. For a more correct
+	 * implementation we would at least need to also scan the disk and tape
+	 * devices.
+	 *
+	 * This still does not include scanners and other SCSI devices.
+	 * Help is needed!
+	 */
+	if (busno >= 0 && tgt >= 0 && tlun >= 0) {
+		struct scsi_addr mysaddr;
+
+		for (l = 0; l < 10; l++) {
+		nextslice1:
+			sprintf(dev_name, "/dev/rcd%d%c", l, 'a' + myslicename);
+			f = open(dev_name, O_RDWR);
+			if (f >= 0) {
+				if (ioctl(f, SCIOCIDENTIFY, &mysaddr) < 0) {
+#ifndef	USE_GETRAWPARTITION
+					if (errno == ENOTTY && myslicename < 16) {
+						close(f);
+						myslicename = getslice(l);
+						goto nextslice1;
+					}
+#endif
+					close(f);
+					errno = EINVAL;
+					return (0);
+				}
+				if (busno == SADDR_BUS(mysaddr) &&
+				    tgt == SADDR_TARGET(mysaddr) &&
+				    tlun == SADDR_LUN(mysaddr)) {
+					scglocal(scgp)->scgfiles[busno][tgt][tlun] = f;
+					return (1);
+				}
+			} else {
+#if	defined(__OpenBSD__) && defined(ENOMEDIUM)
+				if (errno == ENOMEDIUM && myslicename < 16) {
+					myslicename = getslice(l);
+					goto nextslice1;
+				}
+#endif
+			}
+		}
+	} else for (l = 0; l < 10; l++) {
+		struct scsi_addr mysaddr;
+
+	nextslice2:
+		sprintf(dev_name, "/dev/rcd%d%c", l, 'a' + myslicename);
+		f = open(dev_name, O_RDWR);
+		if (f >= 0) {
+			if (ioctl(f, SCIOCIDENTIFY, &mysaddr) < 0) {
+#ifndef	USE_GETRAWPARTITION
+				if (errno == ENOTTY && myslicename < 16) {
+					close(f);
+					myslicename = getslice(l);
+					goto nextslice2;
+				}
+#endif
+				close(f);
+				errno = EINVAL;
+				return (0);
+			}
+			if (scg_setup(scgp, f, busno, tgt, tlun))
+				nopen++;
+		} else {
+#if	defined(__OpenBSD__) && defined(ENOMEDIUM)
+			if (errno == ENOMEDIUM && myslicename < 16) {
+				myslicename = getslice(l);
+				goto nextslice2;
+			}
+#endif
+		}
+	}
+#else /* ! __NetBSD__ || __OpenBSD__ */
 	if (busno >= 0 && tgt >= 0 && tlun >= 0) {
 
-		js_snprintf(devname, sizeof(devname),
-				"/dev/su%d-%d-%d", busno, tgt, tlun); 
-		f = open(devname, O_RDWR);
+		js_snprintf(dev_name, sizeof (dev_name),
+				"/dev/su%d-%d-%d", busno, tgt, tlun);
+		f = open(dev_name, O_RDWR);
 		if (f < 0) {
 			goto openbydev;
 		}
 		scglocal(scgp)->scgfiles[busno][tgt][tlun] = f;
-		return(1);
+		return (1);
 
-	} else for (b=0; b < MAX_SCG; b++) {
-		for (t=0; t < MAX_TGT; t++) {
-			for (l=0; l < MAX_LUN ; l++) {
-				js_snprintf(devname, sizeof(devname),
+	} else for (b = 0; b < MAX_SCG; b++) {
+		for (t = 0; t < MAX_TGT; t++) {
+			for (l = 0; l < MAX_LUN; l++) {
+				js_snprintf(dev_name, sizeof (dev_name),
 							"/dev/su%d-%d-%d", b, t, l);
-				f = open(devname, O_RDWR);
-/*				error("open (%s) = %d\n", devname, f);*/
+				f = open(dev_name, O_RDWR);
+/*				error("open (%s) = %d\n", dev_name, f);*/
 
 				if (f < 0) {
 					if (errno != ENOENT &&
@@ -192,7 +293,7 @@ scgo_open(scgp, device)
 						if (scgp->errstr)
 							js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
 								"Cannot open '%s'",
-								devname);
+								dev_name);
 						return (0);
 					}
 				} else {
@@ -202,6 +303,7 @@ scgo_open(scgp, device)
 			}
 		}
 	}
+#endif /* ! __NetBSD__ || __OpebBSD__ */
 	/*
 	 * Could not open /dev/su-* or got dev=devname:b,l,l / dev=devname:@,l
 	 * We do the apropriate tests and try our best.
@@ -242,6 +344,35 @@ openbydev:
 	return (nopen);
 }
 
+#if defined(__NetBSD__) || defined(__OpenBSD__)
+/*
+ * Avoid to call getrawpartition() because this would force us to
+ * link against libutil.
+ */
+LOCAL int
+getslice(n)
+	int	n;
+{
+	struct scsi_addr saddr;
+	char		dev_name[64];
+	int		slice;
+	int		f;
+
+	for (slice = 0; slice < 16; slice++) {
+		sprintf(dev_name, "/dev/rcd%d%c", n, 'a' + slice);
+		f = open(dev_name, O_RDWR);
+		if (f >= 0) {
+			if (ioctl(f, SCIOCIDENTIFY, &saddr) >= 0) {
+				close(f);
+				break;
+			}
+			close(f);
+		}
+	}
+	return (slice);
+}
+#endif
+
 LOCAL int
 scgo_close(scgp)
 	SCSI	*scgp;
@@ -254,9 +385,9 @@ scgo_close(scgp)
 	if (scgp->local == NULL)
 		return (-1);
 
-	for (b=0; b < MAX_SCG; b++) {
-		for (t=0; t < MAX_TGT; t++) {
-			for (l=0; l < MAX_LUN ; l++) {
+	for (b = 0; b < MAX_SCG; b++) {
+		for (t = 0; t < MAX_TGT; t++) {
+			for (l = 0; l < MAX_LUN; l++) {
 				f = scglocal(scgp)->scgfiles[b][t][l];
 				if (f >= 0)
 					close(f);
@@ -303,8 +434,10 @@ scg_setup(scgp, f, busno, tgt, tlun)
 		return (FALSE);
 	}
 
-	if (scglocal(scgp)->scgfiles[Bus][Target][Lun] == (short)-1)
+	if (scglocal(scgp)->scgfiles[Bus][Target][Lun] == (short)-1) {
 		scglocal(scgp)->scgfiles[Bus][Target][Lun] = (short)f;
+		return (TRUE);
+	}
 
 	if (onetarget) {
 		if (Bus == busno && Target == tgt && Lun == tlun) {
@@ -349,6 +482,13 @@ scgo_freebuf(scgp)
 	scgp->bufbase = NULL;
 }
 
+LOCAL int
+scgo_numbus(scgp)
+	SCSI	*scgp;
+{
+	return (MAX_SCG);
+}
+
 LOCAL BOOL
 scgo_havebus(scgp, busno)
 	SCSI	*scgp;
@@ -363,8 +503,8 @@ scgo_havebus(scgp, busno)
 	if (scgp->local == NULL)
 		return (FALSE);
 
-	for (t=0; t < MAX_TGT; t++) {
-		for (l=0; l < MAX_LUN ; l++)
+	for (t = 0; t < MAX_TGT; t++) {
+		for (l = 0; l < MAX_LUN; l++)
 			if (scglocal(scgp)->scgfiles[busno][t][l] >= 0)
 				return (TRUE);
 	}
@@ -462,9 +602,9 @@ scgo_send(scgp)
 	req.databuf = sp->addr;
 	req.datalen = sp->size;
 	req.datalen_used = 0;
-	fillbytes(req.sense, sizeof(req.sense), '\0');
-	if (sp->sense_len > sizeof(req.sense))
-		req.senselen = sizeof(req.sense);
+	fillbytes(req.sense, sizeof (req.sense), '\0');
+	if (sp->sense_len > sizeof (req.sense))
+		req.senselen = sizeof (req.sense);
 	else if (sp->sense_len < 0)
 		req.senselen = 0;
 	else
@@ -484,8 +624,8 @@ scgo_send(scgp)
 		if (req.retsts != SCCMD_OK)
 			sp->ux_errno = EIO;
 	}
-	fillbytes(&sp->scb, sizeof(sp->scb), '\0');
-	fillbytes(&sp->u_sense.cmd_sense, sizeof(sp->u_sense.cmd_sense), '\0');
+	fillbytes(&sp->scb, sizeof (sp->scb), '\0');
+	fillbytes(&sp->u_sense.cmd_sense, sizeof (sp->u_sense.cmd_sense), '\0');
 	sp->resid = req.datalen - req.datalen_used;
 	sp->sense_count = req.senselen_used;
 	if (sp->sense_count > SCG_MAX_SENSE)
@@ -518,7 +658,7 @@ scgo_send(scgp)
  *
  * Copyright (c) 1998 Michael Smith <msmith@freebsd.org>
  * Copyright (c) 1998 Kenneth D. Merry <ken@kdm.org>
- * Copyright (c) 1998 Joerg Schilling <schilling@fokus.gmd.de>
+ * Copyright (c) 1998-2011 Joerg Schilling <joerg.schilling@fokus.fraunhofer.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -545,8 +685,8 @@ scgo_send(scgp)
  */
 
 #undef	sense
-#define scsi_sense CAM_scsi_sense
-#define scsi_inquiry CAM_scsi_inquiry
+#define	scsi_sense CAM_scsi_sense
+#define	scsi_inquiry CAM_scsi_inquiry
 #include <sys/param.h>
 #include <cam/cam.h>
 #include <cam/cam_ccb.h>
@@ -561,13 +701,13 @@ scgo_send(scgp)
  *	Choose your name instead of "schily" and make clear that the version
  *	string is related to a modified source.
  */
-LOCAL	char	_scg_trans_version[] = "scsi-bsd.c-1.41";	/* The version for this transport*/
+LOCAL	char	_scg_trans_version[] = "scsi-bsd.c-1.52";	/* The version for this transport*/
 
-#define CAM_MAXDEVS	128
+#define	CAM_MAXDEVS	128
 struct scg_local {
 	struct cam_device *cam_devices[CAM_MAXDEVS + 1];
 };
-#define scglocal(p)	((struct scg_local *)((p)->local)) 
+#define	scglocal(p)	((struct scg_local *)((p)->local))
 
 /*
  * Return version information for the low level SCSI transport code.
@@ -626,6 +766,7 @@ scgo_open(scgp, device)
 	struct periph_match_pattern	*match_pat;
 	int				fd;
 
+	seterrno(0);
 	if ((device != NULL && *device != '\0') || (busno == -2 && tgt == -2)) {
 		errno = EINVAL;
 		if (scgp->errstr)
@@ -635,7 +776,7 @@ scgo_open(scgp, device)
 	}
 
 	if (scgp->local == NULL) {
-		scgp->local = malloc(sizeof(struct scg_local));
+		scgp->local = malloc(sizeof (struct scg_local));
 		if (scgp->local == NULL)
 			return (0);
 
@@ -651,9 +792,9 @@ scgo_open(scgp, device)
 	if (busno >= 0 && tgt >= 0 && tlun >= 0) {
 		scglocal(scgp)->cam_devices[0] = cam_open_btl(busno, tgt, tlun, O_RDWR, NULL);
 		if (scglocal(scgp)->cam_devices[0] == NULL)
-			return(-1);
+			return (-1);
 		nopen++;
-		return(nopen);
+		return (nopen);
 	}
 
 	/*
@@ -665,20 +806,23 @@ scgo_open(scgp, device)
 		if (scgp->errstr)
 			js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
 				"Open of %s failed", XPT_DEVICE);
-		return(-1);
+		return (-1);
 	}
-	fillbytes(&ccb, sizeof(union ccb), '\0');
+	fillbytes(&ccb, sizeof (union ccb), '\0');
 
 	/*
 	 * Get a list of up to CAM_MAXDEVS passthrough devices in the
 	 * system.
 	 */
 	ccb.ccb_h.func_code = XPT_DEV_MATCH;
+	ccb.ccb_h.path_id = CAM_XPT_PATH_ID;
+	ccb.ccb_h.target_id = CAM_TARGET_WILDCARD;
+	ccb.ccb_h.target_lun = CAM_LUN_WILDCARD;
 
 	/*
 	 * Setup the result buffer.
 	 */
-	bufsize = sizeof(struct dev_match_result) * CAM_MAXDEVS;
+	bufsize = sizeof (struct dev_match_result) * CAM_MAXDEVS;
 	ccb.cdm.match_buf_len = bufsize;
 	ccb.cdm.matches = (struct dev_match_result *)malloc(bufsize);
 	if (ccb.cdm.matches == NULL) {
@@ -686,7 +830,7 @@ scgo_open(scgp, device)
 			js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
 				"Couldn't malloc match buffer");
 		close(fd);
-		return(-1);
+		return (-1);
 	}
 	ccb.cdm.num_matches = 0;
 
@@ -695,20 +839,20 @@ scgo_open(scgp, device)
 	 * peripherals named "pass".
 	 */
 	ccb.cdm.num_patterns = 1;
-	ccb.cdm.pattern_buf_len = sizeof(struct dev_match_pattern);
+	ccb.cdm.pattern_buf_len = sizeof (struct dev_match_pattern);
 	ccb.cdm.patterns = (struct dev_match_pattern *)malloc(
-		sizeof(struct dev_match_pattern));
+		sizeof (struct dev_match_pattern));
 	if (ccb.cdm.patterns == NULL) {
 		if (scgp->errstr)
 			js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
 				"Couldn't malloc pattern buffer");
 		close(fd);
 		free(ccb.cdm.matches);
-		return(-1);
+		return (-1);
 	}
 	ccb.cdm.patterns[0].type = DEV_MATCH_PERIPH;
 	match_pat = &ccb.cdm.patterns[0].pattern.periph_pattern;
-	js_snprintf(match_pat->periph_name, sizeof(match_pat->periph_name),
+	js_snprintf(match_pat->periph_name, sizeof (match_pat->periph_name),
 								"pass");
 	match_pat->flags = PERIPH_MATCH_NAME;
 
@@ -719,12 +863,12 @@ scgo_open(scgp, device)
 		close(fd);
 		free(ccb.cdm.matches);
 		free(ccb.cdm.patterns);
-		return(-1);
+		return (-1);
 	}
 
-	if ((ccb.ccb_h.status != CAM_REQ_CMP)
-	 || ((ccb.cdm.status != CAM_DEV_MATCH_LAST)
-	    && (ccb.cdm.status != CAM_DEV_MATCH_MORE))) {
+	if ((ccb.ccb_h.status != CAM_REQ_CMP) ||
+	    ((ccb.cdm.status != CAM_DEV_MATCH_LAST) &&
+	    (ccb.cdm.status != CAM_DEV_MATCH_MORE))) {
 /*		errmsgno(EX_BAD, "Got CAM error 0x%X, CDM error %d.\n",*/
 		if (scgp->errstr)
 			js_snprintf(scgp->errstr, SCSI_ERRSTR_SIZE,
@@ -733,7 +877,7 @@ scgo_open(scgp, device)
 		close(fd);
 		free(ccb.cdm.matches);
 		free(ccb.cdm.patterns);
-		return(-1);
+		return (-1);
 	}
 
 	free(ccb.cdm.patterns);
@@ -754,11 +898,11 @@ scgo_open(scgp, device)
 					"Kernel error! got periph match type %d!!",
 					ccb.cdm.matches[unit].type);
 			free(ccb.cdm.matches);
-			return(-1);
+			return (-1);
 		}
 		periph_result = &ccb.cdm.matches[unit].result.periph_result;
 
-		js_snprintf(name, sizeof(name),
+		js_snprintf(name, sizeof (name),
 				"/dev/%s%d", periph_result->periph_name,
 			periph_result->unit_number);
 
@@ -809,12 +953,12 @@ scgo_close(scgp)
 		if (scglocal(scgp)->cam_devices[i] != (struct cam_device *)-1)
 			cam_close_device(scglocal(scgp)->cam_devices[i]);
 		scglocal(scgp)->cam_devices[i] = (struct cam_device *)-1;
-        }
+	}
 	return (0);
 }
 
-LOCAL
-long scgo_maxdma(scgp, amt)
+LOCAL long
+scgo_maxdma(scgp, amt)
 	SCSI	*scgp;
 	long	amt;
 {
@@ -847,6 +991,36 @@ scgo_freebuf(scgp)
 	scgp->bufbase = NULL;
 }
 
+LOCAL int
+scgo_numbus(scgp)
+	SCSI	*scgp;
+{
+	unsigned int	unit;
+	unsigned int	maxbus	= 0;
+	BOOL		found_bus = FALSE;
+
+	if (scgp->local == NULL)
+		return (0);
+
+	/*
+	 * There's a "cleaner" way to do this, using the matching code, but
+	 * it would involve more code than this solution...
+	 */
+	for (unit = 0; scglocal(scgp)->cam_devices[unit] != (struct cam_device *)-1; unit++) {
+		if (scglocal(scgp)->cam_devices[unit] == NULL)
+			continue;
+		found_bus = TRUE;
+		/*
+		 * path_id is unsigned, so maxbus cannot be initialized to -1.
+		 */
+		if (scglocal(scgp)->cam_devices[unit]->path_id > maxbus)
+			maxbus = scglocal(scgp)->cam_devices[unit]->path_id;
+	}
+	if (!found_bus)
+		return (0);
+	return (maxbus+1);
+}
+
 LOCAL BOOL
 scgo_havebus(scgp, busno)
 	SCSI	*scgp;
@@ -926,7 +1100,8 @@ scgo_send(scgp)
 	union ccb		ccb_space;
 	union ccb		*ccb = &ccb_space;
 	int			rv, result;
-	u_int32_t		ccb_flags;
+	u_int32_t		flags;
+	int			sense_len;
 
 	if (scgp->fd < 0) {
 #if 0
@@ -938,50 +1113,78 @@ scgo_send(scgp)
 	}
 
 	dev = scglocal(scgp)->cam_devices[scgp->fd];
-	fillbytes(&ccb->ccb_h, sizeof(struct ccb_hdr), '\0');
+	fillbytes(&ccb->ccb_h, sizeof (struct ccb_hdr), '\0');
 	ccb->ccb_h.path_id = dev->path_id;
 	ccb->ccb_h.target_id = dev->target_id;
 	ccb->ccb_h.target_lun = dev->target_lun;
 
 	/* Build the CCB */
-	fillbytes(&(&ccb->ccb_h)[1], sizeof(struct ccb_scsiio), '\0');
+	fillbytes(&(&ccb->ccb_h)[1], sizeof (struct ccb_scsiio), '\0');
 	movebytes(sp->cdb.cmd_cdb, &ccb->csio.cdb_io.cdb_bytes, sp->cdb_len);
 
 	/*
 	 * Set the data direction flags.
 	 */
 	if (sp->size != 0) {
-		ccb_flags = (sp->flags & SCG_RECV_DATA) ? CAM_DIR_IN :
-							   CAM_DIR_OUT;
+		flags = (sp->flags & SCG_RECV_DATA) ?   CAM_DIR_IN :
+							    CAM_DIR_OUT;
 	} else {
-		ccb_flags = CAM_DIR_NONE;
+		flags = CAM_DIR_NONE;
 	}
 
-	ccb_flags |= CAM_DEV_QFRZDIS;
+	flags |= CAM_DEV_QFRZDIS;
 
 	/*
 	 * If you don't want to bother with sending tagged commands under CAM,
 	 * we don't need to do anything to cdrecord.  If you want to send
 	 * tagged commands to those devices that support it, we'll need to set
 	 * the tag action valid field like this in scgo_send():
- 	 *         
-	 *        ccb_flags |= CAM_DEV_QFRZDIS | CAM_TAG_ACTION_VALID;
+	 *
+	 *	flags |= CAM_DEV_QFRZDIS | CAM_TAG_ACTION_VALID;
+	 *
+	 * Note that SSD_FULL_SIZE is 32 on FreeBSD and that it is impossible
+	 * to get more that SSD_FULL_SIZE == sizeof ((struct scsi_sense_data)
+	 * on FreeBSD.
 	 */
+	sense_len = sp->sense_len;
+	if (sense_len > SSD_FULL_SIZE)
+		sense_len = SSD_FULL_SIZE;
 
 	cam_fill_csio(&ccb->csio,
-		      /* retries */ 1,
-		      /* cbfncp */ NULL,
-		      /* flags */ ccb_flags,
-		      /* tag_action */ MSG_SIMPLE_Q_TAG,
-		      /* data_ptr */ (u_int8_t *)sp->addr,
-		      /* dxfer_len */ sp->size,
-		      /* sense_len */ SSD_FULL_SIZE,
-		      /* cdb_len */ sp->cdb_len,
-		      /* timeout */ sp->timeout * 1000);
+			/* retries */	1,
+			/* cbfncp */	NULL,
+			/* flags */	flags,
+			/* tag_action */ MSG_SIMPLE_Q_TAG,
+			/* data_ptr */	(u_int8_t *)sp->addr,
+			/* dxfer_len */	sp->size,
+			/* sense_len */	sense_len,
+			/* cdb_len */	sp->cdb_len,
+			/* timeout */	sp->timeout * 1000);
 
 	/* Run the command */
 	errno = 0;
-	if ((rv = cam_send_ccb(dev, ccb)) == -1) {
+	result = 16;
+	do {
+		/*
+		 * CAM_REQUEUE_REQ is a very unspecified status code. For this
+		 * reason, only the kernel could know why it happened.
+		 * This is therefore a loop that should be handled inside the
+		 * kernel in order not to break the layering model.
+		 * FreBSD unfortunately has dozens of places in the kernel that
+		 * set CAM_REQUEUE_REQ and most of the places that set
+		 * CAM_REQUEUE_REQ are handling missing kernel resources. As
+		 * it seems to be unlikely that there will be a clean fix in
+		 * the kernel soon, we are forced to handle this status here.
+		 *
+		 * Note that for May 2010, there is at least one definite bug
+		 * in the kernel where CAM_REQUEUE_REQ is returned for a SCSI
+		 * reservation conflict.
+		 */
+		rv = cam_send_ccb(dev, ccb);
+	} while (rv != -1 && --result > 0 &&
+		(ccb->ccb_h.status & CAM_STATUS_MASK) == CAM_REQUEUE_REQ);
+
+	if (rv == -1) {
 		return (rv);
 	} else {
 		/*
@@ -990,7 +1193,7 @@ scgo_send(scgp)
 		 * error.  If we completed successfully, there's (obviously)
 		 * no error.  We declare anything else "retryable".
 		 */
-		switch(ccb->ccb_h.status & CAM_STATUS_MASK) {
+		switch (ccb->ccb_h.status & CAM_STATUS_MASK) {
 			case CAM_SEL_TIMEOUT:
 				result = SCG_FATAL;
 				break;
@@ -1011,13 +1214,15 @@ scgo_send(scgp)
 		sp->ux_errno = EIO;
 
 	/* Pass the result back up */
-	fillbytes(&sp->scb, sizeof(sp->scb), '\0');
-	fillbytes(&sp->u_sense.cmd_sense, sizeof(sp->u_sense.cmd_sense), '\0');
+	fillbytes(&sp->scb, sizeof (sp->scb), '\0');
+#ifdef	CLEAR_SENSE
+	fillbytes(&sp->u_sense.cmd_sense, sizeof (sp->u_sense.cmd_sense), '\0');
+#endif
 	sp->resid = ccb->csio.resid;
-	sp->sense_count = SSD_FULL_SIZE - ccb->csio.sense_resid;
+	sp->sense_count = sense_len - ccb->csio.sense_resid;
 
 	/*
-	 * Determine how much room we have for sense data.
+	 * Determine how much room we have for sense data in struct scg_cmd.
 	 */
 	if (sp->sense_count > SCG_MAX_SENSE)
 		sp->sense_count = SCG_MAX_SENSE;
@@ -1032,6 +1237,6 @@ scgo_send(scgp)
 
 #undef scsi_sense
 #undef scsi_inquiry
-#define sense u_sense.Sense
+#define	sense u_sense.Sense
 
 #endif /* BSD_CAM */
