@@ -1,7 +1,7 @@
-/* @(#)scsitransp.c	1.86 03/05/03 Copyright 1988,1995,2000 J. Schilling */
+/* @(#)scsitransp.c	1.99 13/05/28 Copyright 1988,1995,2000-2013 J. Schilling */
 /*#ifndef lint*/
 static	char sccsid[] =
-	"@(#)scsitransp.c	1.86 03/05/03 Copyright 1988,1995,2000 J. Schilling";
+	"@(#)scsitransp.c	1.99 13/05/28 Copyright 1988,1995,2000-2013 J. Schilling";
 /*#endif*/
 /*
  *	SCSI user level command transport routines (generic part).
@@ -12,33 +12,37 @@ static	char sccsid[] =
  *	Choose your name instead of "schily" and make clear that the version
  *	string is related to a modified source.
  *
- *	Copyright (c) 1988,1995,2000 J. Schilling
+ *	Copyright (c) 1988,1995,2000-2013 J. Schilling
  */
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * The contents of this file are subject to the terms of the
+ * Common Development and Distribution License, Version 1.0 only
+ * (the "License").  You may not use this file except in compliance
+ * with the License.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * See the file CDDL.Schily.txt in this distribution for details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; see the file COPYING.  If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * The following exceptions apply:
+ * CDDL 3.6 needs to be replaced by: "You may create a Larger Work by
+ * combining Covered Software with other code if all other code is governed by
+ * the terms of a license that is OSI approved (see www.opensource.org) and
+ * you may distribute the Larger Work as a single product. In such a case,
+ * You must make sure the requirements of this License are fulfilled for
+ * the Covered Software."
+ *
+ * When distributing Covered Code, include this CDDL HEADER in each
+ * file and include the License file CDDL.Schily.txt from this distribution.
  */
 
-#include <mconfig.h>
-#include <stdio.h>
-#include <standard.h>
-#include <stdxlib.h>
-#include <unixstd.h>
-#include <errno.h>
-#include <timedefs.h>
-#include <strdefs.h>
-#include <schily.h>
+#include <schily/mconfig.h>
+#include <schily/stdio.h>
+#include <schily/standard.h>
+#include <schily/stdlib.h>
+#include <schily/unistd.h>
+#include <schily/errno.h>
+#include <schily/time.h>
+#include <schily/string.h>
+#include <schily/schily.h>
 
 #include <scg/scgcmd.h>
 #include <scg/scsireg.h>
@@ -52,7 +56,7 @@ static	char sccsid[] =
  *	Choose your name instead of "schily" and make clear that the version
  *	string is related to a modified source.
  */
-LOCAL	char	_scg_version[]		= "0.7";	/* The global libscg version	*/
+LOCAL	char	_scg_version[]		= "0.9";	/* The global libscg version	*/
 LOCAL	char	_scg_auth_schily[]	= "schily";	/* The author for this module	*/
 
 #define	DEFTIMEOUT	20	/* Default timeout for SCSI command transport */
@@ -60,6 +64,7 @@ LOCAL	char	_scg_auth_schily[]	= "schily";	/* The author for this module	*/
 EXPORT	char	*scg_version	__PR((SCSI *scgp, int what));
 EXPORT	int	scg__open	__PR((SCSI *scgp, char *device));
 EXPORT	int	scg__close	__PR((SCSI *scgp));
+EXPORT	int	scg_numbus	__PR((SCSI *scgp));
 EXPORT	BOOL	scg_havebus	__PR((SCSI *scgp, int));
 EXPORT	int	scg_initiator_id __PR((SCSI *scgp));
 EXPORT	int	scg_isatapi	__PR((SCSI *scgp));
@@ -159,6 +164,24 @@ scg__open(scgp, device)
 	scg_ops_t *ops;
 extern	scg_ops_t scg_std_ops;
 
+	/*
+	 * Begin restricted code for quality assurance.
+	 *
+	 * Warning: you are not allowed to modify the quality ensurance code below.
+	 *
+	 * This restiction is introduced because this way, I hope that people
+	 * contribute to the project instead of creating branches.
+	 */
+#if	!defined(IS_SCHILY_XCONFIG)
+	printf("\nWarning: This version of libscg has not been configured via the standard\n");
+	printf("autoconfiguration method of the Schily makefile system. There is a high risk\n");
+	printf("that the code is not configured correctly and for this reason will not behave\n");
+	printf("as expected.\n");
+#endif
+	/*
+	 * End restricted code for quality assurance.
+	 */
+
 	scgp->ops = &scg_std_ops;
 
 	if (device && strncmp(device, "REMOTE", 6) == 0) {
@@ -239,6 +262,16 @@ scg_freebuf(scgp)
 	scgp->bufptr = NULL;
 }
 
+/*
+ * Return the max. number of SCSI busses.
+ */
+EXPORT BOOL
+scg_numbus(scgp)
+	SCSI	*scgp;
+{
+	return (SCGO_NUMBUS(scgp));
+}
+
 /*
  * Check if 'busno' is a valid SCSI bus number.
  */
@@ -305,10 +338,10 @@ scg_yes(msg)
 
 	js_printf("%s", msg);
 	flush();
-	if (getline(okbuf, sizeof(okbuf)) == EOF)
+	if (getline(okbuf, sizeof (okbuf)) == EOF)
 		exit(EX_BAD);
-	if(streql(okbuf, "y") || streql(okbuf, "yes") ||
-	   streql(okbuf, "Y") || streql(okbuf, "YES"))
+	if (streql(okbuf, "y") || streql(okbuf, "yes") ||
+	    streql(okbuf, "Y") || streql(okbuf, "YES"))
 		return (TRUE);
 	else
 		return (FALSE);
@@ -340,7 +373,7 @@ EXPORT int
 scg_cmd(scgp)
 	SCSI	*scgp;
 {
-		 int		ret;
+		int		ret;
 	register struct	scg_cmd	*scmd = scgp->scmd;
 
 	/*
@@ -382,12 +415,6 @@ scg_cmd(scgp)
 	scgp->running = FALSE;
 	__scg_times(scgp);
 	if (ret < 0) {
-		/*
-		 * Old /dev/scg versions will not allow to access targets > 7.
-		 * Include a workaround to make this non fatal.
-		 */
-		if (scg_target(scgp) < 8 || geterrno() != EINVAL)
-			comerr("Cannot send SCSI cmd via ioctl\n");
 		if (scmd->ux_errno == 0)
 			scmd->ux_errno = geterrno();
 		if (scmd->error == SCG_NO_ERROR)
@@ -396,6 +423,36 @@ scg_cmd(scgp)
 			errmsg("ret < 0 errno: %d ux_errno: %d error: %d\n",
 					geterrno(), scmd->ux_errno, scmd->error);
 		}
+		if (scmd->ux_errno == EPERM && scgp->flags & SCGF_PERM_PRINT) {
+			char	errbuf[SCSI_ERRSTR_SIZE];
+			int	amt;
+
+			amt = scg__sprinterr(scgp, errbuf, sizeof (errbuf));
+			if (amt > 0) {
+				FILE	*f = scgp->errfile;
+
+				if (f == NULL)
+					f = stderr;
+				filewrite(f, errbuf, amt);
+				ferrmsgno(f, scmd->ux_errno,
+					"Cannot send SCSI cmd via ioctl.\n");
+				fflush(f);
+			}
+		}
+		/*
+		 * Old /dev/scg versions will not allow to access targets > 7.
+		 * Include a workaround to make this non fatal.
+		 */
+		if (scg_target(scgp) < 8 || scmd->ux_errno != EINVAL) {
+
+			if (scmd->ux_errno != EPERM ||
+			    (scgp->flags & SCGF_PERM_PRINT) == 0) {
+				errmsgno(scmd->ux_errno,
+					"Cannot send SCSI cmd via ioctl.\n");
+			}
+			if (scgp->flags & SCGF_PERM_EXIT)
+				comexit(scmd->ux_errno);
+		}
 	}
 
 	ret = scg_vtail(scgp);
@@ -443,7 +500,7 @@ scg_svhead(scgp, buf, maxcnt)
 	p += amt;
 	maxcnt -= amt;
 
-	amt= scg_sprintcdb(scgp, p, maxcnt);
+	amt = scg_sprintcdb(scgp, p, maxcnt);
 	if (amt < 0)
 		return (amt);
 	p += amt;
@@ -578,7 +635,7 @@ scg_cmd_err(scgp)
 {
 	register struct scg_cmd *cp = scgp->scmd;
 
-	if(cp->error != SCG_NO_ERROR ||
+	if (cp->error != SCG_NO_ERROR ||
 				cp->ux_errno != 0 ||
 				*(Uchar *)&cp->scb != 0 ||
 				cp->u_sense.cmd_sense[0] != 0)	/* Paranioa */
@@ -629,7 +686,7 @@ scg_fprinterr(scgp, f)
 	char	errbuf[SCSI_ERRSTR_SIZE];
 	int	amt;
 
-	amt = scg_sprinterr(scgp, errbuf, sizeof(errbuf));
+	amt = scg_sprinterr(scgp, errbuf, sizeof (errbuf));
 	if (amt > 0) {
 		filewrite(f, errbuf, amt);
 		fflush(f);
@@ -685,9 +742,9 @@ scg__sprinterr(scgp, buf, maxcnt)
 	int	maxcnt;
 {
 	register struct scg_cmd *cp = scgp->scmd;
-	register char	*err;
-		 char	*cmdname = "SCSI command name not set by caller";
-		 char	errbuf[64];
+	register char		*err;
+		char		*cmdname = "SCSI command name not set by caller";
+		char		errbuf[64];
 	register char		*p = buf;
 	register int		amt;
 
@@ -700,14 +757,14 @@ scg__sprinterr(scgp, buf, maxcnt)
 				 * We need to cast timeval->* to long because
 				 * of the broken sys/time.h in Linux.
 				 */
-	case SCG_TIMEOUT  :	js_snprintf(errbuf, sizeof(errbuf),
+	case SCG_TIMEOUT  :	js_snprintf(errbuf, sizeof (errbuf),
 					"cmd timeout after %ld.%03ld (%d) s",
 					(long)scgp->cmdstop->tv_sec,
 					(long)scgp->cmdstop->tv_usec/1000,
 								cp->timeout);
 				err = errbuf;
 				break;
-	default:		js_snprintf(errbuf, sizeof(errbuf),
+	default:		js_snprintf(errbuf, sizeof (errbuf),
 					"error: %d", cp->error);
 				err = errbuf;
 	}
@@ -947,7 +1004,7 @@ scg_printstatus(scgp)
 	char	errbuf[SCSI_ERRSTR_SIZE];
 	int	amt;
 
-	amt = scg_sprintstatus(scgp, errbuf, sizeof(errbuf));
+	amt = scg_sprintstatus(scgp, errbuf, sizeof (errbuf));
 	if (amt > 0) {
 		filewrite((FILE *)scgp->errfile, errbuf, amt);
 		fflush((FILE *)scgp->errfile);
@@ -964,8 +1021,8 @@ scg_sprintstatus(scgp, buf, maxcnt)
 	int	maxcnt;
 {
 	register struct scg_cmd *cp = scgp->scmd;
-		 char	*err;
-		 char	*err2 = "";
+		char	*err;
+		char	*err2 = "";
 	register char	*p = buf;
 	register int	amt;
 
@@ -1213,11 +1270,11 @@ scg_sense_key(scgp)
 	register struct scg_cmd *cp = scgp->scmd;
 	int	key = -1;
 
-	if(!scg_cmd_err(scgp))
+	if (!scg_cmd_err(scgp))
 		return (0);
 
 	if (cp->sense.code >= 0x70)
-		key = ((struct scsi_ext_sense*)&(cp->sense))->key;
+		key = ((struct scsi_ext_sense *)&(cp->sense))->key;
 	return (key);
 }
 
@@ -1231,11 +1288,11 @@ scg_sense_code(scgp)
 	register struct scg_cmd *cp = scgp->scmd;
 	int	code = -1;
 
-	if(!scg_cmd_err(scgp))
+	if (!scg_cmd_err(scgp))
 		return (0);
 
 	if (cp->sense.code >= 0x70)
-		code = ((struct scsi_ext_sense*)&(cp->sense))->sense_code;
+		code = ((struct scsi_ext_sense *)&(cp->sense))->sense_code;
 	else
 		code = cp->sense.code;
 	return (code);
@@ -1250,11 +1307,11 @@ scg_sense_qual(scgp)
 {
 	register struct scg_cmd *cp = scgp->scmd;
 
-	if(!scg_cmd_err(scgp))
+	if (!scg_cmd_err(scgp))
 		return (0);
 
 	if (cp->sense.code >= 0x70)
-		return (((struct scsi_ext_sense*)&(cp->sense))->qual_code);
+		return (((struct scsi_ext_sense *)&(cp->sense))->qual_code);
 	else
 		return (0);
 }
@@ -1287,7 +1344,8 @@ scg_fprintdev(f, ip)
 			js_fprintf(f, "unsupported ");
 			break;
 		default:
-			js_fprintf(f, "vendor specific %d ", ip->qualifier);
+			js_fprintf(f, "vendor specific %d ",
+						(int)ip->qualifier);
 		}
 	}
 	switch (ip->type) {
@@ -1307,7 +1365,7 @@ scg_fprintdev(f, ip)
 	case INQ_SCAN:
 		js_fprintf(f, "Scanner");	break;
 	case INQ_OMEM:
-		js_fprintf(f, "Optical Storage");break;
+		js_fprintf(f, "Optical Storage"); break;
 	case INQ_JUKE:
 		js_fprintf(f, "Juke Box");	break;
 	case INQ_COMM:
@@ -1319,19 +1377,19 @@ scg_fprintdev(f, ip)
 	case INQ_STARR:
 		js_fprintf(f, "Storage array");	break;
 	case INQ_ENCL:
-		js_fprintf(f, "Enclosure services");break;
+		js_fprintf(f, "Enclosure services"); break;
 	case INQ_SDAD:
-		js_fprintf(f, "Simple direct access");break;
+		js_fprintf(f, "Simple direct access"); break;
 	case INQ_OCRW:
-		js_fprintf(f, "Optical card r/w");break;
+		js_fprintf(f, "Optical card r/w"); break;
 	case INQ_BRIDGE:
-		js_fprintf(f, "Bridging expander");break;
+		js_fprintf(f, "Bridging expander"); break;
 	case INQ_OSD:
-		js_fprintf(f, "Object based storage");break;
+		js_fprintf(f, "Object based storage"); break;
 	case INQ_ADC:
-		js_fprintf(f, "Automation/Drive Interface");break;
+		js_fprintf(f, "Automation/Drive Interface"); break;
 	case INQ_WELLKNOWN:
-		js_fprintf(f, "Well known lun");break;
+		js_fprintf(f, "Well known lun"); break;
 
 	case INQ_NODEV:
 		if (ip->data_format >= 2) {
@@ -1342,7 +1400,8 @@ scg_fprintdev(f, ip)
 			break;
 		}
 	default:
-		js_fprintf(f, "unknown device type 0x%x", ip->type);
+		js_fprintf(f, "unknown device type 0x%x",
+						(int)ip->type);
 	}
 	js_fprintf(f, "\n");
 }
@@ -1357,7 +1416,7 @@ scg_printdev(ip)
 	scg_fprintdev(stdout, ip);
 }
 
-#include <vadefs.h>
+#include <schily/varargs.h>
 
 /*
  * print into the SCSI error buffer, adjust the next write pointer.
