@@ -1,4 +1,4 @@
-/* @(#)breakline.c	1.9 03/06/15 Copyright 1985, 1995-2003 J. Schilling */
+/* @(#)breakline.c	1.11 06/09/13 Copyright 1985, 1995-2003 J. Schilling */
 /*
  *	break a line pointed to by *buf into fields
  *	returns the number of tokens, the line was broken into (>= 1)
@@ -11,24 +11,20 @@
  *	Copyright (c) 1985, 1995-2003 J. Schilling
  */
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * The contents of this file are subject to the terms of the
+ * Common Development and Distribution License, Version 1.0 only
+ * (the "License").  You may not use this file except in compliance
+ * with the License.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * See the file CDDL.Schily.txt in this distribution for details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; see the file COPYING.  If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * When distributing Covered Code, include this CDDL HEADER in each
+ * file and include the License file CDDL.Schily.txt from this distribution.
  */
 
-#include <mconfig.h>
-#include <standard.h>
-#include <schily.h>
+#include <schily/mconfig.h>
+#include <schily/standard.h>
+#include <schily/schily.h>
 
 #ifdef	PROTOTYPES
 EXPORT int
@@ -52,6 +48,7 @@ breakline(buf, delim, array, len)
 
 	for (i = 0, found = 1; i < len; i++) {
 		for (dp = bp; *dp != '\0' && *dp != delim; dp++)
+			/* LINTED */
 			;
 
 		array[i] = bp;
