@@ -1,8 +1,8 @@
-/* @(#)fnmatch.c	8.15 10/05/08 2005-2010 J. Schilling from 8.2 (Berkeley) */
+/* @(#)fnmatch.c	8.20 15/07/06 2005-2015 J. Schilling from 8.2 (Berkeley) */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)fnmatch.c	8.15 10/05/08 2005-2010 J. Schilling from 8.2 (Berkeley)";
+	"@(#)fnmatch.c	8.20 15/07/06 2005-2015 J. Schilling from 8.2 (Berkeley)";
 #endif
 /*
  * Copyright (c) 1989, 1993, 1994
@@ -37,9 +37,9 @@ static	UConst char sccsid[] =
  */
 
 #if defined(LIBC_SCCS) && !defined(lint)
-static UConst char sccsid[] = "@(#)fnmatch.c	8.15 (Berkeley) 05/08/10";
+static UConst char sccsid[] = "@(#)fnmatch.c	8.20 (Berkeley) 07/06/15";
 #endif /* LIBC_SCCS and not lint */
-/*__FBSDID("$FreeBSD: src/lib/libc/gen/fnmatch.c,v 1.19 2010/04/16 22:29:24 jilles Exp $");*/
+/* "FBSD src/lib/libc/gen/fnmatch.c,v 1.19 2010/04/16 22:29:24 jilles Exp $" */
 
 /*
  * Function fnmatch() as specified in POSIX 1003.2-1992, section B.6.
@@ -58,12 +58,12 @@ static UConst char sccsid[] = "@(#)fnmatch.c	8.15 (Berkeley) 05/08/10";
  */
 
 #include <schily/mconfig.h>
-#if	!defined(HAVE_FNMATCH) || !defined(HAVE_FNMATCH_IGNORECASE)
 #include <schily/fnmatch.h>
 #include <schily/limits.h>
 #include <schily/string.h>
 #include <schily/wchar.h>
 #include <schily/wctype.h>
+#include <schily/libport.h>	/* Define missing prototypes */
 
 #define	EOS	'\0'
 
@@ -75,11 +75,35 @@ static int rangematch __PR((const char *, wchar_t, int, char **, mbstate_t *));
 static int fnmatch1 __PR((const char *, const char *, const char *, int,
 				mbstate_t, mbstate_t));
 
+#ifndef	HAVE_FNMATCH
+#undef	fnmatch
+
+/*
+ * The Cygwin compile environment incorrectly implements #pragma weak.
+ * The weak symbols are only defined as local symbols making it impossible
+ * to use them from outside the scope of this source file.
+ * A platform that allows linking with global symbols has HAVE_LINK_WEAK
+ * defined.
+ */
+#if defined(HAVE_PRAGMA_WEAK) && defined(HAVE_LINK_WEAK)
+#pragma	weak fnmatch =	js_fnmatch
+#else
 int
 fnmatch(pattern, string, flags)
 	const char	*pattern;
 	const char	*string;
 	int		flags;
+{
+	return (js_fnmatch(pattern, string, flags));
+}
+#endif
+#endif
+
+int
+js_fnmatch(pattern, string, flags)
+	const char	*pattern;
+	const char	*string;
+	int		flags;
 {
 	/*
 	 * SunPro C gives a warning if we do not initialize an object:
@@ -315,4 +339,3 @@ rangematch(pattern, test, flags, newp, patmbs)
 	*newp = (char *)pattern;
 	return (ok == negate ? RANGE_NOMATCH : RANGE_MATCH);
 }
-#endif /* !defined(HAVE_FNMATCH) || !defined(HAVE_FNMATCH_IGNORECASE) */
