@@ -1,8 +1,8 @@
-/* @(#)sic_nls.c	1.16 10/05/24 Copyright 2007-2010 J. Schilling */
+/* @(#)sic_nls.c	1.18 14/01/15 Copyright 2007-2014 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)sic_nls.c	1.16 10/05/24 Copyright 2007-2010 J. Schilling";
+	"@(#)sic_nls.c	1.18 14/01/15 Copyright 2007-2014 J. Schilling";
 #endif
 /*
  * This code reads translation files in the format used by
@@ -12,7 +12,7 @@ static	UConst char sccsid[] =
  * from single byte character sets to unicode.
  * We use this code on systems that do not provide the iconv() function.
  *
- * Copyright 2007-2010 J. Schilling
+ * Copyright 2007-2014 J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -21,6 +21,8 @@ static	UConst char sccsid[] =
  * with the License.
  *
  * See the file CDDL.Schily.txt in this distribution for details.
+ * A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  *
  * When distributing Covered Code, include this CDDL HEADER in each
  * file and include the License file CDDL.Schily.txt from this distribution.
@@ -30,6 +32,7 @@ static	UConst char sccsid[] =
 #include <schily/stdlib.h>
 #include <schily/string.h>
 #include <schily/libport.h>	/* For strdup() */
+#include <schily/unistd.h>	/* For R_OK	*/
 #include <schily/schily.h>
 #include <schily/dirent.h>
 #include <schily/siconv.h>
@@ -67,10 +70,12 @@ Error Table size too small
 #endif
 
 LOCAL UInt8_t	nullpage[TAB_SIZE] = { 0 };
+LOCAL char	*ins_base;
 
 LOCAL	siconvt_t	*insert_sic		__PR((siconvt_t *sip));
 LOCAL	int		remove_sic		__PR((siconvt_t *sip));
 EXPORT	siconvt_t	*sic_open		__PR((char *name));
+EXPORT	const char	*sic_base		__PR((void));
 EXPORT	int		sic_close		__PR((siconvt_t *sip));
 EXPORT	int		sic_list		__PR((FILE *f));
 LOCAL	void		freetbl			__PR((UInt8_t **uni2cs));
@@ -174,6 +179,24 @@ sic_open(charset)
 	return (create_sic(charset));
 }
 
+/*
+ * Open a new translation
+ */
+EXPORT const char *
+sic_base()
+{
+	if (ins_base == NULL) {
+		ins_base = searchfileinpath("lib/siconv/iso8859-1", R_OK,
+					SIP_PLAIN_FILE, NULL);
+		if (ins_base != NULL) {
+			int	len = strlen(ins_base);
+
+			ins_base[len - 9] = '\0';
+		}
+	}
+	return (ins_base);
+}
+
 /*
  * Close a translation
  */
@@ -215,7 +238,13 @@ sic_list(f)
 	struct dirent	*dp;
 	int		i = 0;
 
-	snprintf(path, sizeof (path), "%s/lib/siconv/", INS_BASE);
+	if (ins_base == NULL)
+		(void) sic_base();
+
+	if (ins_base != NULL)
+		snprintf(path, sizeof (path), "%s", ins_base);
+	else
+		snprintf(path, sizeof (path), "%s/lib/siconv/", INS_BASE);
 	if ((d = opendir(path)) == NULL)
 		return (-1);
 
@@ -258,18 +287,24 @@ pfopen(name)
 	char	*name;
 {
 	char	path[1024];
-	FILE	*f;
-
-	if ((f = fopen(name, "r")) != NULL)
-		return (f);
+	char	*p;
 
 	if (strchr(name, '/'))
-		return ((FILE *)NULL);
+		return (fopen(name, "r"));
 
+	if (ins_base == NULL)
+		(void) sic_base();
+
+	p = ins_base;
+	if (p != NULL) {
+		snprintf(path, sizeof (path), "%s%s", p, name);
+		return (fopen(path, "r"));
+	}
 	snprintf(path, sizeof (path), "%s/lib/siconv/%s", INS_BASE, name);
 	return (fopen(path, "r"));
 }
 
+
 /*
  * Create a new translation either from a file or from iconv_open()
  */
