@@ -1,18 +1,18 @@
-/* @(#)defaults.h	1.18 04/06/01 joerg */
+/* @(#)defaults.h	1.25 13/02/14 joerg */
 /*
  * Header file defaults.h - assorted default values for character strings in
  * the volume descriptor.
  *
- * Copyright (c) 1999-2004 J. Schilling
+ * Copyright (c) 1999-2013 J. Schilling
  */
 
 #define	PREPARER_DEFAULT 	NULL
 #define	PUBLISHER_DEFAULT	NULL
 #ifndef	APPID_DEFAULT
 #ifdef	APPLE_HYB
-#define	APPID_DEFAULT 		"MKISOFS ISO 9660/HFS FILESYSTEM BUILDER (C) 1993 E.YOUNGDALE (C) 1997 J.PEARSON/J.SCHILLING"
+#define	APPID_DEFAULT 		"MKISOFS ISO9660/HFS/UDF FILESYSTEM BUILDER (C) 1993 E.YOUNGDALE (C) 1997 J.PEARSON/J.SCHILLING"
 #else
-#define	APPID_DEFAULT 		"MKISOFS ISO 9660 FILESYSTEM BUILDER (C) 1993 E.YOUNGDALE (C) 1997 J.PEARSON/J.SCHILLING"
+#define	APPID_DEFAULT 		"MKISOFS ISO9660/UDF FILESYSTEM BUILDER (C) 1993 E.YOUNGDALE (C) 1997 J.PEARSON/J.SCHILLING"
 #endif /* APPLE_HYB */
 #endif
 #define	COPYRIGHT_DEFAULT 	NULL
@@ -59,14 +59,22 @@
 #define	SYSTEM_ID_DEFAULT	"Win32"
 #endif /* _WIN */
 
+#if	!defined(SYSTEM_ID_DEFAULT) && defined(__MINGW32__)
+#define	SYSTEM_ID_DEFAULT	"Win32/MinGW"
+#endif /* __MINGW32__ */
+
 #ifdef __EMX__
 #define	SYSTEM_ID_DEFAULT	"OS/2"
 #endif
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 #define	SYSTEM_ID_DEFAULT	"FreeBSD"
 #endif
 
+#ifdef __DragonFly__
+#define	SYSTEM_ID_DEFAULT	"DragonFly"
+#endif
+
 #ifdef __NetBSD__
 #define	SYSTEM_ID_DEFAULT	"NetBSD"
 #endif
@@ -97,6 +105,10 @@
 #define	SYSTEM_ID_DEFAULT	"BeOS"
 #endif
 
+#ifdef __HAIKU__
+#define	SYSTEM_ID_DEFAULT	"Haiku"
+#endif
+
 #ifdef __OS2
 #define	SYSTEM_ID_DEFAULT	"OS/2"
 #endif
@@ -121,6 +133,18 @@
 #define	SYSTEM_ID_DEFAULT	"DOS"
 #endif
 
+#ifdef	__MINT__
+#define	SYSTEM_ID_DEFAULT	"ATARI-MiNT"
+#endif
+
+#ifdef	__SYLLABLE__
+#define	SYSTEM_ID_DEFAULT	"Syllable"
+#endif
+
+#ifdef	AMIGA
+#define	SYSTEM_ID_DEFAULT	"AMIGA"
+#endif
+
 #ifndef SYSTEM_ID_DEFAULT
 #define	SYSTEM_ID_DEFAULT	"UNIX"
 #endif
