@@ -1,7 +1,8 @@
-/* @(#)dump.c	1.21 04/09/08 joerg */
+/* @(#)dump.c	1.39 12/12/02 joerg */
+#include <schily/mconfig.h>
 #ifndef lint
-static	char sccsid[] =
-	"@(#)dump.c	1.21 04/09/08 joerg";
+static	UConst char sccsid[] =
+	"@(#)dump.c	1.39 12/12/02 joerg";
 #endif
 /*
  * File dump.c - dump a file/device both in hex and in ASCII.
@@ -9,37 +10,37 @@ static	char sccsid[] =
  * Written by Eric Youngdale (1993).
  *
  * Copyright 1993 Yggdrasil Computing, Incorporated
- * Copyright (c) 1999-2004 J. Schilling
+ * Copyright (c) 1999-2012 J. Schilling
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along with
+ * this program; see the file COPYING.  If not, write to the Free Software
+ * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <mconfig.h>
-#include <stdxlib.h>
-#include <unixstd.h>
-#include <strdefs.h>
-#include <utypes.h>
+#include <schily/stdlib.h>
+#include <schily/unistd.h>
+#include <schily/string.h>
+#include <schily/utypes.h>
 
-#include <stdio.h>
-#include <standard.h>
-#include <ttydefs.h>
-#include <signal.h>
-#include <schily.h>
+#include <schily/stdio.h>
+#include <schily/standard.h>
+#include <schily/termios.h>
+#include <schily/signal.h>
+#include <schily/schily.h>
+#include <schily/nlsdefs.h>
 
 #include "../scsi.h"
-#include "../../cdrecord/defaults.h"
+#include "cdrdeflt.h"
+#include "../../cdrecord/version.h"
 
 /*
  * Note: always use these macros to avoid problems.
@@ -55,6 +56,7 @@ static	char sccsid[] =
 
 #define	infile	in_image
 EXPORT	FILE		*infile = NULL;
+EXPORT	BOOL		ignerr = FALSE;
 LOCAL	off_t		file_addr;
 LOCAL	off_t		sec_addr = (off_t)-1;
 LOCAL	Uchar		sector[2048];
@@ -66,9 +68,11 @@ LOCAL	Uchar		search[64];
 LOCAL	struct sgttyb	savetty;
 LOCAL	struct sgttyb	newtty;
 #else
+#ifdef	USE_TERMIOS
 LOCAL	struct termios	savetty;
 LOCAL	struct termios	newtty;
 #endif
+#endif
 
 LOCAL void	reset_tty	__PR((void));
 LOCAL void	set_tty		__PR((void));
@@ -86,18 +90,18 @@ reset_tty()
 #ifdef USE_V7_TTY
 	if (ioctl(STDIN_FILENO, TIOCSETN, &savetty) == -1) {
 #else
-#ifdef TCSANOW
+#ifdef	USE_TERMIOS
+#ifdef	TCSANOW
 	if (tcsetattr(STDIN_FILENO, TCSANOW, &savetty) == -1) {
 #else
 	if (ioctl(STDIN_FILENO, TCSETAF, &savetty) == -1) {
 #endif
+#else	/* USE_TERMIOS */
+	if (0) {
+#endif	/* USE_TERMIOS */
 #endif
-#ifdef	USE_LIBSCHILY
-		comerr("Cannot put tty into normal mode\n");
-#else
-		printf("Cannot put tty into normal mode\n");
+		printf(_("Cannot put tty into normal mode\n"));
 		exit(1);
-#endif
 	}
 }
 
@@ -107,18 +111,18 @@ set_tty()
 #ifdef USE_V7_TTY
 	if (ioctl(STDIN_FILENO, TIOCSETN, &newtty) == -1) {
 #else
-#ifdef TCSANOW
+#ifdef	USE_TERMIOS
+#ifdef	TCSANOW
 	if (tcsetattr(STDIN_FILENO, TCSANOW, &newtty) == -1) {
 #else
 	if (ioctl(STDIN_FILENO, TCSETAF, &newtty) == -1) {
 #endif
+#else	/* USE_TERMIOS */
+	if (0) {
+#endif	/* USE_TERMIOS */
 #endif
-#ifdef	USE_LIBSCHILY
-		comerr("Cannot put tty into raw mode\n");
-#else
-		printf("Cannot put tty into raw mode\n");
+		printf(_("Cannot put tty into raw mode\n"));
 		exit(1);
-#endif
 	}
 }
 
@@ -211,10 +215,10 @@ showblock(flag)
 	}
 	crsr2(20, 1);
 	if (sizeof (file_addr) > sizeof (long)) {
-		printf(" Zone, zone offset: %14llx %12.12llx  ",
+		printf(_(" Zone, zone offset: %14llx %12.12llx  "),
 			(Llong)file_addr>>11, (Llong)file_addr & 0x7ff);
 	} else {
-		printf(" Zone, zone offset: %6lx %4.4lx  ",
+		printf(_(" Zone, zone offset: %6lx %4.4lx  "),
 			(long)(file_addr>>11), (long)(file_addr & 0x7ff));
 	}
 	fflush(stdout);
@@ -236,15 +240,16 @@ LOCAL void
 usage(excode)
 	int	excode;
 {
-	errmsgno(EX_BAD, "Usage: %s [options] [image]\n",
+	errmsgno(EX_BAD, _("Usage: %s [options] [image]\n"),
 						get_progname());
 
-	error("Options:\n");
-	error("\t-help, -h	Print this help\n");
-	error("\t-version	Print version info and exit\n");
-	error("\t-i filename	Filename to read ISO-9660 image from\n");
-	error("\tdev=target	SCSI target to use as CD/DVD-Recorder\n");
-	error("\nIf neither -i nor dev= are speficied, <image> is needed.\n");
+	error(_("Options:\n"));
+	error(_("\t-help, -h	Print this help\n"));
+	error(_("\t-version	Print version info and exit\n"));
+	error(_("\t-inore-error	Ignore errors\n"));
+	error(_("\t-i filename	Filename to read ISO-9660 image from\n"));
+	error(_("\tdev=target	SCSI target to use as CD/DVD-Recorder\n"));
+	error(_("\nIf neither -i nor dev= are speficied, <image> is needed.\n"));
 	exit(excode);
 }
 
@@ -255,64 +260,84 @@ main(argc, argv)
 {
 	int	cac;
 	char	* const *cav;
-	char	*opts = "help,h,version,i*,dev*";
+	char	*opts = "help,h,version,ignore-error,i*,dev*";
 	BOOL	help = FALSE;
 	BOOL	prvers = FALSE;
 	char	*filename = NULL;
-	char	*devname = NULL;
+	char	*sdevname = NULL;
+#if	defined(USE_NLS)
+	char	*dir;
+#endif
 	char	c;
 	int	i;
 	int	j;
 
 	save_args(argc, argv);
 
+#if	defined(USE_NLS)
+	setlocale(LC_ALL, "");
+#if !defined(TEXT_DOMAIN)	/* Should be defined by cc -D */
+#define	TEXT_DOMAIN "isoinfo"	/* Use this only if it weren't */
+#endif
+	dir = searchfileinpath("share/locale", F_OK,
+					SIP_ANY_FILE|SIP_NO_PATH, NULL);
+	if (dir)
+		(void) bindtextdomain(TEXT_DOMAIN, dir);
+	else
+#if defined(PROTOTYPES) && defined(INS_BASE)
+	(void) bindtextdomain(TEXT_DOMAIN, INS_BASE "/share/locale");
+#else
+	(void) bindtextdomain(TEXT_DOMAIN, "/usr/share/locale");
+#endif
+	(void) textdomain(TEXT_DOMAIN);
+#endif
+
 	cac = argc - 1;
 	cav = argv + 1;
-	if (getallargs(&cac, &cav, opts, &help, &help, &prvers,
-			&filename, &devname) < 0) {
-		errmsgno(EX_BAD, "Bad Option: '%s'\n", cav[0]);
+	if (getallargs(&cac, &cav, opts, &help, &help, &prvers, &ignerr,
+			&filename, &sdevname) < 0) {
+		errmsgno(EX_BAD, _("Bad Option: '%s'\n"), cav[0]);
 		usage(EX_BAD);
 	}
 	if (help)
 		usage(0);
 	if (prvers) {
-		printf("devdump %s (%s-%s-%s)\n", "2.01",
-					HOST_CPU, HOST_VENDOR, HOST_OS);
+		printf(_("devdump %s (%s-%s-%s) Copyright (C) 1993-1999 %s (C) 1999-2012 %s\n"),
+					VERSION,
+					HOST_CPU, HOST_VENDOR, HOST_OS,
+					_("Eric Youngdale"),
+					_("Joerg Schilling"));
 		exit(0);
 	}
 	cac = argc - 1;
 	cav = argv + 1;
-	if (filename == NULL && devname == NULL) {
+	if (filename == NULL && sdevname == NULL) {
 		if (getfiles(&cac, &cav, opts) != 0) {
 			filename = cav[0];
 			cac--, cav++;
 		}
 	}
 	if (getfiles(&cac, &cav, opts) != 0) {
-		errmsgno(EX_BAD, "Bad Argument: '%s'\n", cav[0]);
+		errmsgno(EX_BAD, _("Bad Argument: '%s'\n"), cav[0]);
 		usage(EX_BAD);
 	}
-	if (filename != NULL && devname != NULL) {
-		errmsgno(EX_BAD, "Only one of -i or dev= allowed\n");
+	if (filename != NULL && sdevname != NULL) {
+		errmsgno(EX_BAD, _("Only one of -i or dev= allowed\n"));
 		usage(EX_BAD);
 	}
 #ifdef	USE_SCG
-	if (filename == NULL && devname == NULL)
-		cdr_defaults(&devname, NULL, NULL, NULL);
-#endif
-	if (filename == NULL && devname == NULL) {
-#ifdef	USE_LIBSCHILY
-		errmsgno(EX_BAD, "ISO-9660 image not specified\n");
-#else
-		fprintf(stderr, "ISO-9660 image not specified\n");
+	if (filename == NULL && sdevname == NULL)
+		cdr_defaults(&sdevname, NULL, NULL, NULL, NULL);
 #endif
+	if (filename == NULL && sdevname == NULL) {
+		fprintf(stderr, _("ISO-9660 image not specified\n"));
 		usage(EX_BAD);
 	}
 
 	if (filename != NULL)
 		infile = fopen(filename, "rb");
 	else
-		filename = devname;
+		filename = sdevname;
 
 	if (infile != NULL) {
 		/* EMPTY */;
@@ -321,12 +346,8 @@ main(argc, argv)
 #else
 	} else {
 #endif
-#ifdef	USE_LIBSCHILY
-		comerr("Cannot open '%s'\n", filename);
-#else
-		fprintf(stderr, "Cannot open '%s'\n", filename);
+		fprintf(stderr, _("Cannot open '%s'\n"), filename);
 		exit(1);
-#endif
 	}
 
 	for (i = 0; i < 30; i++)
@@ -339,27 +360,30 @@ main(argc, argv)
 #ifdef USE_V7_TTY
 	if (ioctl(STDIN_FILENO, TIOCGETP, &savetty) == -1) {
 #else
-#ifdef TCSANOW
+#ifdef	USE_TERMIOS
+#ifdef	TCSANOW
 	if (tcgetattr(STDIN_FILENO, &savetty) == -1) {
 #else
 	if (ioctl(STDIN_FILENO, TCGETA, &savetty) == -1) {
 #endif
+#else	/* USE_TERMIOS */
+	if (0) {
+#endif	/* USE_TERMIOS */
 #endif
-#ifdef	USE_LIBSCHILY
-		comerr("Stdin must be a tty\n");
-#else
-		printf("Stdin must be a tty\n");
+		printf(_("Stdin must be a tty\n"));
 		exit(1);
-#endif
 	}
-	newtty = savetty;
 #ifdef USE_V7_TTY
+	newtty = savetty;
 	newtty.sg_flags  &= ~(ECHO|CRMOD);
 	newtty.sg_flags  |= CBREAK;
 #else
+#ifdef	USE_TERMIOS
+	newtty = savetty;
 	newtty.c_lflag   &= ~ICANON;
 	newtty.c_lflag   &= ~ECHO;
 	newtty.c_cc[VMIN] = 1;
+#endif
 #endif
 	set_tty();
 #ifdef	SIGTSTP
@@ -370,14 +394,18 @@ main(argc, argv)
 	do {
 		if (file_addr < (off_t)0) file_addr = (off_t)0;
 		showblock(1);
+#ifdef	USE_GETCH
+		c = getch();	/* DOS console input */
+#else
 		read(STDIN_FILENO, &c, 1);
+#endif
 		if (c == 'a')
 			file_addr -= PAGE;
 		if (c == 'b')
 			file_addr += PAGE;
 		if (c == 'g') {
 			crsr2(20, 1);
-			printf("Enter new starting block (in hex):");
+			printf(_("Enter new starting block (in hex):"));
 			if (sizeof (file_addr) > sizeof (long)) {
 				Llong	ll;
 				scanf("%llx", &ll);
@@ -393,7 +421,7 @@ main(argc, argv)
 		}
 		if (c == 'f') {
 			crsr2(20, 1);
-			printf("Enter new search string:");
+			printf(_("Enter new search string:"));
 			fgets((char *)search, sizeof (search), stdin);
 			while (search[strlen((char *)search)-1] == '\n')
 				search[strlen((char *)search)-1] = 0;
@@ -424,6 +452,7 @@ main(argc, argv)
 			break;
 	} while (1 == 1);
 	reset_tty();
-	fclose(infile);
+	if (infile != NULL)
+		fclose(infile);
 	return (0);
 }
