@@ -1,7 +1,8 @@
-/* @(#)isodump.c	1.24 04/09/08 joerg */
+/* @(#)isodump.c	1.49 15/02/25 joerg */
+#include <schily/mconfig.h>
 #ifndef lint
-static	char sccsid[] =
-	"@(#)isodump.c	1.24 04/09/08 joerg";
+static	UConst char sccsid[] =
+	"@(#)isodump.c	1.49 15/02/25 joerg";
 #endif
 /*
  * File isodump.c - dump iso9660 directory information.
@@ -10,37 +11,39 @@ static	char sccsid[] =
  * Written by Eric Youngdale (1993).
  *
  * Copyright 1993 Yggdrasil Computing, Incorporated
- * Copyright (c) 1999-2004 J. Schilling
+ * Copyright (c) 1999-2015 J. Schilling
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along with
+ * this program; see the file COPYING.  If not, write to the Free Software
+ * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <mconfig.h>
-#include <stdxlib.h>
-#include <unixstd.h>
-#include <strdefs.h>
-#include <utypes.h>
+#include <schily/stdlib.h>
+#include <schily/unistd.h>
+#include <schily/string.h>
+#include <schily/utypes.h>
 
-#include <stdio.h>
-#include <standard.h>
-#include <ttydefs.h>
-#include <signal.h>
-#include <schily.h>
+#include <schily/stdio.h>
+#include <schily/standard.h>
+#include <schily/termios.h>
+#include <schily/signal.h>
+#include <schily/schily.h>
+#include <schily/nlsdefs.h>
 
+#include "../iso9660.h"
+#include "../rock.h"
 #include "../scsi.h"
-#include "../../cdrecord/defaults.h"
+#include "cdrdeflt.h"
+#include "../../cdrecord/version.h"
 
 /*
  * XXX JS: Some structures have odd lengths!
@@ -67,76 +70,29 @@ static	char sccsid[] =
 #define	ISO_BLOCKS(X)	(((X) / SECTOR_SIZE) + (((X)%SECTOR_SIZE)?1:0))
 
 #define	infile	in_image
-FILE	*infile = NULL;
-off_t	file_addr;
-unsigned char buffer[2048];
-unsigned char search[64];
-int blocksize;
+EXPORT FILE	*infile = NULL;
+EXPORT BOOL	ignerr = FALSE;
+LOCAL off_t	file_addr;
+LOCAL int	su_version;
+LOCAL int	rr_version;
+LOCAL int	use_rock = TRUE;
+LOCAL unsigned char buffer[2048];
+LOCAL unsigned char search[64];
+LOCAL int blocksize;
 
 #define	PAGE	sizeof (buffer)
 
-#define	ISODCL(from, to)	(to - from + 1)
-
-struct iso_primary_descriptor {
-	unsigned char type			[ISODCL(1,   1)]; /* 711 */
-	unsigned char id			[ISODCL(2,   6)];
-	unsigned char version			[ISODCL(7,   7)]; /* 711 */
-	unsigned char unused1			[ISODCL(8,   8)];
-	unsigned char system_id			[ISODCL(9,   40)]; /* aunsigned chars */
-	unsigned char volume_id			[ISODCL(41,  72)]; /* dunsigned chars */
-	unsigned char unused2			[ISODCL(73,  80)];
-	unsigned char volume_space_size		[ISODCL(81,  88)]; /* 733 */
-	unsigned char unused3			[ISODCL(89,  120)];
-	unsigned char volume_set_size		[ISODCL(121, 124)]; /* 723 */
-	unsigned char volume_sequence_number	[ISODCL(125, 128)]; /* 723 */
-	unsigned char logical_block_size	[ISODCL(129, 132)]; /* 723 */
-	unsigned char path_table_size		[ISODCL(133, 140)]; /* 733 */
-	unsigned char type_l_path_table		[ISODCL(141, 144)]; /* 731 */
-	unsigned char opt_type_l_path_table	[ISODCL(145, 148)]; /* 731 */
-	unsigned char type_m_path_table		[ISODCL(149, 152)]; /* 732 */
-	unsigned char opt_type_m_path_table	[ISODCL(153, 156)]; /* 732 */
-	unsigned char root_directory_record	[ISODCL(157, 190)]; /* 9.1 */
-	unsigned char volume_set_id		[ISODCL(191, 318)]; /* dunsigned chars */
-	unsigned char publisher_id		[ISODCL(319, 446)]; /* achars */
-	unsigned char preparer_id		[ISODCL(447, 574)]; /* achars */
-	unsigned char application_id		[ISODCL(575, 702)]; /* achars */
-	unsigned char copyright_file_id		[ISODCL(703, 739)]; /* 7.5 dchars */
-	unsigned char abstract_file_id		[ISODCL(740, 776)]; /* 7.5 dchars */
-	unsigned char bibliographic_file_id	[ISODCL(777, 813)]; /* 7.5 dchars */
-	unsigned char creation_date		[ISODCL(814, 830)]; /* 8.4.26.1 */
-	unsigned char modification_date		[ISODCL(831, 847)]; /* 8.4.26.1 */
-	unsigned char expiration_date		[ISODCL(848, 864)]; /* 8.4.26.1 */
-	unsigned char effective_date		[ISODCL(865, 881)]; /* 8.4.26.1 */
-	unsigned char file_structure_version	[ISODCL(882, 882)]; /* 711 */
-	unsigned char unused4			[ISODCL(883, 883)];
-	unsigned char application_data		[ISODCL(884, 1395)];
-	unsigned char unused5			[ISODCL(1396, 2048)];
-};
-
-struct iso_directory_record {
-	unsigned char length			[ISODCL(1, 1)]; /* 711 */
-	unsigned char ext_attr_length		[ISODCL(2, 2)]; /* 711 */
-	unsigned char extent			[ISODCL(3, 10)]; /* 733 */
-	unsigned char size			[ISODCL(11, 18)]; /* 733 */
-	unsigned char date			[ISODCL(19, 25)]; /* 7 by 711 */
-	unsigned char flags			[ISODCL(26, 26)];
-	unsigned char file_unit_size		[ISODCL(27, 27)]; /* 711 */
-	unsigned char interleave		[ISODCL(28, 28)]; /* 711 */
-	unsigned char volume_sequence_number	[ISODCL(29, 32)]; /* 723 */
-	unsigned char name_len			[ISODCL(33, 33)]; /* 711 */
-	unsigned char name			[1];
-};
-
 LOCAL int	isonum_731	__PR((char * p));
 LOCAL int	isonum_721	__PR((char * p));
 LOCAL int	isonum_723	__PR((char * p));
-LOCAL int	isonum_733	__PR((unsigned char * p));
+LOCAL int	isonum_733	__PR((char * p));
 LOCAL void	reset_tty	__PR((void));
 LOCAL void	set_tty		__PR((void));
 LOCAL void	onsusp		__PR((int signo));
 LOCAL void	crsr2		__PR((int row, int col));
 LOCAL int	parse_rr	__PR((unsigned char * pnt, int len, int cont_flag));
-LOCAL void	dump_rr		__PR((struct iso_directory_record * idr));
+LOCAL void	find_rr		__PR((struct iso_directory_record * idr, Uchar **pntp, int *lenp));
+LOCAL int	dump_rr		__PR((struct iso_directory_record * idr));
 LOCAL void	showblock	__PR((int flag));
 LOCAL int	getbyte		__PR((void));
 LOCAL void	usage		__PR((int excode));
@@ -165,12 +121,8 @@ isonum_723(p)
 {
 #if 0
 	if (p[0] != p[3] || p[1] != p[2]) {
-#ifdef	USE_LIBSCHILY
-		comerrno(EX_BAD, "invalid format 7.2.3 number\n");
-#else
 		fprintf(stderr, "invalid format 7.2.3 number\n");
 		exit(1);
-#endif
 	}
 #endif
 	return (isonum_721(p));
@@ -179,18 +131,20 @@ isonum_723(p)
 
 LOCAL int
 isonum_733(p)
-	unsigned char *p;
+	char *p;
 {
-	return (isonum_731((char *)p));
+	return (isonum_731(p));
 }
 
 #ifdef	USE_V7_TTY
 LOCAL	struct sgttyb	savetty;
 LOCAL	struct sgttyb	newtty;
 #else
+#ifdef	USE_TERMIOS
 LOCAL	struct termios savetty;
 LOCAL	struct termios newtty;
 #endif
+#endif
 
 LOCAL void
 reset_tty()
@@ -198,18 +152,18 @@ reset_tty()
 #ifdef USE_V7_TTY
 	if (ioctl(STDIN_FILENO, TIOCSETN, &savetty) == -1) {
 #else
-#ifdef TCSANOW
-	if (tcsetattr(STDIN_FILENO, TCSANOW, &savetty) == -1)  {
+#ifdef	USE_TERMIOS
+#ifdef	TCSANOW
+	if (tcsetattr(STDIN_FILENO, TCSANOW, &savetty) == -1) {
 #else
 	if (ioctl(STDIN_FILENO, TCSETAF, &savetty) == -1) {
 #endif
+#else	/* USE_TERMIOS */
+	if (0) {
+#endif	/* USE_TERMIOS */
 #endif
-#ifdef	USE_LIBSCHILY
-		comerr("Cannot put tty into normal mode\n");
-#else
-		printf("Cannot put tty into normal mode\n");
+		printf(_("Cannot put tty into normal mode\n"));
 		exit(1);
-#endif
 	}
 }
 
@@ -219,18 +173,18 @@ set_tty()
 #ifdef USE_V7_TTY
 	if (ioctl(STDIN_FILENO, TIOCSETN, &newtty) == -1) {
 #else
-#ifdef TCSANOW
+#ifdef	USE_TERMIOS
+#ifdef	TCSANOW
 	if (tcsetattr(STDIN_FILENO, TCSANOW, &newtty) == -1) {
 #else
 	if (ioctl(STDIN_FILENO, TCSETAF, &newtty) == -1) {
 #endif
+#else	/* USE_TERMIOS */
+	if (0) {
+#endif	/* USE_TERMIOS */
 #endif
-#ifdef	USE_LIBSCHILY
-		comerr("Cannot put tty into raw mode\n");
-#else
-		printf("Cannot put tty into raw mode\n");
+		printf(_("Cannot put tty into raw mode\n"));
 		exit(1);
-#endif
 	}
 }
 
@@ -287,7 +241,7 @@ parse_rr(pnt, len, cont_flag)
 	unsigned char	*pnts;
 	char		symlinkname[1024];
 	char		name[1024];
-	int		goof;
+	int		goof = 0;
 
 /*	printf(" RRlen=%d ", len); */
 
@@ -297,7 +251,8 @@ parse_rr(pnt, len, cont_flag)
 	cont_offset = cont_size = 0;
 
 	ncount = 0;
-	flag1 = flag2 = 0;
+	flag1 = -1;
+	flag2 = 0;
 	while (len >= 4) {
 		if (ncount)
 			printf(",");
@@ -305,16 +260,16 @@ parse_rr(pnt, len, cont_flag)
 			printf("[");
 		printf("%c%c", pnt[0], pnt[1]);
 		if (pnt[3] != 1 && pnt[3] != 2) {
-			printf("**BAD RRVERSION (%d) for %c%c\n", pnt[3], pnt[0], pnt[1]);
+			printf(_("**BAD RRVERSION (%d) for %c%c\n"), pnt[3], pnt[0], pnt[1]);
 			return (0);	/* JS ??? Is this right ??? */
 		} else if (pnt[0] == 'R' && pnt[1] == 'R') {
-			printf("=%d", pnt[3]);
+			printf("=%d", pnt[3]);			/* RR version */
 		}
 		ncount++;
 		if (pnt[0] == 'R' && pnt[1] == 'R') flag1 = pnt[4] & 0xff;
-		if (strncmp((char *)pnt, "PX", 2) == 0) flag2 |= 1;
-		if (strncmp((char *)pnt, "PN", 2) == 0) flag2 |= 2;
-		if (strncmp((char *)pnt, "SL", 2) == 0) flag2 |= 4;
+		if (strncmp((char *)pnt, "PX", 2) == 0) flag2 |= RR_FLAG_PX;
+		if (strncmp((char *)pnt, "PN", 2) == 0) flag2 |= RR_FLAG_PN;
+		if (strncmp((char *)pnt, "SL", 2) == 0) flag2 |= RR_FLAG_SL;
 		if (strncmp((char *)pnt, "NM", 2) == 0) {
 			slen = pnt[2] - 5;
 			pnts = pnt+5;
@@ -324,35 +279,57 @@ parse_rr(pnt, len, cont_flag)
 			memset(name, 0, sizeof (name));
 			memcpy(name, pnts, slen);
 			printf("=%s", name);
-			flag2 |= 8;
+			flag2 |= RR_FLAG_NM;
 		}
-		if (strncmp((char *)pnt, "CL", 2) == 0) flag2 |= 16;
-		if (strncmp((char *)pnt, "PL", 2) == 0) flag2 |= 32;
-		if (strncmp((char *)pnt, "RE", 2) == 0) flag2 |= 64;
-		if (strncmp((char *)pnt, "TF", 2) == 0) flag2 |= 128;
+		if (strncmp((char *)pnt, "CL", 2) == 0) flag2 |= RR_FLAG_CL;
+		if (strncmp((char *)pnt, "PL", 2) == 0) flag2 |= RR_FLAG_PL;
+		if (strncmp((char *)pnt, "RE", 2) == 0) flag2 |= RR_FLAG_RE;
+		if (strncmp((char *)pnt, "TF", 2) == 0) flag2 |= RR_FLAG_TF;
 
 		if (strncmp((char *)pnt, "PX", 2) == 0) {
-			extent = isonum_733(pnt+12);
+			extent = isonum_733((char *)pnt+12);	/* Link count */
 			printf("=%x", extent);
 		}
 
 		if (strncmp((char *)pnt, "CE", 2) == 0) {
-			cont_extent = (off_t)isonum_733(pnt+4);
-			cont_offset = isonum_733(pnt+12);
-			cont_size = isonum_733(pnt+20);
+			cont_extent = (off_t)isonum_733((char *)pnt+4);
+			cont_offset = isonum_733((char *)pnt+12);
+			cont_size = isonum_733((char *)pnt+20);
 			printf("=[%x,%x,%d]", (int)cont_extent, cont_offset,
 								cont_size);
 		}
 
+		if (strncmp((char *)pnt, "ER", 2) == 0) {		/* ER */
+			int	lid = pnt[4] & 0xFF;			/* Len ID  */
+			int	ldes = pnt[5] & 0xFF;			/* Len des */
+			int	lsrc = pnt[6] & 0xFF;			/* Len src */
+			int	xver = pnt[7] & 0xFF;			/* X vers  */
+			flag2 |= RR_FLAG_ER;				/* ER record */
+
+			rr_version = xver;
+			printf(_("=[len_id=%d,len_des=%d,len_src=%d,ext_ver=%d,id=\"%.*s\"]"),
+				lid, ldes, lsrc, xver, lid, &pnt[8]);
+
+		}
+		if (strncmp((char *)pnt, "SP", 2) == 0) {		/* SUSP */
+			flag2 |= RR_FLAG_SP;				/* SUSP record */
+			su_version = pnt[3] & 0xff;
+			printf(_("=[skip=%d]"), pnt[6] & 0xFF);		/* SUSP skip off */
+
+		}
+		if (strncmp((char *)pnt, "ST", 2) == 0) {		/* Terminate SUSP */
+			break;
+		}
+
 		if (strncmp((char *)pnt, "PL", 2) == 0 || strncmp((char *)pnt, "CL", 2) == 0) {
-			extent = isonum_733(pnt+4);
-			printf("=%x", extent);
+			extent = isonum_733((char *)pnt+4);
+			printf("=%x", extent);				/* DIR extent */
 		}
 
 		if (strncmp((char *)pnt, "SL", 2) == 0) {
 			int	cflag;
 
-			cflag = pnt[4];
+			cflag = pnt[4];					/* Component flag */
 			pnts = pnt+5;
 			slen = pnt[2] - 5;
 			while (slen >= 1) {
@@ -372,19 +349,19 @@ parse_rr(pnt, len, cont_flag)
 					break;
 				case 16:
 					strcat(symlinkname, "/mnt");
-					printf("Warning - mount point requested");
+					printf(_("Warning - mount point requested"));
 					break;
 				case 32:
 					strcat(symlinkname, "kafka");
-					printf("Warning - host_name requested");
+					printf(_("Warning - host_name requested"));
 					break;
 				default:
-					printf("Reserved bit setting in symlink");
+					printf(_("Reserved bit setting in symlink"));
 					goof++;
 					break;
 				}
 				if ((pnts[0] & 0xfe) && pnts[1] != 0) {
-					printf("Incorrect length in symlink component");
+					printf(_("Incorrect length in symlink component"));
 				}
 				if ((pnts[0] & 1) == 0)
 					strcat(symlinkname, "/");
@@ -400,37 +377,44 @@ parse_rr(pnt, len, cont_flag)
 
 		len -= pnt[2];
 		pnt += pnt[2];
-		if (len <= 3 && cont_extent) {
-			unsigned char sector[2048];
+	}
+	if (cont_extent) {
+		unsigned char sector[2048];
 
 #ifdef	USE_SCG
-			readsecs(cont_extent * blocksize / 2048, sector, ISO_BLOCKS(sizeof (sector)));
+		readsecs(cont_extent * blocksize / 2048, sector, ISO_BLOCKS(sizeof (sector)));
 #else
-			lseek(fileno(infile), cont_extent * blocksize, SEEK_SET);
-			read(fileno(infile), sector, sizeof (sector));
+		lseek(fileno(infile), cont_extent * blocksize, SEEK_SET);
+		read(fileno(infile), sector, sizeof (sector));
 #endif
-			flag2 |= parse_rr(&sector[cont_offset], cont_size, 1);
-		}
+		flag2 |= parse_rr(&sector[cont_offset], cont_size, 1);
 	}
 	if (ncount)
 		printf("]");
-	if (!cont_flag && flag1 != flag2) {
-		printf("Flag %x != %x", flag1, flag2);
+	if (!cont_flag && flag1 != -1 && flag1 != (flag2 & 0xFF)) {
+		printf(_("Flag %x != %x"), flag1, flag2);
 		goof++;
 	}
+	/*
+	 * XXX Check goof?
+	 */
 	return (flag2);
 }
 
 LOCAL void
-dump_rr(idr)
+find_rr(idr, pntp, lenp)
 	struct iso_directory_record *idr;
+	Uchar	**pntp;
+	int	*lenp;
 {
-	int		len;
-	unsigned char	*pnt;
+	struct iso_xa_dir_record *xadp;
+	int len;
+	unsigned char * pnt;
 
 	len = idr->length[0] & 0xff;
 	len -= offsetof(struct iso_directory_record, name[0]);
 	len -= idr->name_len[0];
+
 	pnt = (unsigned char *) idr;
 	pnt += offsetof(struct iso_directory_record, name[0]);
 	pnt += idr->name_len[0];
@@ -438,7 +422,28 @@ dump_rr(idr)
 		pnt++;
 		len--;
 	}
-	parse_rr(pnt, len, 0);
+	if (len >= 14) {
+		xadp = (struct iso_xa_dir_record *)pnt;
+
+		if (xadp->signature[0] == 'X' && xadp->signature[1] == 'A' &&
+		    xadp->reserved[0] == '\0') {
+			len -= 14;
+			pnt += 14;
+		}
+	}
+	*pntp = pnt;
+	*lenp = len;
+}
+
+LOCAL int
+dump_rr(idr)
+	struct iso_directory_record *idr;
+{
+	int		len;
+	unsigned char	*pnt;
+
+	find_rr(idr, &pnt, &len);
+	return (parse_rr(pnt, len, 0));
 }
 
 
@@ -481,7 +486,8 @@ showblock(flag)
 				for (j = 0; j < (int)idr->name_len[0]; j++) printf("%c", idr->name[j]);
 				for (j = 0; j < (14 - (int)idr->name_len[0]); j++) printf(" ");
 			}
-			dump_rr(idr);
+			if (use_rock)
+				dump_rr(idr);
 			printf("\n");
 			i += buffer[i];
 			if (i > 2048 - offsetof(struct iso_directory_record, name[0]))
@@ -491,11 +497,11 @@ showblock(flag)
 	}
 	printf("\n");
 	if (sizeof (file_addr) > sizeof (long)) {
-		printf(" Zone, zone offset: %14llx %12.12llx  ",
+		printf(_(" Zone, zone offset: %14llx %12.12llx  "),
 			(Llong)file_addr / blocksize,
 			(Llong)file_addr & (Llong)(blocksize - 1));
 	} else {
-		printf(" Zone, zone offset: %6lx %4.4lx  ",
+		printf(_(" Zone, zone offset: %6lx %4.4lx  "),
 			(long) (file_addr / blocksize),
 			(long) file_addr & (blocksize - 1));
 	}
@@ -518,15 +524,16 @@ LOCAL void
 usage(excode)
 	int	excode;
 {
-	errmsgno(EX_BAD, "Usage: %s [options] image\n",
+	errmsgno(EX_BAD, _("Usage: %s [options] image\n"),
 						get_progname());
 
-	error("Options:\n");
-	error("\t-help, -h	Print this help\n");
-	error("\t-version	Print version info and exit\n");
-	error("\t-i filename	Filename to read ISO-9660 image from\n");
-	error("\tdev=target	SCSI target to use as CD/DVD-Recorder\n");
-	error("\nIf neither -i nor dev= are speficied, <image> is needed.\n");
+	error(_("Options:\n"));
+	error(_("\t-help, -h	Print this help\n"));
+	error(_("\t-version	Print version info and exit\n"));
+	error(_("\t-inore-error	Ignore errors\n"));
+	error(_("\t-i filename	Filename to read ISO-9660 image from\n"));
+	error(_("\tdev=target	SCSI target to use as CD/DVD-Recorder\n"));
+	error(_("\nIf neither -i nor dev= are speficied, <image> is needed.\n"));
 	exit(excode);
 }
 
@@ -537,11 +544,14 @@ main(argc, argv)
 {
 	int	cac;
 	char	* const *cav;
-	char	*opts = "help,h,version,i*,dev*";
+	char	*opts = "help,h,version,ignore-error,i*,dev*";
 	BOOL	help = FALSE;
 	BOOL	prvers = FALSE;
 	char	*filename = NULL;
-	char	*devname = NULL;
+	char	*sdevname = NULL;
+#if	defined(USE_NLS)
+	char	*dir;
+#endif
 	char	c;
 	int	i;
 	struct iso_primary_descriptor	ipd;
@@ -549,53 +559,70 @@ main(argc, argv)
 
 	save_args(argc, argv);
 
+#if	defined(USE_NLS)
+	setlocale(LC_ALL, "");
+#if !defined(TEXT_DOMAIN)	/* Should be defined by cc -D */
+#define	TEXT_DOMAIN "isoinfo"	/* Use this only if it weren't */
+#endif
+	dir = searchfileinpath("share/locale", F_OK,
+					SIP_ANY_FILE|SIP_NO_PATH, NULL);
+	if (dir)
+		(void) bindtextdomain(TEXT_DOMAIN, dir);
+	else
+#if defined(PROTOTYPES) && defined(INS_BASE)
+	(void) bindtextdomain(TEXT_DOMAIN, INS_BASE "/share/locale");
+#else
+	(void) bindtextdomain(TEXT_DOMAIN, "/usr/share/locale");
+#endif
+	(void) textdomain(TEXT_DOMAIN);
+#endif
+
 	cac = argc - 1;
 	cav = argv + 1;
-	if (getallargs(&cac, &cav, opts, &help, &help, &prvers,
-			&filename, &devname) < 0) {
-		errmsgno(EX_BAD, "Bad Option: '%s'\n", cav[0]);
+	if (getallargs(&cac, &cav, opts, &help, &help, &prvers, &ignerr,
+			&filename, &sdevname) < 0) {
+		errmsgno(EX_BAD, _("Bad Option: '%s'\n"), cav[0]);
 		usage(EX_BAD);
 	}
 	if (help)
 		usage(0);
 	if (prvers) {
-		printf("isodump %s (%s-%s-%s)\n", "2.01",
-					HOST_CPU, HOST_VENDOR, HOST_OS);
+		printf(_("isodump %s (%s-%s-%s) Copyright (C) 1993-1999 %s (C) 1999-2015 %s\n"),
+					VERSION,
+					HOST_CPU, HOST_VENDOR, HOST_OS,
+					_("Eric Youngdale"),
+					_("Joerg Schilling"));
 		exit(0);
 	}
 	cac = argc - 1;
 	cav = argv + 1;
-	if (filename == NULL && devname == NULL) {
+	if (filename == NULL && sdevname == NULL) {
 		if (getfiles(&cac, &cav, opts) != 0) {
 			filename = cav[0];
 			cac--, cav++;
 		}
 	}
 	if (getfiles(&cac, &cav, opts) != 0) {
-		errmsgno(EX_BAD, "Bad Argument: '%s'\n", cav[0]);
+		errmsgno(EX_BAD, _("Bad Argument: '%s'\n"), cav[0]);
 		usage(EX_BAD);
 	}
-	if (filename != NULL && devname != NULL) {
-		errmsgno(EX_BAD, "Only one of -i or dev= allowed\n");
+	if (filename != NULL && sdevname != NULL) {
+		errmsgno(EX_BAD, _("Only one of -i or dev= allowed\n"));
 		usage(EX_BAD);
 	}
 #ifdef	USE_SCG
-	if (filename == NULL && devname == NULL)
-		cdr_defaults(&devname, NULL, NULL, NULL);
-#endif
-	if (filename == NULL && devname == NULL) {
-#ifdef	USE_LIBSCHILY
-		errmsgno(EX_BAD, "ISO-9660 image not specified\n");
-#else
-		fprintf(stderr, "ISO-9660 image not specified\n");
+	if (filename == NULL && sdevname == NULL)
+		cdr_defaults(&sdevname, NULL, NULL, NULL, NULL);
 #endif
+	if (filename == NULL && sdevname == NULL) {
+		fprintf(stderr, _("ISO-9660 image not specified\n"));
 		usage(EX_BAD);
 	}
 
 	if (filename != NULL)
 		infile = fopen(filename, "rb");
 	else
-		filename = devname;
+		filename = sdevname;
 
 	if (infile != NULL) {
 		/* EMPTY */;
@@ -604,12 +631,8 @@ main(argc, argv)
 #else
 	} else {
 #endif
-#ifdef	USE_LIBSCHILY
-		comerr("Cannot open '%s'\n", filename);
-#else
-		fprintf(stderr, "Cannot open '%s'\n", filename);
+		fprintf(stderr, _("Cannot open '%s'\n"), filename);
 		exit(1);
-#endif
 	}
 
 	file_addr = (off_t) (16 << 11);
@@ -629,31 +652,46 @@ main(argc, argv)
 	file_addr = (off_t)isonum_733(idr->extent);
 	file_addr = file_addr * blocksize;
 
+#ifdef	USE_SCG
+	readsecs(file_addr / 2048, buffer, ISO_BLOCKS(sizeof (buffer)));
+#else
+	lseek(fileno(infile), file_addr, SEEK_SET);
+	read(fileno(infile), buffer, sizeof (buffer));
+#endif
+	i = dump_rr((struct iso_directory_record *) buffer);
+	if (i == 0 ||
+	    (i & (RR_FLAG_SP | RR_FLAG_ER)) == 0 || su_version < 1 || rr_version < 1) {
+		use_rock = FALSE;
+	}
+
 /* Now setup the keyboard for single character input. */
 #ifdef USE_V7_TTY
 	if (ioctl(STDIN_FILENO, TIOCGETP, &savetty) == -1) {
 #else
-#ifdef TCSANOW
+#ifdef	USE_TERMIOS
+#ifdef	TCSANOW
 	if (tcgetattr(STDIN_FILENO, &savetty) == -1) {
 #else
 	if (ioctl(STDIN_FILENO, TCGETA, &savetty) == -1) {
 #endif
+#else	/* USE_TERMIOS */
+	if (0) {
+#endif	/* USE_TERMIOS */
 #endif
-#ifdef	USE_LIBSCHILY
-		comerr("Stdin must be a tty\n");
-#else
-		printf("Stdin must be a tty\n");
+		printf(_("Stdin must be a tty\n"));
 		exit(1);
-#endif
 	}
-	newtty = savetty;
 #ifdef USE_V7_TTY
+	newtty = savetty;
 	newtty.sg_flags  &= ~(ECHO|CRMOD);
 	newtty.sg_flags  |= CBREAK;
 #else
+#ifdef	USE_TERMIOS
+	newtty = savetty;
 	newtty.c_lflag   &= ~ICANON;
 	newtty.c_lflag   &= ~ECHO;
 	newtty.c_cc[VMIN] = 1;
+#endif
 #endif
 	set_tty();
 #ifdef	SIGTSTP
@@ -665,14 +703,18 @@ main(argc, argv)
 		if (file_addr < 0)
 			file_addr = (off_t)0;
 		showblock(1);
+#ifdef	USE_GETCH
+		c = getch();	/* DOS console input */
+#else
 		read(STDIN_FILENO, &c, 1);
+#endif
 		if (c == 'a')
 			file_addr -= blocksize;
 		if (c == 'b')
 			file_addr += blocksize;
 		if (c == 'g') {
 			crsr2(20, 1);
-			printf("Enter new starting block (in hex):");
+			printf(_("Enter new starting block (in hex):"));
 			if (sizeof (file_addr) > sizeof (long)) {
 				Llong	ll;
 				scanf("%llx", &ll);
@@ -688,7 +730,7 @@ main(argc, argv)
 		}
 		if (c == 'f') {
 			crsr2(20, 1);
-			printf("Enter new search string:");
+			printf(_("Enter new search string:"));
 			fgets((char *)search, sizeof (search), stdin);
 			while (search[strlen((char *)search)-1] == '\n')
 				search[strlen((char *)search)-1] = 0;
@@ -719,6 +761,7 @@ main(argc, argv)
 			break;
 	} while (1 == 1);
 	reset_tty();
-	fclose(infile);
+	if (infile != NULL)
+		fclose(infile);
 	return (0);
 }
