@@ -1,8 +1,8 @@
-/* @(#)isovfy.c	1.43 10/05/24 joerg */
+/* @(#)isovfy.c	1.46 15/01/27 joerg */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)isovfy.c	1.43 10/05/24 joerg";
+	"@(#)isovfy.c	1.46 15/01/27 joerg";
 #endif
 /*
  * File isovfy.c - verify consistency of iso9660 filesystem.
@@ -11,7 +11,7 @@ static	UConst char sccsid[] =
  * Written by Eric Youngdale (1993).
  *
  * Copyright 1993 Yggdrasil Computing, Incorporated
- * Copyright (c) 1999-2010 J. Schilling
+ * Copyright (c) 1999-2015 J. Schilling
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2
@@ -37,6 +37,7 @@ static	UConst char sccsid[] =
 #include <schily/standard.h>
 #include <schily/signal.h>
 #include <schily/schily.h>
+#include <schily/nlsdefs.h>
 
 #include "../iso9660.h"
 #include "../rock.h"
@@ -69,8 +70,13 @@ static	UConst char sccsid[] =
 #define	ISO_BLOCKS(X)	(((X) / SECTOR_SIZE) + (((X)%SECTOR_SIZE)?1:0))
 
 #define	infile	in_image
-FILE *infile = NULL;
-int blocksize;
+EXPORT FILE *infile = NULL;
+EXPORT BOOL ignerr = FALSE;
+LOCAL int	su_version;
+LOCAL int	rr_version;
+LOCAL int	aa_version;
+LOCAL int	use_rock = TRUE;
+LOCAL int	blocksize;
 
 #define	PAGE	sizeof (buffer)
 
@@ -200,7 +206,7 @@ parse_rr(pnt, len, cont_flag)
 		iline += strlen(lbuffer + iline);
 		if (pnt[0] < 'A' || pnt[0] > 'Z' || pnt[1] < 'A' ||
 		    pnt[1] > 'Z') {
-			sprintf(lbuffer+iline, "**BAD SUSP %d %d]",
+			sprintf(lbuffer+iline, _("**BAD SUSP %d %d]"),
 					pnt[0], pnt[1]);
 			rr_goof++;
 			iline += strlen(lbuffer + iline);
@@ -208,7 +214,7 @@ parse_rr(pnt, len, cont_flag)
 		}
 
 		if (pnt[3] != 1 && pnt[3] != 2) {
-			sprintf(lbuffer+iline, "**BAD RRVERSION (%d)\n", pnt[3]);
+			sprintf(lbuffer+iline, _("**BAD RRVERSION (%d)\n"), pnt[3]);
 			rr_goof++;
 			iline += strlen(lbuffer + iline);
 			return (flag2);
@@ -224,6 +230,20 @@ parse_rr(pnt, len, cont_flag)
 		if (strncmp((char *)pnt, "RE", 2) == 0) flag2 |= RR_FLAG_RE;
 		if (strncmp((char *)pnt, "TF", 2) == 0) flag2 |= RR_FLAG_TF;
 
+		if (strncmp((char *)pnt, "SP", 2) == 0) {
+			flag2 |= RR_FLAG_SP;					/* SUSP record */
+			su_version = pnt[3] & 0xff;
+		}
+		if (strncmp((char *)pnt, "AA", 2) == 0) {			/* Neither SUSP nor RR */
+			flag2 |= RR_FLAG_AA;					/* Apple Signature record */
+			aa_version = pnt[3] & 0xff;
+		}
+		if (strncmp((char *)pnt, "ER", 2) == 0) {
+			flag2 |= RR_FLAG_ER;					/* ER record */
+			rr_version = pnt[7] & 0xff;				/* Ext Version */
+/*			strlcpy(er_id, (char *)&pnt[8], (pnt[4] & 0xFF) + 1);*/
+		}
+
 		if (strncmp((char *)pnt, "CE", 2) == 0) {
 			cont_extent = (off_t)isonum_733((char *)pnt+4);
 			cont_offset = isonum_733((char *)pnt+12);
@@ -263,22 +283,22 @@ parse_rr(pnt, len, cont_flag)
 					break;
 				case 16:
 					strcat(symlinkname, "/mnt");
-					sprintf(lbuffer+iline, "Warning - mount point requested");
+					sprintf(lbuffer+iline, _("Warning - mount point requested"));
 					iline += strlen(lbuffer + iline);
 					break;
 				case 32:
 					strcat(symlinkname, "kafka");
-					sprintf(lbuffer+iline, "Warning - host_name requested");
+					sprintf(lbuffer+iline, _("Warning - host_name requested"));
 					iline += strlen(lbuffer + iline);
 					break;
 				default:
-					sprintf(lbuffer+iline, "Reserved bit setting in symlink");
+					sprintf(lbuffer+iline, _("Reserved bit setting in symlink"));
 					rr_goof++;
 					iline += strlen(lbuffer + iline);
 					break;
 				}
 				if ((pnts[0] & 0xfe) && pnts[1] != 0) {
-					sprintf(lbuffer+iline, "Incorrect length in symlink component");
+					sprintf(lbuffer+iline, _("Incorrect length in symlink component"));
 					iline += strlen(lbuffer + iline);
 				}
 				if ((pnts[0] & 1) == 0)
@@ -311,7 +331,7 @@ parse_rr(pnt, len, cont_flag)
 		iline += strlen(lbuffer + iline);
 	}
 	if (!cont_flag && flag1 != -1 && flag1 != (flag2 & 0xFF)) {
-		sprintf(lbuffer+iline, "Flag %x != %x", flag1, flag2);
+		sprintf(lbuffer+iline, _("Flag %x != %x"), flag1, flag2);
 		rr_goof++;
 		iline += strlen(lbuffer + iline);
 	}
@@ -361,8 +381,7 @@ dump_rr(idr)
 
 	rr_goof = 0;
 	find_rr(idr, &pnt, &len);
-	parse_rr(pnt, len, 0);
-	return (rr_goof);
+	return (parse_rr(pnt, len, 0));
 }
 
 
@@ -410,7 +429,7 @@ check_tree(file_addr, file_size, parent_addr)
 	dir_size_count += file_size / blocksize;
 
 	if (file_size & 0x3ff)
-		printf("********Directory has unusual size\n");
+		printf(_("********Directory has unusual size\n"));
 
 	for (k = 0; k < (file_size / sizeof (buffer)); k++) {
 #ifdef	USE_SCG
@@ -436,7 +455,7 @@ check_tree(file_addr, file_size, parent_addr)
 			iline += strlen(lbuffer + iline);
 
 			if (idr->name_len[0] > 33) {
-				sprintf(&lbuffer[iline], "File name length=(%d)",
+				sprintf(&lbuffer[iline], _("File name length=(%d)"),
 							idr->name_len[0]);
 				goof++;
 				iline += strlen(lbuffer + iline);
@@ -444,14 +463,18 @@ check_tree(file_addr, file_size, parent_addr)
 				sprintf(&lbuffer[iline], ".             ");
 				iline += strlen(lbuffer + iline);
 				rflag = 0;
+#ifdef	Eric_seems_to_be_wrong
 				if (orig_file_addr != (off_t)(isonum_733(idr->extent) +
 							isonum_711((char *) idr->ext_attr_length))) {
-					sprintf(&lbuffer[iline], "***** Directory has null extent.");
+#else
+				if (orig_file_addr != (off_t)isonum_733(idr->extent)) {
+#endif
+					sprintf(&lbuffer[iline], _("***** Directory has null extent."));
 					goof++;
 					iline += strlen(lbuffer + iline);
 				}
 				if (i1) {
-					sprintf(&lbuffer[iline], "***** . not  first entry.");
+					sprintf(&lbuffer[iline], _("***** . not  first entry."));
 					rr_goof++;
 					iline += strlen(lbuffer + iline);
 				}
@@ -459,20 +482,24 @@ check_tree(file_addr, file_size, parent_addr)
 				sprintf(&lbuffer[iline], "..            ");
 				iline += strlen(lbuffer + iline);
 				rflag = 0;
+#ifdef	Eric_seems_to_be_wrong
 				if (parent_file_addr != (off_t)(isonum_733(idr->extent) +
 							isonum_711((char *) idr->ext_attr_length))) {
-					sprintf(&lbuffer[iline], "***** Directory has null extent.");
+#else
+				if (parent_file_addr != (off_t)isonum_733(idr->extent)) {
+#endif
+					sprintf(&lbuffer[iline], _("***** Directory has null extent."));
 					goof++;
 					iline += strlen(lbuffer + iline);
 				}
 				if (i1 != 1) {
-					sprintf(&lbuffer[iline], "***** .. not second entry.");
+					sprintf(&lbuffer[iline], _("***** .. not second entry."));
 					rr_goof++;
 					iline += strlen(lbuffer + iline);
 				}
 			} else {
 				if (i1 < 2) {
-					sprintf(&lbuffer[iline], " Improper sorting.");
+					sprintf(&lbuffer[iline], _(" Improper sorting."));
 					rr_goof++;
 				}
 				for (j = 0; j < (int)idr->name_len[0]; j++) {
@@ -486,43 +513,46 @@ check_tree(file_addr, file_size, parent_addr)
 			}
 
 			if (size && extent == 0) {
-				sprintf(&lbuffer[iline], "****Extent==0, size != 0");
+				sprintf(&lbuffer[iline], _("****Extent==0, size != 0"));
 				goof++;
 				iline += strlen(lbuffer + iline);
 			}
 #if 0
 			/* This is apparently legal. */
 			if (size == 0 && extent) {
-				sprintf(&lbuffer[iline], "****Extent!=0, size == 0");
+				sprintf(&lbuffer[iline], _("****Extent!=0, size == 0"));
 				goof++;
 				iline += strlen(lbuffer + iline);
 			}
 #endif
 
 			if (idr->flags[0] & 0xf5) {
-				sprintf(&lbuffer[iline], "Flags=(%x) ", idr->flags[0]);
+				sprintf(&lbuffer[iline], _("Flags=(%x) "), idr->flags[0]);
 				goof++;
 				iline += strlen(lbuffer + iline);
 			}
 			if (idr->interleave[0]) {
-				sprintf(&lbuffer[iline], "Interleave=(%d) ", idr->interleave[0]);
+				sprintf(&lbuffer[iline], _("Interleave=(%d) "), idr->interleave[0]);
 				goof++;
 				iline += strlen(lbuffer + iline);
 			}
 
 			if (idr->file_unit_size[0]) {
-				sprintf(&lbuffer[iline], "File unit size=(%d) ", idr->file_unit_size[0]);
+				sprintf(&lbuffer[iline], _("File unit size=(%d) "), idr->file_unit_size[0]);
 				goof++;
 				iline += strlen(lbuffer + iline);
 			}
 
 			if (idr->volume_sequence_number[0] != 1) {
-				sprintf(&lbuffer[iline], "Volume sequence number=(%d) ", idr->volume_sequence_number[0]);
+				sprintf(&lbuffer[iline], _("Volume sequence number=(%d) "), idr->volume_sequence_number[0]);
 				goof++;
 				iline += strlen(lbuffer + iline);
 			}
 
-			goof += dump_rr(idr);
+			if (use_rock) {
+				dump_rr(idr);
+				goof += rr_goof;
+			}
 			sprintf(&lbuffer[iline], "\n");
 			iline += strlen(lbuffer + iline);
 
@@ -540,9 +570,15 @@ check_tree(file_addr, file_size, parent_addr)
 
 
 			if (rflag && (idr->flags[0] & 2))
+#ifdef	Eric_seems_to_be_wrong
 				check_tree((off_t)(isonum_733(idr->extent) + isonum_711((char *)idr->ext_attr_length)) * blocksize,
 						isonum_733(idr->size),
 						orig_file_addr * blocksize);
+#else
+				check_tree((off_t)isonum_733(idr->extent) * blocksize,
+						isonum_733(idr->size),
+						orig_file_addr * blocksize);
+#endif
 			i += buffer[i];
 			i1++;
 			if (i > 2048 - offsetof(struct iso_directory_record, name[0]))
@@ -653,15 +689,16 @@ LOCAL void
 usage(excode)
 	int	excode;
 {
-	errmsgno(EX_BAD, "Usage: %s [options] image\n",
+	errmsgno(EX_BAD, _("Usage: %s [options] image\n"),
 						get_progname());
 
-	error("Options:\n");
-	error("\t-help, -h	Print this help\n");
-	error("\t-version	Print version info and exit\n");
-	error("\t-i filename	Filename to read ISO-9660 image from\n");
-	error("\tdev=target	SCSI target to use as CD/DVD-Recorder\n");
-	error("\nIf neither -i nor dev= are speficied, <image> is needed.\n");
+	error(_("Options:\n"));
+	error(_("\t-help, -h	Print this help\n"));
+	error(_("\t-version	Print version info and exit\n"));
+	error(_("\t-inore-error	Ignore errors\n"));
+	error(_("\t-i filename	Filename to read ISO-9660 image from\n"));
+	error(_("\tdev=target	SCSI target to use as CD/DVD-Recorder\n"));
+	error(_("\nIf neither -i nor dev= are speficied, <image> is needed.\n"));
 	exit(excode);
 }
 
@@ -672,11 +709,15 @@ main(argc, argv)
 {
 	int	cac;
 	char	* const *cav;
-	char	*opts = "help,h,version,i*,dev*";
+	int	c;
+	char	*opts = "help,h,version,ignore-error,i*,dev*";
 	BOOL	help = FALSE;
 	BOOL	prvers = FALSE;
 	char	*filename = NULL;
 	char	*sdevname = NULL;
+#if	defined(USE_NLS)
+	char	*dir;
+#endif
 	off_t	file_addr;
 	int	file_size;
 	struct iso_primary_descriptor	ipd;
@@ -684,22 +725,43 @@ main(argc, argv)
 	int	typel_extent;
 	int	typem_extent;
 	int	path_table_size;
+	char	buffer[2048];
 
 	save_args(argc, argv);
 
+#if	defined(USE_NLS)
+	setlocale(LC_ALL, "");
+#if !defined(TEXT_DOMAIN)	/* Should be defined by cc -D */
+#define	TEXT_DOMAIN "isoinfo"	/* Use this only if it weren't */
+#endif
+	dir = searchfileinpath("share/locale", F_OK,
+					SIP_ANY_FILE|SIP_NO_PATH, NULL);
+	if (dir)
+		(void) bindtextdomain(TEXT_DOMAIN, dir);
+	else
+#if defined(PROTOTYPES) && defined(INS_BASE)
+	(void) bindtextdomain(TEXT_DOMAIN, INS_BASE "/share/locale");
+#else
+	(void) bindtextdomain(TEXT_DOMAIN, "/usr/share/locale");
+#endif
+	(void) textdomain(TEXT_DOMAIN);
+#endif
+
 	cac = argc - 1;
 	cav = argv + 1;
-	if (getallargs(&cac, &cav, opts, &help, &help, &prvers,
+	if (getallargs(&cac, &cav, opts, &help, &help, &prvers, &ignerr,
 			&filename, &sdevname) < 0) {
-		errmsgno(EX_BAD, "Bad Option: '%s'\n", cav[0]);
+		errmsgno(EX_BAD, _("Bad Option: '%s'\n"), cav[0]);
 		usage(EX_BAD);
 	}
 	if (help)
 		usage(0);
 	if (prvers) {
-		printf("isovfy %s (%s-%s-%s) Copyright (C) 1993-1999 Eric Youngdale (C) 1999-2010 Jrg Schilling\n",
+		printf(_("isovfy %s (%s-%s-%s) Copyright (C) 1993-1999 %s (C) 1999-2015 %s\n"),
 					VERSION,
-					HOST_CPU, HOST_VENDOR, HOST_OS);
+					HOST_CPU, HOST_VENDOR, HOST_OS,
+					_("Eric Youngdale"),
+					_("Joerg Schilling"));
 		exit(0);
 	}
 	cac = argc - 1;
@@ -711,11 +773,11 @@ main(argc, argv)
 		}
 	}
 	if (getfiles(&cac, &cav, opts) != 0) {
-		errmsgno(EX_BAD, "Bad Argument: '%s'\n", cav[0]);
+		errmsgno(EX_BAD, _("Bad Argument: '%s'\n"), cav[0]);
 		usage(EX_BAD);
 	}
 	if (filename != NULL && sdevname != NULL) {
-		errmsgno(EX_BAD, "Only one of -i or dev= allowed\n");
+		errmsgno(EX_BAD, _("Only one of -i or dev= allowed\n"));
 		usage(EX_BAD);
 	}
 #ifdef	USE_SCG
@@ -723,7 +785,7 @@ main(argc, argv)
 		cdr_defaults(&sdevname, NULL, NULL, NULL, NULL);
 #endif
 	if (filename == NULL && sdevname == NULL) {
-		fprintf(stderr, "ISO-9660 image not specified\n");
+		fprintf(stderr, _("ISO-9660 image not specified\n"));
 		usage(EX_BAD);
 	}
 
@@ -739,7 +801,7 @@ main(argc, argv)
 #else
 	} else {
 #endif
-		fprintf(stderr, "Cannot open '%s'\n", filename);
+		fprintf(stderr, _("Cannot open '%s'\n"), filename);
 		exit(1);
 	}
 
@@ -759,13 +821,29 @@ main(argc, argv)
 		blocksize = 2048;
 	}
 
+#ifdef	Eric_seems_to_be_wrong
 	file_addr = (off_t)isonum_733(idr->extent) + isonum_711((char *)idr->ext_attr_length);
+#else
+	file_addr = (off_t)isonum_733(idr->extent);
+#endif
 	file_size = isonum_733(idr->size);
 
+#ifdef	USE_SCG
+	readsecs(file_addr, buffer, ISO_BLOCKS(sizeof (buffer)));
+#else
+	lseek(fileno(infile), file_addr * blocksize, SEEK_SET);
+	read(fileno(infile), buffer, sizeof (buffer));
+#endif
+	c = dump_rr((struct iso_directory_record *) buffer);
+	if (c == 0 ||
+	    (c & (RR_FLAG_SP | RR_FLAG_ER)) == 0 || su_version < 1 || rr_version < 1) {
+		use_rock = FALSE;
+	}
+
 	if (sizeof (file_addr) > sizeof (long)) {
-		printf("Root at extent %llx, %d bytes\n", (Llong)file_addr, file_size);
+		printf(_("Root at extent %llx, %d bytes\n"), (Llong)file_addr, file_size);
 	} else {
-		printf("Root at extent %lx, %d bytes\n", (long)file_addr, file_size);
+		printf(_("Root at extent %lx, %d bytes\n"), (long)file_addr, file_size);
 	}
 	file_addr = file_addr * blocksize;
 
@@ -784,6 +862,6 @@ main(argc, argv)
 		fclose(infile);
 
 	if (!ngoof)
-		printf("No errors found\n");
+		printf(_("No errors found\n"));
 	return (0);
 }
