@@ -1,30 +1,32 @@
-/* @(#)hash.c	1.15 02/02/10 joerg */
+/* @(#)hash.c	1.28 10/12/19 joerg */
+#include <schily/mconfig.h>
 #ifndef lint
-static	char sccsid[] =
-	"@(#)hash.c	1.15 02/02/10 joerg";
+static	UConst char sccsid[] =
+	"@(#)hash.c	1.28 10/12/19 joerg";
 
 #endif
 /*
  * File hash.c - generate hash tables for iso9660 filesystem.
-
-   Written by Eric Youngdale (1993).
-
-   Copyright 1993 Yggdrasil Computing, Incorporated
-   Copyright (c) 1999,2000 J. Schilling
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+ *
+ * Written by Eric Youngdale (1993).
+ *
+ * Copyright 1993 Yggdrasil Computing, Incorporated
+ * Copyright (c) 1999,2000-2010 J. Schilling
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
 /* APPLE_HYB James Pearson j.pearson@ge.ucl.ac.uk 23/2/2000 */
 
@@ -45,16 +47,18 @@ static	char sccsid[] =
  * allow to disable the mkisofs inode cache.
  */
 
-#include <mconfig.h>
 #include "mkisofs.h"
-#include <schily.h>
+#include <schily/schily.h>
 
-#define NR_HASH 1024
+#define	NR_HASH	(16*1024)
 
-#define HASH_FN(DEV, INO) ((DEV + INO + (INO >> 2) + (INO << 8)) % NR_HASH)
+#define	HASH_FN(DEV, INO)	((DEV + INO  + (INO >> 8) + (INO << 16)) % NR_HASH)
 
-static struct file_hash *hash_table[NR_HASH] = {0,};
+static struct file_hash *hash_table[NR_HASH];
 
+#ifdef	HASH_DEBUG
+EXPORT	void		debug_hash	__PR((void));
+#endif
 EXPORT	void		add_hash	__PR((struct directory_entry *spnt));
 EXPORT	struct file_hash *find_hash	__PR((dev_t dev, ino_t inode));
 EXPORT	void		flush_hash	__PR((void));
@@ -63,9 +67,45 @@ EXPORT	struct file_hash *find_directory_hash __PR((dev_t dev, ino_t inode));
 LOCAL	unsigned int	name_hash	__PR((const char *name));
 EXPORT	void		add_file_hash	__PR((struct directory_entry *de));
 EXPORT	struct directory_entry *find_file_hash __PR((char *name));
+LOCAL	BOOL		isoname_endsok	__PR((char *name));
 EXPORT	int		delete_file_hash __PR((struct directory_entry *de));
 EXPORT	void		flush_file_hash	__PR((void));
 
+#ifdef	HASH_DEBUG
+EXPORT void
+debug_hash()
+{
+	struct file_hash **p = hash_table;
+	int	i;
+	int	j = 0;
+	struct file_hash *p2;
+	int	k;
+	int	maxlen = 0;
+	int	minlen = 100000000;
+	int	tot = 0;
+
+	for (i = 0; i < NR_HASH; i++, p++) {
+		if (*p == 0)
+			j++;
+		else {
+			p2 = *p;
+			k = 0;
+			while (p2) {
+				tot++;
+				k++;
+				p2 = p2->next;
+			}
+			if (k > maxlen)
+				maxlen = k;
+			if (k < minlen)
+				minlen = k;
+		}
+	}
+	error("Unbenutzt: %d von %d Eintrgen maxlen %d minlen %d total %d optmittel %d\n",
+		j, NR_HASH, maxlen, minlen, tot, tot/NR_HASH);
+}
+#endif
+
 EXPORT void
 add_hash(spnt)
 	struct directory_entry	*spnt;
@@ -74,23 +114,16 @@ add_hash(spnt)
 	unsigned int    hash_number;
 
 	if (spnt->size == 0 || spnt->starting_block == 0)
-		if (spnt->size != 0 || spnt->starting_block != 0) {
-#ifdef	USE_LIBSCHILY
+		if (spnt->size != 0 && spnt->starting_block == 0) {
 			comerrno(EX_BAD,
-			"Non zero-length file '%s' assigned zero extent.\n",
+			_("Non zero-length file '%s' assigned zero extent.\n"),
 							spnt->name);
-#else
-			fprintf(stderr,
-			"Non zero-length file '%s' assigned zero extent.\n",
-							spnt->name);
-			exit(1);
-#endif
 		};
 
 	if (!cache_inodes)
 		return;
-	if (spnt->dev == (dev_t) UNCACHED_DEVICE ||
-				spnt->inode == UNCACHED_INODE) {
+	if (spnt->dev == UNCACHED_DEVICE &&
+	    (spnt->inode == TABLE_INODE || spnt->inode == UNCACHED_INODE)) {
 		return;
 	}
 	hash_number = HASH_FN((unsigned int) spnt->dev,
@@ -100,10 +133,11 @@ add_hash(spnt)
 	if (verbose > 1)
 		fprintf(stderr, "%s ", spnt->name);
 #endif
-	s_hash = (struct file_hash *) e_malloc(sizeof(struct file_hash));
+	s_hash = (struct file_hash *)e_malloc(sizeof (struct file_hash));
 	s_hash->next = hash_table[hash_number];
 	s_hash->inode = spnt->inode;
 	s_hash->dev = spnt->dev;
+	s_hash->nlink = 0;
 	s_hash->starting_block = spnt->starting_block;
 	s_hash->size = spnt->size;
 #ifdef SORTING
@@ -126,18 +160,19 @@ find_hash(dev, inode)
 	struct file_hash *spnt;
 
 	if (!cache_inodes)
-		return NULL;
-	if (dev == (dev_t) UNCACHED_DEVICE || inode == UNCACHED_INODE)
-		return NULL;
+		return (NULL);
+	if (dev == UNCACHED_DEVICE &&
+	    (inode == TABLE_INODE || inode == UNCACHED_INODE))
+		return (NULL);
 
 	hash_number = HASH_FN((unsigned int) dev, (unsigned int) inode);
 	spnt = hash_table[hash_number];
 	while (spnt) {
 		if (spnt->inode == inode && spnt->dev == dev)
-			return spnt;
+			return (spnt);
 		spnt = spnt->next;
 	};
-	return NULL;
+	return (NULL);
 }
 
 /*
@@ -147,9 +182,9 @@ find_hash(dev, inode)
 EXPORT void
 flush_hash()
 {
-	struct file_hash *fh,
-	               *fh1;
-	int             i;
+	struct file_hash	*fh;
+	struct file_hash	*fh1;
+	int			i;
 
 	for (i = 0; i < NR_HASH; i++) {
 		fh = hash_table[i];
@@ -162,7 +197,7 @@ flush_hash()
 	}
 }
 
-static struct file_hash *directory_hash_table[NR_HASH] = {0,};
+static struct file_hash *directory_hash_table[NR_HASH];
 
 #ifdef	PROTOTYPES
 EXPORT void
@@ -179,15 +214,17 @@ add_directory_hash(dev, inode)
 
 	if (!cache_inodes)
 		return;
-	if (dev == (dev_t) UNCACHED_DEVICE || inode == UNCACHED_INODE)
+	if (dev == UNCACHED_DEVICE &&
+	    (inode == TABLE_INODE || inode == UNCACHED_INODE))
 		return;
 
 	hash_number = HASH_FN((unsigned int) dev, (unsigned int) inode);
 
-	s_hash = (struct file_hash *) e_malloc(sizeof(struct file_hash));
+	s_hash = (struct file_hash *)e_malloc(sizeof (struct file_hash));
 	s_hash->next = directory_hash_table[hash_number];
 	s_hash->inode = inode;
 	s_hash->dev = dev;
+	s_hash->nlink = 0;
 	directory_hash_table[hash_number] = s_hash;
 }
 
@@ -205,28 +242,30 @@ find_directory_hash(dev, inode)
 	struct file_hash *spnt;
 
 	if (!cache_inodes)
-		return NULL;
-	if (dev == (dev_t) UNCACHED_DEVICE || inode == UNCACHED_INODE)
-		return NULL;
+		return (NULL);
+	if (dev == UNCACHED_DEVICE &&
+	    (inode == TABLE_INODE || inode == UNCACHED_INODE))
+		return (NULL);
 
 	hash_number = HASH_FN((unsigned int) dev, (unsigned int) inode);
 	spnt = directory_hash_table[hash_number];
 	while (spnt) {
 		if (spnt->inode == inode && spnt->dev == dev)
-			return spnt;
+			return (spnt);
 		spnt = spnt->next;
 	};
-	return NULL;
+	return (NULL);
 }
 
 struct name_hash {
 	struct name_hash *next;
 	struct directory_entry *de;
+	int	sum;
 };
 
-#define NR_NAME_HASH 128
+#define	NR_NAME_HASH	128
 
-static struct name_hash *name_hash_table[NR_NAME_HASH] = {0,};
+static struct name_hash *name_hash_table[NR_NAME_HASH] = {0, };
 
 /*
  * Find the hash bucket for this name.
@@ -235,8 +274,8 @@ LOCAL unsigned int
 name_hash(name)
 	const char	*name;
 {
-	unsigned int    hash = 0;
-	const char     *p;
+	unsigned int	hash = 0;
+	const char	*p;
 
 	p = name;
 
@@ -249,21 +288,29 @@ name_hash(name)
 		if (*p == ';') {
 			break;
 		}
-		hash = (hash << 15) + (hash << 3) + (hash >> 3) + *p++;
+		hash = (hash << 15) + (hash << 3) + (hash >> 3) + (*p++ & 0xFF);
 	}
-	return hash % NR_NAME_HASH;
+	return (hash % NR_NAME_HASH);
 }
 
 EXPORT void
 add_file_hash(de)
 	struct directory_entry	*de;
 {
-	struct name_hash *new;
-	int             hash;
+	struct name_hash	*new;
+	int			hash;
+	Uchar			*p;
+	int			sum = 0;
 
-	new = (struct name_hash *) e_malloc(sizeof(struct name_hash));
+	new = (struct name_hash *)e_malloc(sizeof (struct name_hash));
 	new->de = de;
 	new->next = NULL;
+	for (p = (Uchar *)de->isorec.name; *p; p++) {
+		if (*p == ';')
+			break;
+		sum += *p & 0xFF;
+	}
+	new->sum = sum;
 	hash = name_hash(de->isorec.name);
 
 	/* Now insert into the hash table */
@@ -273,45 +320,109 @@ add_file_hash(de)
 
 EXPORT struct directory_entry *
 find_file_hash(name)
-	char	*name;
+	register char			*name;
 {
-	struct name_hash *nh;
-	char           *p1;
-	char           *p2;
+	register char			*p1;
+	register char			*p2;
+	register struct name_hash	*nh;
+	register int			sum = 0;
+
+	if (debug > 1)
+		error("find_hash('%s')\n", name);
+
+	for (p1 = name; *p1; p1++) {
+		if (*p1 == ';')
+			break;
+		sum += *p1 & 0xFF;
+	}
 
 	for (nh = name_hash_table[name_hash(name)]; nh; nh = nh->next) {
+		if (nh->sum != sum)
+			continue;
+
 		p1 = name;
 		p2 = nh->de->isorec.name;
+		if (debug > 1)
+			error(_("Checking name '%s' isorec.name '%s'\n"), p1, p2);
 
 		/* Look for end of string, or a mismatch. */
 		while (1 == 1) {
-			if ((*p1 == '\0' || *p1 == ';')
-				|| (*p2 == '\0' || *p2 == ';')
-				|| (*p1 != *p2)) {
+			if ((*p1 == '\0' || *p1 == ';') ||
+			    (*p2 == '\0' || *p2 == ';') ||
+			    (*p1 != *p2)) {
 				break;
 			}
 			p1++;
 			p2++;
 		}
+		if (!isoname_endsok(p1) || !isoname_endsok(p2)) {
+			if (debug > 1) {
+				if (!isoname_endsok(p1))
+					error(_("'%s' does NOT END OK\n"), p1);
+				if (!isoname_endsok(p2))
+					error(_("'%s' does NOT END OK\n"), p2);
+			}
+			/*
+			 * If one file does not end with a valid version number
+			 * and the other name ends here, we found a miss match.
+			 */
+			if (*p1 == '\0' || *p2 == '\0')
+				continue;
+
+			if (*p1 == ';' && *p2 == ';') {
+				p1++;
+				p2++;
+				continue;
+			}
+		}
 
 		/*
 		 * If we are at the end of both strings, then we have a match.
 		 */
-		if ((*p1 == '\0' || *p1 == ';')
-			&& (*p2 == '\0' || *p2 == ';')) {
-			return nh->de;
+		if ((*p1 == '\0' || *p1 == ';') &&
+		    (*p2 == '\0' || *p2 == ';')) {
+			return (nh->de);
 		}
 	}
-	return NULL;
+	return (NULL);
+}
+
+/*
+ * The macro 'eo' is just an idea on how one might speed up isoname_endsok()
+ */
+#define	eo(p)	(((p)[0] == '\0') || \
+		((p)[0] == ';' && (p)[1] == '1' && (p)[2] == '\0') || \
+		isoname_endsok(p))
+
+LOCAL BOOL
+isoname_endsok(name)
+	char	*name;
+{
+	int	i;
+	char	*p;
+
+	if (*name == '\0')
+		return (TRUE);
+	if (*name != ';')
+		return (FALSE);
+
+	for (p = ++name, i = 0; *p && i < 5; p++, i++) {
+		if (*p < '0' || *p > '9')
+			return (FALSE);
+	}
+	i = atoi(name);
+	if (i < 1 || i > 32767)
+		return (FALSE);
+	return (TRUE);
 }
 
 EXPORT int
 delete_file_hash(de)
 	struct directory_entry	*de;
 {
-	struct name_hash *nh,
-	               *prev;
-	int             hash;
+	struct name_hash	*nh;
+	struct name_hash	*prev;
+	int			hash;
 
 	prev = NULL;
 	hash = name_hash(de->isorec.name);
@@ -321,21 +432,21 @@ delete_file_hash(de)
 		prev = nh;
 	}
 	if (!nh)
-		return 1;
+		return (1);
 	if (!prev)
 		name_hash_table[hash] = nh->next;
 	else
 		prev->next = nh->next;
 	free(nh);
-	return 0;
+	return (0);
 }
 
 EXPORT void
 flush_file_hash()
 {
-	struct name_hash *nh,
-	               *nh1;
-	int             i;
+	struct name_hash	*nh;
+	struct name_hash	*nh1;
+	int			i;
 
 	for (i = 0; i < NR_NAME_HASH; i++) {
 		nh = name_hash_table[i];
