@@ -1,11 +1,11 @@
-/* @(#)scgcheck.c	1.19 10/05/24 Copyright 1998-2010 J. Schilling */
+/* @(#)scgcheck.c	1.21 11/08/03 Copyright 1998-2011 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)scgcheck.c	1.19 10/05/24 Copyright 1998-2010 J. Schilling";
+	"@(#)scgcheck.c	1.21 11/08/03 Copyright 1998-2011 J. Schilling";
 #endif
 /*
- *	Copyright (c) 1998-2010 J. Schilling
+ *	Copyright (c) 1998-2011 J. Schilling
  *
  * Warning: This program has been written to verify the correctness
  * of the upper layer interface from the library "libscg". If you
@@ -29,6 +29,7 @@ static	UConst char sccsid[] =
 #include <schily/unistd.h>
 #include <schily/string.h>
 #include <schily/schily.h>
+#include <schily/nlsdefs.h>
 #include <schily/standard.h>
 
 #include <schily/utypes.h>
@@ -81,18 +82,18 @@ LOCAL void
 usage(ret)
 	int	ret;
 {
-	error("Usage:\tscgcheck [options]\n");
-	error("Options:\n");
-	error("\t-version	print version information and exit\n");
-	error("\tdev=target	SCSI target to use\n");
-	error("\ttimeout=#	set the default SCSI command timeout to #.\n");
-	error("\tdebug=#,-d	Set to # or increment misc debug level\n");
-	error("\tkdebug=#,kd=#	do Kernel debugging\n");
-	error("\t-verbose,-v	increment general verbose level by one\n");
-	error("\t-Verbose,-V	increment SCSI command transport verbose level by one\n");
-	error("\t-silent,-s	do not print status of failed SCSI commands\n");
-	error("\tf=filename	Name of file to write log data to.\n");
-	error("\t-auto		try to do a fully automated test\n");
+	error(_("Usage:\tscgcheck [options]\n"));
+	error(_("Options:\n"));
+	error(_("\t-version	print version information and exit\n"));
+	error(_("\tdev=target	SCSI target to use\n"));
+	error(_("\ttimeout=#	set the default SCSI command timeout to #.\n"));
+	error(_("\tdebug=#,-d	Set to # or increment misc debug level\n"));
+	error(_("\tkdebug=#,kd=#	do Kernel debugging\n"));
+	error(_("\t-verbose,-v	increment general verbose level by one\n"));
+	error(_("\t-Verbose,-V	increment SCSI command transport verbose level by one\n"));
+	error(_("\t-silent,-s	do not print status of failed SCSI commands\n"));
+	error(_("\tf=filename	Name of file to write log data to.\n"));
+	error(_("\t-auto		try to do a fully automated test\n"));
 	error("\n");
 	exit(ret);
 }
@@ -114,9 +115,30 @@ main(ac, av)
 	BOOL	help = FALSE;
 	BOOL	pversion = FALSE;
 	char	*filename = "check.log";
+#if	defined(USE_NLS)
+	char	*dir;
+#endif
 
 	save_args(ac, av);
 
+#if	defined(USE_NLS)
+	(void) setlocale(LC_ALL, "");
+#if !defined(TEXT_DOMAIN)	/* Should be defined by cc -D */
+#define	TEXT_DOMAIN "scgcheck"	/* Use this only if it weren't */
+#endif
+	dir = searchfileinpath("share/locale", F_OK,
+					SIP_ANY_FILE|SIP_NO_PATH, NULL);
+	if (dir)
+		(void) bindtextdomain(TEXT_DOMAIN, dir);
+	else
+#if defined(PROTOTYPES) && defined(INS_BASE)
+	(void) bindtextdomain(TEXT_DOMAIN, INS_BASE "/share/locale");
+#else
+	(void) bindtextdomain(TEXT_DOMAIN, "/usr/share/locale");
+#endif
+	(void) textdomain(TEXT_DOMAIN);
+#endif
+
 	cac = --ac;
 	cav = ++av;
 
@@ -131,15 +153,16 @@ main(ac, av)
 			&help, &help, &pversion,
 			&dev,
 			&filename, &autotest) < 0) {
-		errmsgno(EX_BAD, "Bad flag: %s.\n", cav[0]);
+		errmsgno(EX_BAD, _("Bad flag: %s.\n"), cav[0]);
 		usage(EX_BAD);
 	}
 	if (help)
 		usage(0);
 	if (pversion) {
-		printf("scgcheck %s (%s-%s-%s) Copyright (C) 1998-2010 Jrg Schilling\n",
+		printf(_("scgcheck %s (%s-%s-%s) Copyright (C) 1998-2011 %s\n"),
 								scgc_version,
-								HOST_CPU, HOST_VENDOR, HOST_OS);
+								HOST_CPU, HOST_VENDOR, HOST_OS,
+								_("Joerg Schilling"));
 		exit(0);
 	}
 
@@ -153,17 +176,18 @@ main(ac, av)
 		cav++;
 	}
 	if (fcount > 0)
-		comerrno(EX_BAD, "Bad argument(s).\n");
+		comerrno(EX_BAD, _("Bad argument(s).\n"));
 /*error("dev: '%s'\n", dev);*/
 
 	logfile = fileopen(filename, "wct");
 	if (logfile == NULL)
-		comerr("Cannot open logfile.\n");
+		comerr(_("Cannot open logfile.\n"));
 
-	chkprint("Scgcheck %s (%s-%s-%s) SCSI user level transport library ABI checker.\n\
-Copyright (C) 1998-2008 Jrg Schilling\n",
+	chkprint(_("Scgcheck %s (%s-%s-%s) SCSI user level transport library ABI checker.\n\
+Copyright (C) 1998-2011 %s\n"),
 						scgc_version,
-						HOST_CPU, HOST_VENDOR, HOST_OS);
+						HOST_CPU, HOST_VENDOR, HOST_OS,
+						_("Joerg Schilling"));
 	/*
 	 * Call scg_remote() to force loading the remote SCSI transport library
 	 * code that is located in librscg instead of the dummy remote routines
@@ -173,29 +197,29 @@ Copyright (C) 1998-2008 Jrg Schilling\n",
 
 	basefds = countopen();
 	if (xdebug)
-		error("nopen: %d\n", basefds);
+		error(_("nopen: %d\n"), basefds);
 
-	chkprint("**********> Checking whether your implementation supports to scan the SCSI bus.\n");
-	chkprint("Trying to open device: '%s'.\n", dev);
+	chkprint(_("**********> Checking whether your implementation supports to scan the SCSI bus.\n"));
+	chkprint(_("Trying to open device: '%s'.\n"), dev);
 	scgp = doopen(dev);
 
 	if (xdebug) {
-		error("nopen: %d\n", countopen());
-		error("Scanopen opened %d new files.\n", countopen() - basefds);
+		error(_("nopen: %d\n"), countopen());
+		error(_("Scanopen opened %d new files.\n"), countopen() - basefds);
 	}
 
 	device[0] = '\0';
 	if (scgp == NULL) do {
-		error("SCSI open failed...\n");
-		if (!scg_yes("Retry with different device name? "))
+		error(_("SCSI open failed...\n"));
+		if (!scg_yes(_("Retry with different device name? ")))
 			break;
-		error("Enter SCSI device name for bus scanning [%s]: ", device);
+		error(_("Enter SCSI device name for bus scanning [%s]: "), device);
 		flushit();
 		(void) getline(device, sizeof (device));
 		if (device[0] == '\0')
 			strcpy(device, "0,6,0");
 
-		chkprint("Trying to open device: '%s'.\n", device);
+		chkprint(_("Trying to open device: '%s'.\n"), device);
 		scgp = doopen(device);
 	} while (scgp == NULL);
 	if (scgp) {
@@ -205,18 +229,18 @@ Copyright (C) 1998-2008 Jrg Schilling\n",
 		ret = select_target(scgp, stdout);
 		select_target(scgp, logfile);
 		if (ret < 1) {
-			chkprint("----------> SCSI scan bus test: found NO TARGETS\n");
+			chkprint(_("----------> SCSI scan bus test: found NO TARGETS\n"));
 		} else {
-			chkprint("----------> SCSI scan bus test PASSED\n");
+			chkprint(_("----------> SCSI scan bus test PASSED\n"));
 		}
 	} else {
-		chkprint("----------> SCSI scan bus test FAILED\n");
+		chkprint(_("----------> SCSI scan bus test FAILED\n"));
 	}
 
 	if (xdebug)
-		error("nopen: %d\n", countopen());
-	chkprint("For the next test we need to open a single SCSI device.\n");
-	chkprint("Best results will be obtained if you specify a modern CD-ROM drive.\n");
+		error(_("nopen: %d\n"), countopen());
+	chkprint(_("For the next test we need to open a single SCSI device.\n"));
+	chkprint(_("Best results will be obtained if you specify a modern CD-ROM drive.\n"));
 
 	if (scgp) {		/* Scanbus works / may work */
 		int	i;
@@ -239,12 +263,12 @@ Copyright (C) 1998-2008 Jrg Schilling\n",
 	do {
 		char	Device[128];
 
-		error("Enter SCSI device name [%s]: ", device);
+		error(_("Enter SCSI device name [%s]: "), device);
 		(void) chkgetline(Device, sizeof (Device));
 		if (Device[0] != '\0')
 			strcpy(device, Device);
 
-		chkprint("Trying to open device: '%s'.\n", device);
+		chkprint(_("Trying to open device: '%s'.\n"), device);
 		scgp = doopen(device);
 		if (scgp) {
 			checkversion(scgp);
@@ -255,7 +279,7 @@ Copyright (C) 1998-2008 Jrg Schilling\n",
 		 */
 	} while (scgp == NULL);
 	if (xdebug)
-		error("nopen: %d\n", countopen());
+		error(_("nopen: %d\n"), countopen());
 	/*
 	 * First try to check which type of SCSI device we
 	 * have.
@@ -266,7 +290,7 @@ Copyright (C) 1998-2008 Jrg Schilling\n",
 	getdev(scgp, TRUE);
 	printinq(scgp, logfile);
 
-	printf("Ready to start test for second SCSI open? Enter <CR> to continue: ");
+	printf(_("Ready to start test for second SCSI open? Enter <CR> to continue: "));
 	(void) chkgetline(abuf, sizeof (abuf));
 #define	CHECK_SECOND_OPEN
 #ifdef	CHECK_SECOND_OPEN
@@ -279,51 +303,51 @@ Copyright (C) 1998-2008 Jrg Schilling\n",
 		ret = inquiry(scgp, buf, sizeof (struct scsi_inquiry));
 		scgp->silent--;
 		if (xdebug)
-			error("ret: %d key: %d\n", ret, scg_sense_key(scgp));
+			error(_("ret: %d key: %d\n"), ret, scg_sense_key(scgp));
 		if (ret >= 0 || scgp->scmd->error == SCG_RETRYABLE) {
-			chkprint("First SCSI open OK - device usable\n");
-			chkprint("**********> Checking for second SCSI open.\n");
+			chkprint(_("First SCSI open OK - device usable\n"));
+			chkprint(_("**********> Checking for second SCSI open.\n"));
 			if ((scgp2 = doopen(device)) != NULL) {
 				/*
 				 * XXX Separates getbuf() fuer scgp2?
 				 */
-				chkprint("Second SCSI open for same device succeeded, %d additional file descriptor(s) used.\n",
+				chkprint(_("Second SCSI open for same device succeeded, %d additional file descriptor(s) used.\n"),
 					countopen() - oldopen);
 				scgp->silent++;
 				ret = inquiry(scgp, buf, sizeof (struct scsi_inquiry));
 				scgp->silent--;
 				if (ret >= 0 || scgp->scmd->error == SCG_RETRYABLE) {
-					chkprint("Second SCSI open is usable\n");
+					chkprint(_("Second SCSI open is usable\n"));
 				}
-				chkprint("Closing second SCSI.\n");
+				chkprint(_("Closing second SCSI.\n"));
 				scg_close(scgp2);
 				scgp2 = NULL;
-				chkprint("Checking first SCSI.\n");
+				chkprint(_("Checking first SCSI.\n"));
 				scgp->silent++;
 				ret = inquiry(scgp, buf, sizeof (struct scsi_inquiry));
 				scgp->silent--;
 				if (ret >= 0 || scgp->scmd->error == SCG_RETRYABLE) {
 					second_ok = TRUE;
-					chkprint("First SCSI open is still usable\n");
-					chkprint("----------> Second SCSI open test PASSED.\n");
+					chkprint(_("First SCSI open is still usable\n"));
+					chkprint(_("----------> Second SCSI open test PASSED.\n"));
 				} else if (ret < 0 && scgp->scmd->error == SCG_FATAL) {
 					second_ok = FALSE;
-					chkprint("First SCSI open does not work anymore.\n");
-					chkprint("----------> Second SCSI open test FAILED.\n");
+					chkprint(_("First SCSI open does not work anymore.\n"));
+					chkprint(_("----------> Second SCSI open test FAILED.\n"));
 				} else {
 					second_ok = FALSE;
-					chkprint("First SCSI open has strange problems.\n");
-					chkprint("----------> Second SCSI open test FAILED.\n");
+					chkprint(_("First SCSI open has strange problems.\n"));
+					chkprint(_("----------> Second SCSI open test FAILED.\n"));
 				}
 			} else {
 				second_ok = FALSE;
-				chkprint("Cannot open same SCSI device a second time.\n");
-				chkprint("----------> Second SCSI open test FAILED.\n");
+				chkprint(_("Cannot open same SCSI device a second time.\n"));
+				chkprint(_("----------> Second SCSI open test FAILED.\n"));
 			}
 		} else {
 			second_ok = FALSE;
-			chkprint("First SCSI open is not usable\n");
-			chkprint("----------> Second SCSI open test FAILED.\n");
+			chkprint(_("First SCSI open is not usable\n"));
+			chkprint(_("----------> Second SCSI open test FAILED.\n"));
 		}
 		if (xdebug > 1)
 			error("scgp %p scgp2 %p\n", scgp, scgp2);
@@ -334,7 +358,7 @@ Copyright (C) 1998-2008 Jrg Schilling\n",
 			ret = inquiry(scgp, buf, sizeof (struct scsi_inquiry));
 			scgp->silent--;
 			if (ret >= 0 || scgp->scmd->error == SCG_RETRYABLE) {
-				chkprint("First SCSI open is still usable\n");
+				chkprint(_("First SCSI open is still usable\n"));
 			} else {
 				scg_freebuf(scgp);
 				scg_close(scgp);
@@ -347,26 +371,26 @@ Copyright (C) 1998-2008 Jrg Schilling\n",
 	}
 #endif	/* CHECK_SECOND_OPEN */
 
-	printf("Ready to start test for succeeded command? Enter <CR> to continue: ");
+	printf(_("Ready to start test for succeeded command? Enter <CR> to continue: "));
 	(void) chkgetline(abuf, sizeof (abuf));
-	chkprint("**********> Checking for succeeded SCSI command.\n");
+	chkprint(_("**********> Checking for succeeded SCSI command.\n"));
 	scgp->verbose++;
 	ret = inquiry(scgp, buf, sizeof (struct scsi_inquiry));
 	scg_vsetup(scgp);
 	scg_errfflush(scgp, logfile);
 	scgp->verbose--;
-	scg_fprbytes(logfile, "Inquiry Data   :", (Uchar *)buf,
+	scg_fprbytes(logfile, _("Inquiry Data   :"), (Uchar *)buf,
 			sizeof (struct scsi_inquiry) - scg_getresid(scgp));
 
 	if (ret >= 0 && !scg_cmd_err(scgp)) {
-		chkprint("----------> SCSI succeeded command test PASSED\n");
+		chkprint(_("----------> SCSI succeeded command test PASSED\n"));
 	} else {
-		chkprint("----------> SCSI succeeded command test FAILED\n");
+		chkprint(_("----------> SCSI succeeded command test FAILED\n"));
 	}
 
 	sensetest(scgp);
 	if (!autotest)
-		chkprint("----------> SCSI status byte test NOT YET READY\n");
+		chkprint(_("----------> SCSI status byte test NOT YET READY\n"));
 /*
  * scan OK
  * work OK
@@ -381,7 +405,7 @@ Copyright (C) 1998-2008 Jrg Schilling\n",
  */
 
 	dmaresid(scgp);
-	chkprint("----------> SCSI transport code test NOT YET READY\n");
+	chkprint(_("----------> SCSI transport code test NOT YET READY\n"));
 	return (0);
 }
 
@@ -393,10 +417,10 @@ doopen(sdevname)
 	char	errstr[128];
 
 	if ((scgp = scg_open(sdevname, errstr, sizeof (errstr), debug, lverbose)) == (SCSI *)0) {
-		errmsg("%s%sCannot open SCSI driver.\n", errstr, errstr[0]?". ":"");
-		fprintf(logfile, "%s. %s%sCannot open SCSI driver.\n",
+		errmsg(_("%s%sCannot open SCSI driver.\n"), errstr, errstr[0]?". ":"");
+		fprintf(logfile, _("%s. %s%sCannot open SCSI driver.\n"),
 			errmsgstr(geterrno()), errstr, errstr[0]?". ":"");
-		errmsgno(EX_BAD, "For possible targets try 'cdrecord -scanbus'. Make sure you are root.\n");
+		errmsgno(EX_BAD, _("For possible targets try 'cdrecord -scanbus'. Make sure you are root.\n"));
 		return (scgp);
 	}
 	scg_settimeout(scgp, deftimeout);
@@ -422,43 +446,43 @@ checkversion(scgp)
 	 */
 	vers = scg_version(0, SCG_VERSION);
 	auth = scg_version(0, SCG_AUTHOR);
-	chkprint("Using libscg version '%s-%s'\n", auth, vers);
+	chkprint(_("Using libscg version '%s-%s'\n"), auth, vers);
 	if (auth == 0 || strcmp("schily", auth) != 0) {
 		errmsgno(EX_BAD,
-		"Warning: using inofficial version of libscg (%s-%s '%s').\n",
+		_("Warning: using inofficial version of libscg (%s-%s '%s').\n"),
 			auth, vers, scg_version(0, SCG_SCCS_ID));
 	}
 
 	vers = scg_version(scgp, SCG_VERSION);
 	auth = scg_version(scgp, SCG_AUTHOR);
 	if (lverbose > 1)
-		error("Using libscg transport code version '%s-%s'\n", auth, vers);
-	fprintf(logfile, "Using libscg transport code version '%s-%s'\n", auth, vers);
+		error(_("Using libscg transport code version '%s-%s'\n"), auth, vers);
+	fprintf(logfile, _("Using libscg transport code version '%s-%s'\n"), auth, vers);
 	if (auth == 0 || strcmp("schily", auth) != 0) {
 		errmsgno(EX_BAD,
-		"Warning: using inofficial libscg transport code version (%s-%s '%s').\n",
+		_("Warning: using inofficial libscg transport code version (%s-%s '%s').\n"),
 			auth, vers, scg_version(scgp, SCG_SCCS_ID));
 	}
 	vers = scg_version(scgp, SCG_KVERSION);
 	if (vers == NULL)
 		vers = unavail;
-	fprintf(logfile, "Using kernel transport code version '%s'\n", vers);
+	fprintf(logfile, _("Using kernel transport code version '%s'\n"), vers);
 
 	vers = scg_version(scgp, SCG_RVERSION);
 	auth = scg_version(scgp, SCG_RAUTHOR);
 	if (lverbose > 1 && vers && auth)
-		error("Using remote transport code version '%s-%s'\n", auth, vers);
+		error(_("Using remote transport code version '%s-%s'\n"), auth, vers);
 
 	if (auth != 0 && strcmp("schily", auth) != 0) {
 		errmsgno(EX_BAD,
-		"Warning: using inofficial remote transport code version (%s-%s '%s').\n",
+		_("Warning: using inofficial remote transport code version (%s-%s '%s').\n"),
 			auth, vers, scg_version(scgp, SCG_RSCCS_ID));
 	}
 	if (auth == NULL)
 		auth = unavail;
 	if (vers == NULL)
 		vers = unavail;
-	fprintf(logfile, "Using remote transport code version '%s-%s'\n", auth, vers);
+	fprintf(logfile, _("Using remote transport code version '%s-%s'\n"), auth, vers);
 }
 
 LOCAL void
@@ -466,11 +490,11 @@ getbuf(scgp)
 	SCSI	*scgp;
 {
 	bufsize = scg_bufsize(scgp, MAX_BUF_SIZE);
-	chkprint("Max DMA buffer size: %ld\n", bufsize);
+	chkprint(_("Max DMA buffer size: %ld\n"), bufsize);
 	seterrno(0);
 	if ((buf = scg_getbuf(scgp, bufsize)) == NULL) {
-		errmsg("Cannot get SCSI buffer (%ld bytes).\n", bufsize);
-		fprintf(logfile, "%s. Cannot get SCSI buffer (%ld bytes).\n",
+		errmsg(_("Cannot get SCSI buffer (%ld bytes).\n"), bufsize);
+		fprintf(logfile, _("%s. Cannot get SCSI buffer (%ld bytes).\n"),
 			errmsgstr(geterrno()), bufsize);
 	} else {
 		scg_freebuf(scgp);
@@ -478,19 +502,19 @@ getbuf(scgp)
 
 	bufsize = scg_bufsize(scgp, BUF_SIZE);
 	if (debug)
-		error("SCSI buffer size: %ld\n", bufsize);
+		error(_("SCSI buffer size: %ld\n"), bufsize);
 	if ((buf = scg_getbuf(scgp, bufsize)) == NULL)
-		comerr("Cannot get SCSI I/O buffer.\n");
+		comerr(_("Cannot get SCSI I/O buffer.\n"));
 }
 
 LOCAL void
 scg_openerr(errstr)
 	char	*errstr;
 {
-	errmsg("%s%sCannot open or use SCSI driver.\n", errstr, errstr[0]?". ":"");
-	errmsgno(EX_BAD, "For possible targets try 'cdrecord -scanbus'.%s\n",
-				geteuid() ? " Make sure you are root.":"");
-	errmsgno(EX_BAD, "For possible transport specifiers try 'cdrecord dev=help'.\n");
+	errmsg(_("%s%sCannot open or use SCSI driver.\n"), errstr, errstr[0]?". ":"");
+	errmsgno(EX_BAD, _("For possible targets try 'cdrecord -scanbus'.%s\n"),
+				geteuid() ? _(" Make sure you are root."):"");
+	errmsgno(EX_BAD, _("For possible transport specifiers try 'cdrecord dev=help'.\n"));
 	exit(EX_BAD);
 }
 
@@ -504,7 +528,7 @@ find_drive(scgp, sdevname, flags)
 	int	type = INQ_ROMD;
 
 	if ((flags & F_MSINFO) == 0)
-		error("No target specified, trying to find one...\n");
+		error(_("No target specified, trying to find one...\n"));
 	ntarget = find_target(scgp, type, -1);
 	if (ntarget < 0)
 		return (ntarget);
@@ -522,21 +546,21 @@ find_drive(scgp, sdevname, flags)
 		/*
 		 * No single CD-ROM or WORM found.
 		 */
-		errmsgno(EX_BAD, "No CD/DVD/BD-Recorder target found.\n");
-		errmsgno(EX_BAD, "Your platform may not allow to scan for SCSI devices.\n");
-		comerrno(EX_BAD, "Call 'cdrecord dev=help' or ask your sysadmin for possible targets.\n");
+		errmsgno(EX_BAD, _("No CD/DVD/BD-Recorder target found.\n"));
+		errmsgno(EX_BAD, _("Your platform may not allow to scan for SCSI devices.\n"));
+		comerrno(EX_BAD, _("Call 'cdrecord dev=help' or ask your sysadmin for possible targets.\n"));
 	} else {
-		errmsgno(EX_BAD, "Too many CD/DVD/BD-Recorder targets found.\n");
+		errmsgno(EX_BAD, _("Too many CD/DVD/BD-Recorder targets found.\n"));
 #ifdef	nonono
 		select_target(scgp, stdout);
-		comerrno(EX_BAD, "Select a target from the list above and use 'cdrecord dev=%s%sb,t,l'.\n",
+		comerrno(EX_BAD, _("Select a target from the list above and use 'cdrecord dev=%s%sb,t,l'.\n"),
 			sdevname?sdevname:"",
 			sdevname?(sdevname[strlen(sdevname)-1] == ':'?"":":"):"");
 #endif	/* nonono */
 		find_target(scgp, type, 1);
 	}
 	if ((flags & F_MSINFO) == 0)
-		error("Using dev=%s%s%d,%d,%d.\n",
+		error(_("Using dev=%s%s%d,%d,%d.\n"),
 			sdevname?sdevname:"",
 			sdevname?(sdevname[strlen(sdevname)-1] == ':'?"":":"):"",
 			scg_scsibus(scgp), scg_target(scgp), scg_lun(scgp));
@@ -561,10 +585,12 @@ countopen()
 	int	nopen = 0;
 	int	i;
 
+#ifdef	F_GETFD				/* XXX how to count open fd on MSC */
 	for (i = 0; i < 1000; i++) {
 		if (fcntl(i, F_GETFD, 0) >= 0)
 			nopen++;
 	}
+#endif
 	return (nopen);
 }
 /*--------------------------------------------------------------------------*/
